local Root = BossEncounter3;

local Authorization = Root:GetOrNewModule("Authorization");

local Unit = Root:GetOrNewModule("Unit");

-- --------------------------------------------------------------------
-- **                             Data                               **
-- --------------------------------------------------------------------

Authorization.last = nil;
Authorization.nextCheck = 0;

local actionChecks = {
    ["RAID_WARNING"] = function(unit)
        local rank = Authorization:GetRank(unit);
        return (rank == "LEADER" or rank == "ASSIST");
    end,

    ["SYMBOL"] = function(unit)
        if IsInRaid() then
            local rank = Authorization:GetRank(unit);
            return (rank == "LEADER" or rank == "ASSIST");
      else
            return true;
        end
    end,

    ["KICK"] = function(unit, target)
        local mode = GetLFGMode();
        local inLFG = (mode == "lfgparty");

        if not inLFG then
            local rank = Authorization:GetRank(unit);

            if not target then
                return (rank == "LEADER");
          else
                local targetRank = Authorization:GetRank(target);
                return Authorization:IsSuperiorRank(rank, targetRank);
            end
      else
            return false;
        end
    end,

    ["READY_CHECK"] = function(unit)
        local rank = Authorization:GetRank(unit);
        return (rank == "LEADER" or rank == "ASSIST");
    end,

    ["SET_RAID_RANK"] = function(unit, target)
        if IsInRaid() then
            local rank = Authorization:GetRank(unit);

            if not target then
                return (rank == "LEADER");
          else
                local targetRank = Authorization:GetRank(target);
                return Authorization:IsSuperiorRank(rank, targetRank);
            end
      else
            return false;
        end
    end,
};

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> unit: can be an UID, name or GUID of someone in your raid.    *
-- * It defaults to "player".                                         *
-- ********************************************************************
-- * Get the rank of the given unit in the raid or party.             *
-- * Can be LEADER, ASSIST or NORMAL.                                 *
-- ********************************************************************
function Authorization:GetRank(unit)
    unit = Unit:ResolveUID(unit or "player");
    if unit then
		if UnitIsGroupLeader(unit) then return "LEADER"; end
	    if UnitIsGroupAssistant(unit) then return "ASSIST"; end
    end
    return "NORMAL";
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> rank: is this rank...                                         *
-- * >> relativeTo: ...superior to this one?                          *
-- ********************************************************************
-- * Return true if rank is superior to relativeTo rank.              *
-- ********************************************************************
function Authorization:IsSuperiorRank(rank, relativeTo)
    if rank == "LEADER" then
        return true;

elseif rank == "ASSIST" and relativeTo == "NORMAL" then
        return true;
    end
    return false;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> action: the action name to check authorization for.           *
-- * Possible actions:                                                *
-- * RAID_WARNING, SYMBOL, KICK, READY_CHECK, SET_RAID_RANK.          *
-- * ---------------------------------------------------------------- *
-- * >> unit: who is checked. Can be an UID, name or GUID of someone  *
-- * in the raid. It defaults to "player".                            *
-- * ---------------------------------------------------------------- *
-- * >> target: target of the action, if the action requires one.     *
-- ********************************************************************
-- * Returns whether you have the authorization or not to do a given  *
-- * operation on the raid or the party.                              *
-- ********************************************************************
function Authorization:Check(action, unit, target)
    unit = Unit:ResolveUID(unit or "player");
    if unit then
        local f = actionChecks[action];
        if f then
            return f(unit, target);
        end
    end
    return false;
end

-- ********************************************************************
-- * Check our rank hasn't changed in the meantime.                   *
-- ********************************************************************
function Authorization:CheckChange()
    local rank = self:GetRank("player");
    if self.last ~= rank then
        if self.last then
            Root:InvokeHandler("OnRankChange", rank);
        end
        self.last = rank;
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function Authorization:OnUpdate()
    local t = GetTime();
    if t > self.nextCheck then
        self.nextCheck = t + 2;
        self:CheckChange();
    end
end