﻿local Root = BossEncounter3;

local NPCInfo = Root:GetOrNewModule("NPCInfo");

-- --------------------------------------------------------------------
-- **                             Data                               **
-- --------------------------------------------------------------------

-- Specific modules have to do a call to the Register method of the
-- NPCInfo module to register their boss NPC in the NPC database, so
-- the system can know which module to enable.

local db = { };

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> mobID: the mob ID (not GUID!).                                *
-- ********************************************************************
-- * Grab infos relevant to the given mob ID.                         *
-- *                                                                  *
-- * Return moduleName, generated, volatile, bypass.                  *
-- ********************************************************************
function NPCInfo:Get(mobID)
    if mobID then
        local info = db[mobID];
        if info then
            return info.module, info.volatile or false, info.bypass or false;
        end
    end
    return nil, false, false;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> mobID: the mob ID (not GUID!).                                *
-- * >> module: the module name supposed to handle the encounter.     *
-- * >> volatile: if true, module will be fired as soon as someone    *
-- *              targets the NPC.                                    *
-- * >> bypass: sets whether checks on the NPC are bypassed or not.   *
-- *            These checks include the hostility of the enemy.      *
-- ********************************************************************
-- * Register from an external way (such as within a boss module) a   *
-- * NPC ID to monitor. The associated module will be run whenever    *
-- * the local player targets a NPC with this ID.                     *
-- ********************************************************************
function NPCInfo:Register(mobID, module, volatile, bypass)
    if type(mobID) == "table" then
        local id;
        for _, id in ipairs(mobID) do
            self:Register(id, module, volatile, bypass);
        end
  else
        if not db[mobID] then
            db[mobID] = {
                module = module,
                volatile = volatile,
                bypass = bypass,
            };
        end
    end
end