﻿local Root = BossEncounter3;

local ZoneInfo = Root:GetOrNewModule("ZoneInfo");

-- --------------------------------------------------------------------
-- **                             Data                               **
-- --------------------------------------------------------------------

-- Specific modules have to do a call to the Register method of the
-- ZoneInfo module to register their enable zone in the zone database,
-- so the system can know which module to enable upon entering a zone.

local db = { };

local EMPTY_TABLE = { };

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> zoneName: the name of the zone.                               *
-- ********************************************************************
-- * Grab the list of modules that can handle the specified zone.     *
-- ********************************************************************
function ZoneInfo:Get(zoneName)
    return (zoneName and db[zoneName]) or EMPTY_TABLE;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> zoneName: the name of the zone.                               *
-- * >> module: the module name supposed to handle the encounter.     *
-- * >> requireBossFilter: true if the boss filter must be respected  *
-- *                       for the module to be enabled.              *
-- ********************************************************************
-- * Register from an external way (such as within a boss module) a   *
-- * zone to monitor. The associated module will be run whenever      *
-- * the local player enters the zone.                                *
-- * Multiple modules can be bound to a single zone.                  *
-- * In this situation, the appropriate module is selected according  *
-- * to the state of the player's raid lockout.                       *
-- ********************************************************************
function ZoneInfo:Register(zoneName, module, requireBossFilter)
    local t = db[zoneName];
    if not t then
        t = { };
        db[zoneName] = t;
    end
    tinsert(t, {name = module, hasFilter = requireBossFilter or false});
end