local Root = BossEncounter3;

local Environment = Root:GetOrNewModule("Environment");

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

Environment.last = nil;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Get the format of the current instance (mode and size).          *
-- * This function makes the difference between 5, 5H,                *
-- * 10, 10H, 25, 25H and 25R using two return values.                *
-- * This function also takes dynamic difficulty in account.          *
-- ********************************************************************
function Environment:GetInstanceFormat()
    local name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic = GetInstanceInfo();
    if type == "raid" then
        if IsInLFGDungeon() and difficultyIndex == 2 then
            return "LFR", maxPlayers, maxPlayers.."R";
    elseif difficultyIndex >= 3 or (isDynamic and dynamicDifficulty == 1) then
            return "HEROIC", maxPlayers, maxPlayers.."H";
      else
            return "NORMAL", maxPlayers, maxPlayers.."N";
        end
elseif type == "party" then
        if difficultyIndex == 2 then
            return "HEROIC", maxPlayers, maxPlayers.."H";
      else
            return "NORMAL", maxPlayers, maxPlayers.."N";
        end
  else
        return "NORMAL", 1, "1";
    end
end

-- ********************************************************************
-- * Check instance format hasn't changed in the meantime.            *
-- ********************************************************************
function Environment:CheckChange()
    local mode, players, code = self:GetInstanceFormat();
    if self.last ~= code then
        if self.last then
            Root:InvokeHandler("OnInstanceFormatChange", mode, players);
        end
        self.last = code;
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function Environment:OnBootComplete()
    self:OnWorldEnter();
end

function Environment:OnWorldEnter()
    self:CheckChange();
end