local Root = BossEncounter3;

local Social = Root:GetOrNewModule("Social");
local Utility = Root:GetOrNewModule("Utility");

local Authorization = Root:GetOrNewModule("Authorization");
local Unit = Root:GetOrNewModule("Unit");

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the whisper text.                                       *
-- * >> target: the target of the whisper.                            *
-- * >> usePrefix: if set, the mod signature will be added at the     *
-- *               beginning of the message.                          *
-- ********************************************************************
-- * Send a whisper on the chat.                                      *
-- * If target is prefixed with "BN:", the whisper will be sent on    *
-- * Battle.net instead, to the people with the specified Bnet ID.    *
-- ********************************************************************
function Social:Whisper(text, target, usePrefix)
    if usePrefix then
        text = "[BossEncounter3] "..text;
    end
    if target:sub(1, 3) == "BN:" then
        target = tonumber(target:sub(4));
        if target then
            BNSendWhisper(target, text);
        end
  else
        SendChatMessage(text, "WHISPER", nil, target);
    end
end

-- /run BossEncounter3.Social:Whisper("test", "BN:19", true)

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the text to say on chat.                                *
-- * >> channel: if specified, this will force the output there.      *
-- * >> usePrefix: if set, the mod signature will be added at the     *
-- *               beginning of the message.                          *
-- ********************************************************************
-- * Send a whisper on the appropriate chat.                          *
-- ********************************************************************
function Social:Say(text, channel, usePrefix)
    if usePrefix then
        text = "[BossEncounter3] "..text;
    end
    local tchannel = "SAY";
    if channel then
        tchannel = channel;

elseif IsInRaid() then
        tchannel = "RAID";

elseif GetNumGroupMembers() > 0 then
        tchannel = "PARTY";
    end
    SendChatMessage(text, tchannel);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> where: where to display the text.                             *
-- * >> text: the text to display.                                    *
-- * Can be one of the following:                                     *
-- * RAID_WARNING: the raid warning channel.                          *
-- * OFFICER: the officer channel.                                    *
-- * GUILD: the guild channel.                                        *
-- * GROUP: the raid/party channel.                                   *
-- * SELF: to self.                                                   *
-- * name: whisper to the player named "name".                        *
-- ********************************************************************
-- * Send a message to the chat (advanced mode).                      *
-- ********************************************************************
function Social:Output(where, text)
    if where == "SELF" then
        Root:Print(text, true);

elseif where == "OFFICER" or where == "GUILD" or where == "GROUP" or where == "RAID_WARNING" then
        if where == "GROUP" then
            where = nil;
        end
        self:Say(text, where, false);
  else
        self:Whisper(text, where, false);
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> unit: can be an UID, name or GUID of someone in the raid.     *
-- * >> reason: why the unit was kicked. Optionnal.                   *
-- ********************************************************************
-- * Try to kick an unit from the raid. Will do & say nothing if you  *
-- * do not have the permission to do this.                           *
-- * Return true if kick command was issued successfully.             *
-- * Return false if not.                                             *
-- ********************************************************************
function Social:BootFromRaid(unit, reason)
    unit = Unit:ResolveUID(unit);
    if unit and (not UnitIsUnit(unit, "player")) then
        if Authorization:Check("KICK", "player", unit) then
            if reason then
                self:Say(reason, nil, true);
            end
            UninviteUnit(unit);
            return true;
        end
    end
    return false;
end
