local Root = BossEncounter3;

local Sort = Root:GetOrNewModule("Sort");

-- --------------------------------------------------------------------
-- **                         Local functions                        **
-- --------------------------------------------------------------------

local NUMERIC_SORT_FIELD = "NULL";

local function ByNameSortFunc(item1, item2)
    local name1, name2 = item1, item2;
    if type(item1) == "table" and type(item2) == "table" then
        name1, name2 = item1.name, item2.name;
    end
    if type(name1) ~= "string" or type(name2) ~= "string" then
        return false;
    end

    local length = min(#name1, #name2);
    local b1, b2, i;

    for i=1, length do
        b1 = strbyte(name1, i);
        b2 = strbyte(name2, i);
        if ( b1 < b2 ) then
            return true;
    elseif ( b1 > b2 ) then
            return false;
        end
    end

    if ( #name1 < #name2 ) then
        return true;
    end
    return false;
end

local function ByNumericFieldSortFunc(item1, item2)
    return item1[NUMERIC_SORT_FIELD] < item2[NUMERIC_SORT_FIELD];
end

local function ByNumericFieldRevertSortFunc(item1, item2)
    return item1[NUMERIC_SORT_FIELD] > item2[NUMERIC_SORT_FIELD];
end

-- --------------------------------------------------------------------
-- **                               API                              **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> theTable: the table containing the items to sort.             *
-- ********************************************************************
-- * Sort a table containing sub-tables with .name field in normal    *
-- * or reversed order.                                               *
-- ********************************************************************
function Sort:Name(theTable)
    table.sort(theTable, ByNameSortFunc);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> theTable: the table containing the items to sort.             *
-- * >> revert: revert the sorting order. True will put the highest   *
-- * items first. False will put the lowest items first.              *
-- ********************************************************************
-- * Sort a table containing sub-tables with a custom numeric field   *
-- * in increasing or decreasing order.                               *
-- ********************************************************************
function Sort:NumericField(theTable, field, revert)
    NUMERIC_SORT_FIELD = field;
    if ( not revert ) then
        table.sort(theTable, ByNumericFieldSortFunc);
  else
        table.sort(theTable, ByNumericFieldRevertSortFunc);
    end
end