local Root = BossEncounter3;

local Table = Root:GetOrNewModule("Table");

-- --------------------------------------------------------------------
-- **                            Locals                              **
-- --------------------------------------------------------------------

local freeTables = { };

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Allocate a new table. Unused tables will be used for allocation  *
-- * first instead of creating new ones.                              *
-- * Use Recycle once you have finished using the table.              *
-- ********************************************************************
function Table:Alloc()
    if #freeTables > 0 then
        local t = freeTables[#freeTables];
        freeTables[#freeTables] = nil;
        return t;
    end
    return { };
end

Table.Allocate = Table.Alloc;

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> theTable: a reference to the table you are recycling.         *
-- * >> includeNesting: if set, all tables contained within the table *
-- * will also be recycled (advanced / risky use if careless).        *
-- ********************************************************************
-- * Free a table once you no longer use it, so it can be reused      *
-- * later. This function also returns nil so you can easily void     *
-- * the reference.                                                   *
-- ********************************************************************
function Table:Recycle(theTable, includeNesting)
    local k, v;
    for k, v in pairs(theTable) do
        theTable[k] = nil;
        if includeNesting and type(v) == "table" then
            self:Recycle(v, true);
        end
    end
    freeTables[#freeTables+1] = theTable;
    return nil;
end

Table.Free = Table.Recycle;

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> object: a reference to the table you are copying.             *
-- ********************************************************************
-- * Copy completely the content of a table into a new table.         *
-- ********************************************************************
function Table:Copy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object;
        elseif lookup_table[object] then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _copy(object);
end
