﻿local Root = BossEncounter3;

local Utility = Root:GetOrNewModule("Utility");

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

local backdropLayouts = {
    ["TOOLTIP"] = {
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        insets = { left = 4, right = 4, top = 4, bottom = 4 },
        edgeSize = 16,
        tileSize = 16,
        tile = true,
        borderColor = {0.5, 0.5, 0.5, 1.0},
        color = {0.09, 0.09, 0.19, 0.5},
    },
    ["OPTION"] = {
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        insets = { left = 4, right = 4, top = 4, bottom = 4 },
        edgeSize = 16,
        tileSize = 16,
        tile = true,
        borderColor = {0, 0, 0, 1.0},
        color = {0, 0, 0, 0.5},
    },
    ["BUBBLE"] = {
        bgFile = Root:GetFolder().."gfx\\bubbleInside",
        edgeFile = Root:GetFolder().."gfx\\bubble",
        insets = { left = 8, right = 8, top = 8, bottom = 8 },
        edgeSize = 8,
        tileSize = 16,
        tile = true,
        borderColor = {1.0, 1.0, 1.0, 0.75},
        color = {1.0, 1.0, 1.0, 0.75},
    },
};

local flatSearch = { '\195\160', '\195\168', '\195\172', '\195\178', '\195\185',
                     '\195\161', '\195\169', '\195\173', '\195\179', '\195\186',
                     '\195\162', '\195\170', '\195\174', '\195\180', '\195\187',
                     '\195\163', '\195\171', '\195\175', '\195\181', '\195\188',
                     '\195\164', '\195\166', '\195\167', '\195\177', '\195\182',
                     '\195\184', '\195\132', '\195\150', '\195\156', '\195\169',
                            'Ð',        'ÿ',        'å',        'ø',        'Ø' };

local flatReplacement = { 'a', 'e', 'i', 'o', 'u',
                          'a', 'e', 'i', 'o', 'u',
                          'a', 'e', 'i', 'o', 'u',
                          'a', 'e', 'i', 'o', 'u',
                          'a','ae', 'c', 'n', 'o',
                          'o', 'A', 'O', 'U','ss',
                          'D', 'y', 'a', 'o', 'o' };

local cleanSearch = { ' ', '\226\128\153' };

local cleanReplacement = { ' ', "'" };

local flatLookup = { };

local vowels = { ["a"] = true, ["i"] = true, ["u"] = true, ["e"] = true, ["o"] = true, ["y"] = true };

local ALWAYS_TRUE_FUNC = function() return true; end;

local tooltipEnter = function(self)
    local text, requirement = self.tooltipText, self.tooltipRequirement;
    if text then
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        if type(text) == "number" then
            GameTooltip:SetSpellByID(text);
      else
            GameTooltip:SetText(text, nil, nil, nil, nil, 1);
        end
    end
    if requirement then
        GameTooltip:AddLine(requirement, "", 1.0, 1.0, 1.0);
        GameTooltip:Show();
    end
end;

local tooltipLeave = function(self)
    GameTooltip:Hide();
end;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> table: the table containing incomplete values.                *
-- * This table should be read-only.                                  *
-- * >> index: the index you wish to read. If it is not found, it     *
-- * will be linearly interpolated.                                   *
-- ********************************************************************
-- * Returns a value from a table, or interpolate it if it's missing. *
-- ********************************************************************
function Utility:LinearInterpolation(table, index)
    -- Got a value for given level. Return it.
    if table[index] then
        return table[index];
  else
        local i, maxIndex;
        maxIndex = table.maxIndex;
        if not maxIndex then
            for i in pairs(table) do
                if type(i) == "number" then
                    maxIndex = max(maxIndex or 0, i);
                end
            end
        end
        table.maxIndex = maxIndex or 0;

        -- Interpolate linearly.
        local lastStepValue, nextStepValue = nil, nil;
        local lastStepIndex, nextStepIndex = index, index;

        while ( lastStepIndex > 1 and not lastStepValue ) do
            lastStepIndex = lastStepIndex - 1;
            if table[lastStepIndex] then lastStepValue = table[lastStepIndex]; end
        end
        while ( nextStepIndex < maxIndex and not nextStepValue ) do
            nextStepIndex = nextStepIndex + 1;
            if table[nextStepIndex] then nextStepValue = table[nextStepIndex]; end
        end

        if lastStepValue and nextStepValue then
            local valueDelta = nextStepValue - lastStepValue;
            local indexDelta = nextStepIndex - lastStepIndex;
            if indexDelta > 0 then
                local progression = (index - lastStepIndex) / indexDelta;
                return floor(lastStepValue + (nextStepValue - lastStepValue) * progression);
            end
        end

        return nil;
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> frame: the frame that is to be made draggable.                *
-- * >> conditionFunc: the function that will be called to            *
-- * determinate if the dragging is allowed. The frame itself will    *
-- * be passed to the function as argument. It must return a boolean. *
-- * If you input nil, the dragging will be always allowed.           *
-- * >> startHandler: whether you want to use your own special        *
-- * handler for the dragging. If this argument is not nil, the       *
-- * conditionFunc argument will have no effect.                      *
-- * >> stopHandler: same thing as above except it is the stop        *
-- * dragging handler.                                                *
-- ********************************************************************
-- * Quickly make a frame draggable.                                  *
-- ********************************************************************
function Utility:MakeDraggable(frame, conditionFunc, startHandler, stopHandler)
    frame:EnableMouse(true);
    frame:RegisterForDrag("LeftButton");
    frame:SetMovable(true);

    if type(startHandler) == "function" then
        frame:SetScript("OnDragStart", startHandler);
  else
        frame.DRAG_CONDITION = conditionFunc or ALWAYS_TRUE_FUNC;
        frame:SetScript("OnDragStart", function(self) if self:DRAG_CONDITION() then self:StartMoving(); end end);
    end
    if type(stopHandler) == "function" then
        frame:SetScript("OnDragStop", stopHandler);
  else
        frame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); end);
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> texture: the desired texture.                                 *
-- * >> angle: the rotation angle.                                    *
-- ********************************************************************
-- * Make a texture rotate around its center.                         *
-- ********************************************************************
function Utility:RotateTexture(texture, angle)
    angle = -angle + 135;
    local c, s = cos(angle), sin(angle);
    texture:SetTexCoord(0.5-s, 0.5+c, 0.5+c, 0.5+s, 0.5-c, 0.5-s, 0.5+s, 0.5-c);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> frame: the frame that has its backdrop changed.               *
-- * >> layout: the layout to use for the backdrop. See above.        *
-- ********************************************************************
-- * Quickly change a frame's backdrop according to presets.          *
-- ********************************************************************
function Utility:SetBackdrop(frame, layout)
    local info = backdropLayouts[layout];
    if info then
        frame:SetBackdrop(info);
        frame:SetBackdropBorderColor(unpack(info.borderColor));
        frame:SetBackdropColor(unpack(info.color));
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> texture: the texture on which the icon is displayed.          *
-- * >> icon: either a class name or a raid target ID.                *
-- * 0 and nil can be passed to hide the texture as well.             *
-- ********************************************************************
-- * Quick method of putting an icon on a texture.                    *
-- ********************************************************************
function Utility:SetIcon(texture, icon)
    if type(icon) == "string" then
        texture:SetTexture("Interface\\WorldStateFrame\\Icons-Classes");
        texture:SetTexCoord(unpack(CLASS_BUTTONS[icon]));
        texture:Show();

elseif type(icon) == "number" and icon > 0 then
        texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons");
        SetRaidTargetIconTexture(texture, icon);
        texture:Show();
  else
        texture:Hide();
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> frame: the frame whose tooltip is set.                        *
-- * >> title: the title of the tooltip.                              *
-- * >> explanation: the explanation of the tooltip.                  *
-- ********************************************************************
-- * Quick way to set up a tooltip for a frame.                       *
-- * Pass the magic string 'KEEP' to keep a portion unchanged.        *
-- * If the frame has the method GetText, you can also use 'AUTO'     *
-- * for the title. Use 'CLEAR' in title to clear the tooltip.        *
-- * You can pass an integer to title to set a spell tooltip instead. *
-- ********************************************************************
function Utility:SetTooltip(frame, title, explanation)
    if title == 'CLEAR' then
        frame.tooltipText = nil;
        frame.tooltipRequirement = nil;
        return;

elseif title == 'AUTO' then
        title = frame:GetText();

elseif title == 'KEEP' then
        title = frame.tooltipText;
    end
    if explanation == 'KEEP' then
        explanation = frame.tooltipRequirement;
    end

    if type(title) == "number" then
        frame.tooltipText = title;
        frame.tooltipRequirement = nil;
  else
        frame.tooltipText = "|cffffffff"..title.."|r";
        frame.tooltipRequirement = explanation;
    end

    if ( not frame.tooltipHandler ) then
        frame.tooltipHandler = true;

        -- Install handlers
        if frame:GetScript("OnEnter") then
            frame:HookScript("OnEnter", tooltipEnter);
      else
            frame:SetScript("OnEnter", tooltipEnter);
        end
        if frame:GetScript("OnLeave") then
            frame:HookScript("OnLeave", tooltipLeave);
      else
            frame:SetScript("OnLeave", tooltipLeave);
        end
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> formatString: the format string.                              *
-- * The following tokens can be used:                                *
-- * %S for seconds, %M for minutes, %H for hours, %C for 1/100th s.  *
-- * >> time: the time amount (in sec) that will be used for          *
-- * this formatting.                                                 *
-- * >> xNoPad: if set, it will prevent the format function from      *
-- * adding a "0" if value of x is below 10.                          *
-- ********************************************************************
-- * Format a countdown or countdown-like string.                     *
-- ********************************************************************
function Utility:FormatCountdown(formatString, time, hrsNoPad, minNoPad, secNoPad)
    if type(time) ~= "number" or type(formatString) ~= "string" then return "INVALID"; end
    
    local floor, fmod = math.floor, math.fmod;
    local hrs, min, sec, cen;

    -- Extract each component

    hrs = floor(floor(time) / 3600);
    min = floor(fmod(floor(time) / 60, 60));
    sec = floor(fmod(floor(time), 60));
    cen = floor(fmod(floor(time * 100), 100));

    -- Pads a "0" if below 10 to look pretty (0:4 is dumb for instance, 00:04 is nice),
    -- given there is not a noPad parameter set to prevent us doing this.

    if ( hrs < 10 and not hrsNoPad ) then hrs = "0"..tostring(hrs); else hrs = tostring(hrs); end
    if ( min < 10 and not minNoPad ) then min = "0"..tostring(min); else min = tostring(min); end
    if ( sec < 10 and not secNoPad ) then sec = "0"..tostring(sec); else sec = tostring(sec); end
    if ( cen < 10 ) then cen = "0"..tostring(cen); else cen = tostring(cen); end

    -- Build the result string

    local text = formatString;
    text = text:gsub("%%H", hrs);
    text = text:gsub("%%M", min);
    text = text:gsub("%%S", sec);
    text = text:gsub("%%C", cen);
    return text;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the text processed.                                     *
-- ********************************************************************
-- * Remove accents, umlauts etc. characters from text.               *
-- ********************************************************************
function Utility:ToFlatString(text)
    local original = text;
    if flatLookup[text] then
        return flatLookup[text];
    end
    local i, search, replacement;
    for i=1, #flatSearch do
        search = flatSearch[i];
        replacement = flatReplacement[i];
        text = text:gsub(search, replacement);
    end
    flatLookup[original] = text;
    return text;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the text processed.                                     *
-- ********************************************************************
-- * Normalize space and apostrophes characters.                      *
-- ********************************************************************
function Utility:ToCleanString(text)
    local i, search, replacement;
    for i=1, #cleanSearch do
        search = cleanSearch[i];
        replacement = cleanReplacement[i];
        text = text:gsub(search, replacement);
    end
    return text;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the text processed.                                     *
-- ********************************************************************
-- * Print each character of a string, one by one.                    *
-- ********************************************************************
function Utility:DisassembleString(text)
    local len = #text;
    local i, char;
    Root:PrintFmt("Disassembling string of width %d: [%s]", len, text);
    for i=1, len do
        char = text:sub(i , i);
        Root:PrintFmt("#%d: %s (code: %d)", i, char, strbyte(char));
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> texture: the texture being set up.                            *
-- * >> filename: filename to the texture.                            *
-- * >> textureWidth, textureHeight: real dimensions of the file.     *
-- * >> startX, startY: begin coordinates of the region shown.        *
-- * >> width, height: dimensions of the region shown.                *
-- ********************************************************************
-- * Pick a region on a texture file and display it on the specified  *
-- * texture.                                                         *
-- ********************************************************************
function Utility:ShowTextureRegion(texture, filename, fileWidth, fileHeight, startX, startY, width, height)
    local beginX = startX / fileWidth;
    local endX = (startX + width) / fileWidth;
    local beginY = startY / fileHeight;
    local endY = (startY + height) / fileHeight;

    if filename then
        texture:SetTexture(filename);
    end
    texture:SetTexCoord(beginX, endX, beginY, endY);
    texture:SetWidth(width);
    texture:SetHeight(height);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> pts: the amount of points.                                    *
-- ********************************************************************
-- * Converts a given amount of points in a rank.                     *
-- * Returned value can be: "S", "A", "B", "C", "D", "E".             *
-- ********************************************************************
function Utility:PtsToRank(pts)
    if pts > 300 then
        return "S";
elseif pts < 100 then
        return "E";
elseif pts < 150 then
        return "D";
elseif pts < 200 then
        return "C";
elseif pts < 250 then
        return "B";
  else
        return "A";
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the text to abbreviate.                                 *
-- * >> maxLength: the maximum length of the text.                    *
-- ********************************************************************
-- * Abbreviate a text, making it fit in the specified length.        *
-- ********************************************************************
function Utility:Abbreviate(text, maxLength)
    local initialWords = { };
    local words = { };
    local current = 1;
    local continue = true;
    local final = text;

    for w in text:gmatch("[A-Za-z]+") do
        if #w > 4 then
            tinsert(initialWords, w);
            tinsert(words, w);
        end
    end

    local function shorten(index)
        -- Try to shorten a word.
        local word = words[index];
        local pos = #word;
        while pos > 4 do
            local currentChar = word:sub(pos, pos);
            local prevChar = word:sub(pos-1, pos-1);
            -- if (not vowels[currentChar]) and (not vowels[prevChar]) then
            if not vowels[prevChar] then
                words[index] = word:sub(1, pos-1);
                return true;
            end
            pos = pos - 1;
        end
        return false;
    end

    local function compose()
        local str = text;
        local i, word, replace; 
        for i=1, #words do
            word = initialWords[i];
            replace = words[i];
            if word ~= replace then
                str = str:gsub(word, replace..".");
            end
        end
        return str;
    end

    while continue do
        if #final <= maxLength then
            continue = false;

    elseif current > #words then
            continue = false;

    elseif not shorten(current) then
            current = current + 1;
      else
            final = compose();
        end
    end

    return final:sub(1, maxLength);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> message: the text containing a spell link.                    *
-- ********************************************************************
-- * Extract a spellID from a spell link contained in a string.       *
-- ********************************************************************
function Utility:ExtractSpellID(message)
    local _, _, spellString = message:find("|c%x+|H(.+)|h%[.*%]");
    if spellString then
        local _, _, spellId = spellString:find("^spell:(%d*)");
        if spellId then
            return tonumber(spellId);
        end
    end
    return nil;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> symbol: the index of the symbol to display.                   *
-- ********************************************************************
-- * Get the symbol link.                                             *
-- ********************************************************************
function Utility:GetSymbolLink(symbol)
    local str = ICON_LIST[symbol];
    if str then
        return str.."0|t";
  else
        return "";
    end
end