local Root = BossEncounter3;

local Version = Root:GetOrNewModule("Version");

-- --------------------------------------------------------------------
-- **                             Data                               **
-- --------------------------------------------------------------------

Version.major = 3;
Version.minor = 9;
Version.revision = 15;
Version.string = ("%d.%d.%d"):format(Version.major, Version.minor, Version.revision);

-- --------------------------------------------------------------------
-- **                            Methods                             **
-- --------------------------------------------------------------------

function Version:GetNumber() -- Return the version numbers
    return self.major, self.minor, self.revision;
end

function Version:GetString() -- Return the version as a string
    return self.string;
end

function Version:GetFullInteger() -- Return the version as a full integer.
    return self.major * 10000 + self.minor * 100 + self.revision;
end

function Version:GetStringFromInt(fullInteger)
    local major, minor, revision = floor(fullInteger / 10000), math.fmod(floor(fullInteger / 100), 100), math.fmod(fullInteger, 100);
    return ("%d.%d.%d"):format(major, minor, revision);
end

function Version:IsCompatible(major, minor, revision) -- Return false if the specified remote system cannot cooperate with us.
    local oMajor, oMinor, oRevision = self:GetNumber();
    if ( major ~= oMajor ) then return false; end
    if ( minor ~= oMinor ) then return false; end
    return true;
end

function Version:Compare(major, minor, revision) -- Return -1 if we are older. Return 0 if both versions match. Return 1 if we are newer.
    local oMajor, oMinor, oRevision = self:GetNumber();
    if ( major < oMajor ) then return  1; end
    if ( major > oMajor ) then return -1; end
    if ( minor < oMinor ) then return  1; end
    if ( minor > oMinor ) then return -1; end
    if ( revision < oRevision ) then return  1; end
    if ( revision > oRevision ) then return -1; end
    return 0;
end
