local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local L = Root:GetOrNewModule("Localization");
local Resources = Root:GetOrNewModule("Resources");
local Audio = Root:GetOrNewModule("Audio");

local ClearedAnim = GUI:GetOrNewWidget("ClearedAnim");

ClearedAnim.TEMPLATE = "BossEncounter3_ClearedAnimTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Animation that gets played once an encounter is won.

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

local lettersOffset = {
    [1] = -224,
    [2] = -160,
    [3] = -96,
    [4] = -32,
    [5] = 32,
    [6] = 96,
    [7] = 160,
    [8] = 224,
}

-- All anims parameter
local CLOSE_TIME = 2.00; -- Both anims
local LETTER_ANIM_DURATION = 0.30; -- Normalised value between 0 and 1.

-- NORMAL anim parameters
local ZOOM_DURATION = 1.50;

-- ALTERNATE anim parameters
local APPEAR_DURATION = 2.50;

local ANIMS = {
    ["NORMAL"] = {
        OnStart = function(self, coupDeGrace)
            self.status = "ZOOMING";
            self.timer = ZOOM_DURATION;
            self.coupDeGrace = coupDeGrace;

            self.flash:SetWidth(1024);
            self.flash:SetHeight(1024);
            self.flash:SetAlpha(1);
            self.flash:Show();
            self.blank:Show();

            local i;
            for i=1, #self.letters do
                self.letters[i]:SetDrawLayer("ARTWORK", -3 + i);
                self.letters[i]:Show();
            end

            if coupDeGrace then
                self.coupDeGraceBand:Show();
                Audio:PlaySound("COUPDEGRACE");
          else
                Audio:PlaySound("CLEARED1");
            end
        end,

        OnEnd = function(self)
            self.coupDeGrace = false;

            self.flash:Hide();
            self.blank:Hide();

            local i;
            for i=1, #self.letters do
                self.letters[i]:Hide();
            end

            self.coupDeGraceBand:Hide();
        end,

        OnUpdate = function(self, elapsed)
            local i, letter;
            local letterProgression;
            local globalProgression;
            local beginAnim, endAnim;

            self.timer = max(0, self.timer - elapsed);

            if self.status == "ZOOMING" then
                -- Update each letter.
                globalProgression = 1 - self.timer / ZOOM_DURATION;

                for i=1, #self.letters do
                    -- Determinate the progression, which is a bounded value between 0.00 and 1.00.
                    beginAnim = (i-1) * ((1.0 - LETTER_ANIM_DURATION) / (#self.letters-1));
                    endAnim = beginAnim + LETTER_ANIM_DURATION;
                    letterProgression = (globalProgression - beginAnim) / (endAnim - beginAnim);
                    letterProgression = max(0, min(letterProgression, 1));

                    -- Apply the progression on the letter
                    local zoom = 1 / letterProgression;
                    letter = self.letters[i];
                    letter:SetAlpha(min(1, letterProgression / 0.20));
                    letter:SetWidth(64 * zoom);
                    letter:SetHeight(64 * zoom);
                    letter:ClearAllPoints();
                    letter:SetPoint("CENTER", self, "CENTER", lettersOffset[i] * zoom, 0);
                end

                self.coupDeGraceBand:SetAlpha(globalProgression);
                self.coupDeGraceText:SetPoint("CENTER", self, "CENTER", 512 * (1 - globalProgression)^2, -96);

                -- Transition
                if self.timer == 0 then
                    self.status = "WAITING";

                    self.flash:Hide();
                    self.blank:Hide();
              else
                    local beginX, beginY, endX, endY, x, y;
                    beginX = 0-(512 / WorldFrame:GetWidth());
                    beginY = 0-(512 / WorldFrame:GetHeight());
                    endX = 1+(512 / WorldFrame:GetWidth());
                    endY = 1+(512 / WorldFrame:GetHeight());
                    x = beginX + (endX - beginX) * globalProgression;
                    y = beginY + (endY - beginY) * globalProgression;

                    self.flash:ClearAllPoints();
                    self.flash:SetPoint("CENTER", WorldFrame, "BOTTOMLEFT", x * WorldFrame:GetWidth(), y * WorldFrame:GetHeight());
                    self.blank:SetAlpha((1 - globalProgression) * 0.75);
                end

        elseif self.status == "WAITING" then
                -- Waiting for the Stop method call.

        elseif self.status == "CLOSING" then
                if self.timer == 0 then
                    self:Stop(true);
              else
                    -- Update each letter.
                    globalProgression = 1 - self.timer / CLOSE_TIME;

                    for i=1, #self.letters do
                        -- Determinate the progression, which is a bounded value between 0.000 and 1.000.
                        beginAnim = (i-1) * ((1.0 - LETTER_ANIM_DURATION) / (#self.letters-1));
                        endAnim = beginAnim + LETTER_ANIM_DURATION;
                        letterProgression = ( globalProgression - beginAnim ) / ( endAnim - beginAnim);
                        letterProgression = max(0.000, min(letterProgression, 1.000));

                        -- Apply the progression on the letter
                        self.letters[i]:SetAlpha(1 - letterProgression);
                    end

                    self.coupDeGraceBand:SetAlpha(1 - globalProgression);
                end
            end
        end,
    },
};

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Get the current status of the animation.                         *
-- ********************************************************************
function ClearedAnim:GetStatus()
    return self.status;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> animation: the animation used: NORMAL, ALTERNATE, EPICFAIL.   *
-- * >> coupDeGrace: true if you dealt the Coup de Grace yourself.    *
-- ********************************************************************
-- * Starts playing the cleared animation.                            *
-- ********************************************************************
function ClearedAnim:Play(animation, coupDeGrace)
    if self.status == "STANDBY" or self.status == "CLOSING" then
        self.animation = animation or "NORMAL";
        ANIMS[self.animation].OnStart(self, coupDeGrace or false);
        self:Show();
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> atOnce: if set, the animation will be hidden at once.         *
-- ********************************************************************
-- * Stop playing the cleared animation.                              *
-- ********************************************************************
function ClearedAnim:Stop(atOnce)
    if self.status ~= "STANDBY" and atOnce then
        self.status = "STANDBY";
        ANIMS[self.animation].OnEnd(self);
        self:Hide();

elseif self.status == "ZOOMING" or self.status == "WAITING" or self.status == "APPEARING" then
        self.status = "CLOSING";
        self.timer = CLOSE_TIME;
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function ClearedAnim:OnLoad()
    local _, flashFilename = Resources:GetTextureInfo("FLASH");
    local _, clearedFilename = Resources:GetTextureInfo("CLEARED");

    -- ** Children **
    self.flash = self:CreateTexture(nil, "BACKGROUND");
    self.flash:SetTexture(flashFilename);
    self.flash:Hide();
    self.blank = self:CreateTexture(nil, "OVERLAY");
    self.blank:SetTexture(1, 1, 1, 1);
    self.blank:SetAllPoints(WorldFrame);
    self.blank:Hide();
    self.coupDeGraceBand = GUI:RequestChild("Band", nil, self);
    self.coupDeGraceBand:SetFixedBorders(true);
    self.coupDeGraceText = GUI:RequestChild("MonoFont", nil, self.coupDeGraceBand);
    self.coupDeGraceText:SetText(L:Get("CoupDeGrace"));
    self.coupDeGraceBand:SetPoint("TOPLEFT",     self.coupDeGraceText,     "TOPLEFT", -32,  8);
    self.coupDeGraceBand:SetPoint("BOTTOMRIGHT", self.coupDeGraceText, "BOTTOMRIGHT",  32, -8);
    self.coupDeGraceBand:Hide();

    self.letters = { };
    local i, letter;
    for i=1, #lettersOffset do
        letter = self:CreateTexture(nil, "ARTWORK");
        letter:SetTexture(clearedFilename);
        letter:SetTexCoord((i-1)*(64/512), i*(64/512), 0, 1);
        letter:Hide();
        self.letters[i] = letter;
    end

    -- ** Properties **
    self.status = "STANDBY";
    self.timer = 0;
    self.animation = "NORMAL";
    self.coupDeGrace = false;

    -- ** Initial config **
    self:SetWidth(lettersOffset[#self.letters] + 64);
    self:SetHeight(64);
end

function ClearedAnim:OnUpdate(elapsed)
    if self.status == "STANDBY" or not self.animation then
        self:Hide();
  else
        ANIMS[self.animation].OnUpdate(self, elapsed);
    end
end