local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Utility = Root:GetOrNewModule("Utility");
local Resources = Root:GetOrNewModule("Resources");

local GraphColumn = GUI:GetOrNewWidget("GraphColumn");

GraphColumn.TEMPLATE = "BossEncounter3_GraphColumnTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- A vertical bar for a graph.

-- --------------------------------------------------------------------
-- **                             Locals                             **
-- --------------------------------------------------------------------

local SPECIAL_SWAY_TIME = 1.00;

local BLACK = { r = 0, g = 0, b = 0 };


-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> width: the total width of the column.                         *
-- * >> height: the total height of the column.                       *
-- * >> class: the class represented by this column.                  *
-- * >> dps: the DPS done.                                            *
-- * >> isLocal: true if this column is about the local player.       *
-- * >> killingBlow: true if this entity has the killing blow.        *
-- ********************************************************************
-- * Starts displaying a column in the dps graph.                     *
-- ********************************************************************
function GraphColumn:Set(width, height, class, dps, isLocal, killingBlow)
    self:SetWidth(width);
    self.height = height;
    self.dps = dps;
    self.isLocal = isLocal or false;

    -- Set the color of the bar.
    local color = RAID_CLASS_COLORS[class] or BLACK;
    self.bar:SetTexture(color.r, color.g, color.b);

    -- Display the class icon.
    Utility:SetIcon(self.class, class);

    -- Display the special icon.
    if killingBlow then
        local _, specialFilename = Resources:GetTextureInfo("KILLING_BLOW");
        self.special:SetTexture(specialFilename);
        self.special:Show();
  else
        self.special:Hide();
    end

    self:Update(0, 0);
    self:Show();
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> progression: the progression of the grow.                     *
-- * >> gxProgression: the nice progression of the grow.              *
-- * Both progressions are numbers between 0 and 1.                   *
-- ********************************************************************
-- * Update a column of the graph.                                    *
-- ********************************************************************
function GraphColumn:Update(progression, gxProgression)
    local size = max(1, gxProgression * self.height);
    local displayDPS = self.dps * gxProgression;

    self:SetHeight(size);
    self.label:SetText(("%d"):format(displayDPS));
    self.class:SetAlpha(progression);

    local sway = sin(math.fmod(GetTime(), SPECIAL_SWAY_TIME) * 180 / SPECIAL_SWAY_TIME);

    self.special:SetPoint("BOTTOM", self, "TOP", 0, 16 + sway * 6);
    self.special:SetAlpha(progression);

    if self.isLocal then
        self:SetAlpha(0.3 + 0.7 * sway);
  else
        self:SetAlpha(1);
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function GraphColumn:OnLoad()
    -- ** Children **
    self.bar = self:CreateTexture(nil, "BACKGROUND");
    self.bar:SetAllPoints(true);
    self.class = self:CreateTexture(nil, "BACKGROUND");
    self.class:SetWidth(16);
    self.class:SetHeight(16);
    self.class:SetPoint("TOP", self, "BOTTOM", 0, -2);
    self.special = self:CreateTexture(nil, "OVERLAY");
    self.special:SetWidth(64);
    self.special:SetHeight(8);
    self.label = self:CreateFontString(self, "ARTWORK", "GameFontHighlightSmall");
    self.label:SetPoint("BOTTOM", self, "TOP", 0, 2);

    -- ** Properties **
    self.height = 16;
    self.dps = 0;
    self.isLocal = false;

    -- ** Initial config **
    self:Hide();
end
