local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Utility = Root:GetOrNewModule("Utility");

local ScoreCriteria = GUI:GetOrNewWidget("ScoreCriteria");

ScoreCriteria.TEMPLATE = "BossEncounter3_ScoreCriteriaTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- A bar displaying some criteria.

-- --------------------------------------------------------------------
-- **                             Locals                             **
-- --------------------------------------------------------------------

local LABEL_SCALE = 0.75;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> short: abbreviation of the criteria's label.                  *
-- * >> full: criteria's full label.                                  *
-- * >> explanation: explanations of the criteria.                    *
-- * >> r, g, b: colors of the criteria.                              *
-- ********************************************************************
-- * Set the layout of a score criteria.                              *
-- ********************************************************************
function ScoreCriteria:SetCriteria(short, full, explanation, r, g, b)
    self.label:SetText(short);
    self.counter:SetBarColor(r, g, b);
    Utility:SetTooltip(self, full, explanation);

    self:UpdateSize();
end

-- ********************************************************************
-- * Reset the score criteria to zero.                                *
-- ********************************************************************
function ScoreCriteria:Reset()
    self.counter:SetInfiniteRate();
    self.counter:SetValue(0, 120);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> value: the desired value.                                     *
-- * >> time: time needed to grow.                                    *
-- ********************************************************************
-- * Make the score criteria grow to the specified value.             *
-- ********************************************************************
function ScoreCriteria:Grow(value, time)
    local delta = abs(value - self.counter:GetDisplayValue());

    self.counter:SetRate(delta / (time * 120));
    self.counter:SetValue(value);
end

-- ********************************************************************
-- * Recalculate the appropriate size for the whole score criteria.   *
-- ********************************************************************
function ScoreCriteria:UpdateSize()
    local width = self.label:GetWidth() * LABEL_SCALE + self.counter:GetWidth();
    local height = self.counter:GetHeight();

    self:SetWidth(width + 48);
    self:SetHeight(height + 16);
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function ScoreCriteria:OnLoad()
    -- ** Children **
    self.label = GUI:RequestChild("MonoFont", nil, self, 1);
    self.label:SetPoint("LEFT", self, "LEFT", 24, 0);
    self.label:SetScale(LABEL_SCALE);
    self.counter = GUI:RequestChild("Counter", nil, self, 1);
    self.counter:SetDimensions(3);
    self.counter:SetPoint("LEFT", self.label, "RIGHT", 0, 0);
    self.band = GUI:RequestChild("Band", nil, self, 0);
    self.band:SetAllPoints(self);
    self.band:SetGraphic("BAND_HS");
    self.band:SetFixedBorders(true);

    -- ** Properties **
    self.highValue = false;

    -- ** Initial config **
    self:Reset();
    self:UpdateSize();
    self:Show();
end

function ScoreCriteria:OnUpdate(elapsed)
    local value = self.counter:GetDisplayValue();
    if value > 100 and not self.highValue then
        self.highValue = true;
        self.counter:SetTextColor(1, 1, 0);

elseif value <= 100 and self.highValue then
        self.highValue = false;
        self.counter:SetTextColor(1, 1, 1);
    end
end