local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Utility = Root:GetOrNewModule("Utility");

local ScoreTotal = GUI:GetOrNewWidget("ScoreTotal");

ScoreTotal.TEMPLATE = "BossEncounter3_ScoreTotalTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- A bar displaying a value out of 360, with two possible labels.

-- --------------------------------------------------------------------
-- **                             Locals                             **
-- --------------------------------------------------------------------

local LABEL_SCALE   = 0.50;
local COMMENT_SCALE = 0.75;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> label: the total label.                                       *
-- * >> r, g, b: colors of the bar.                                   *
-- ********************************************************************
-- * Set the layout of a total.                                       *
-- ********************************************************************
function ScoreTotal:Setup(label, r, g, b)
    self.label:SetText(label);
    self.counter:SetBarColor(r, g, b);

    self:UpdateSize();
end

-- ********************************************************************
-- * Reset the total to zero.                                         *
-- ********************************************************************
function ScoreTotal:Reset()
    self:SetValue(0);
    self:SetComment("");
end

-- ********************************************************************
-- * Reset the total to zero.                                         *
-- ********************************************************************
function ScoreTotal:SetValue(value)
    self.counter:SetInfiniteRate();
    self.counter:SetValue(value, 360);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> value: the desired value.                                     *
-- * >> time: time needed to grow.                                    *
-- ********************************************************************
-- * Make the total grow to the specified value.                      *
-- ********************************************************************
function ScoreTotal:Grow(value, time)
    local delta = abs(value - self.counter:GetDisplayValue());

    self.counter:SetRate(delta / (time * 360));
    self.counter:SetValue(value);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the comment text.                                       *
-- ********************************************************************
-- * Set the comment text.                                            *
-- ********************************************************************
function ScoreTotal:SetComment(text)
    self.comment:SetText(text);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> text: the comment text.                                       *
-- * >> time: time needed to appear.                                  *
-- * >> cycles: number of blink cycles.                               *
-- ********************************************************************
-- * Display a comment with an animation.                             *
-- ********************************************************************
function ScoreTotal:ShowComment(text, time, cycles)
    self.commentTimer = time;
    self.commentTimerMax = max(0.1, time);
    self.commentCycles = cycles;

    self:SetComment(text);
end

-- ********************************************************************
-- * Recalculate the appropriate size for the whole total.            *
-- ********************************************************************
function ScoreTotal:UpdateSize()
    local width = self.counter:GetWidth();
    local height = self.counter:GetHeight() + max(self.label:GetWidth() * LABEL_SCALE, self.comment:GetWidth() * COMMENT_SCALE);

    self:SetWidth(width);
    self:SetHeight(height);
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function ScoreTotal:OnLoad()
    -- ** Children **
    self.counter = GUI:RequestChild("Counter", nil, self, 1);
    self.counter:SetDimensions(3);
    self.counter:ResizeBar(256);
    self.counter:SetPoint("BOTTOM", self, "BOTTOM", 0, 0);

    local bar = self.counter:GetBar();
    bar:SetStyle("SIMPLE");

    self.label = GUI:RequestChild("MonoFont", nil, self, 1);
    self.label:SetScale(LABEL_SCALE);
    self.label:SetPoint("BOTTOMLEFT", bar, "TOPLEFT", 0, 0);
    self.comment = GUI:RequestChild("MonoFont", nil, self, 1);
    self.comment:SetColor(0, 1, 0);
    self.comment:SetScale(COMMENT_SCALE);
    self.comment:SetPoint("BOTTOMRIGHT", bar, "TOPRIGHT", 0, 0);

    -- ** Properties **
    self.commentTimer = 0;
    self.commentTimerMax = 1;
    self.commentCycles = 0;

    -- ** Initial config **
    self:Reset();
    self:UpdateSize();
    self:Show();
end

function ScoreTotal:OnUpdate(elapsed)
    self.commentTimer = max(0, self.commentTimer - elapsed);
    self.comment:SetAlpha(0.5 * (1 + cos(self.commentTimer * (360 * self.commentCycles + 180) / self.commentTimerMax)));
end