local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Resources = Root:GetOrNewModule("Resources");

local BossBarToken = GUI:GetOrNewWidget("BossBarToken");

BossBarToken.TEMPLATE = "BossEncounter3_BossBarTokenTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Boss bar token is a mini-health bar below the boss bar.

-- --------------------------------------------------------------------
-- **                            Constants                           **
-- --------------------------------------------------------------------

local SHOW_TIMER = 0.5;
local HIDE_TIMER = 0.5;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

function BossBarToken:Enable()
    if not self:IsShown() then
        self.showTimer = SHOW_TIMER;
        self.hideTimer = 0;
        self:Show();

elseif self.hideTimer > 0 then
        self.showTimer = (1 - self.hideTimer / HIDE_TIMER) * SHOW_TIMER;
        self.hideTimer = 0;
    end
end

function BossBarToken:HideUp()
    if self.hideTimer == 0 then
        self.hideTimer = HIDE_TIMER;
        self.hideDirection = 1;
    end
end

function BossBarToken:HideDown()
    if self.hideTimer == 0 then
        self.hideTimer = HIDE_TIMER;
        self.hideDirection = -1;
    end
end

function BossBarToken:SetSpecial(special)
    local textureName = "BOSSBAR_TOKEN";
    if special then textureName = "BOSSBAR_TOKENSPECIAL"; end
    self.blink = special;

    local _, tokenFilename, tokenWidth, tokenHeight = Resources:GetTextureInfo(textureName);
    self.texture:SetTexture(tokenFilename);
    self.texture:SetWidth(tokenWidth);
    self.texture:SetHeight(tokenHeight);
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function BossBarToken:OnLoad()
    local _, tokenFilename, tokenWidth, tokenHeight = Resources:GetTextureInfo("BOSSBAR_TOKEN");

    -- ** Children **
    self.texture = self:CreateTexture(nil, "ARTWORK");
    self.texture:SetTexture(tokenFilename);
    self.texture:SetWidth(tokenWidth);
    self.texture:SetHeight(tokenHeight);

    -- ** Properties **
    self.showTimer = 0;
    self.hideTimer = 0;
    self.hideDirection = 0;
    self.blink = false;

    -- ** Initial config **
    self:SetSpecial(false);
    self:Show();
end

function BossBarToken:OnUpdate(elapsed)
    local alpha = 1;
    local offset = 0;

    if self.showTimer > 0 then
        self.showTimer = max(0, self.showTimer - elapsed);

        local progression = 1 - self.showTimer / SHOW_TIMER;
        alpha = progression;

        offset = -20 * (1 - progression);

elseif self.hideTimer > 0 then
        self.hideTimer = max(0, self.hideTimer - elapsed);

        local progression = 1 - self.hideTimer / HIDE_TIMER;
        alpha = 1 - progression;
        offset = 20 * progression * self.hideDirection;

        if self.hideTimer == 0 then
            self:Hide();
        end

elseif self.blink then
        alpha = 0.50 * cos(GetTime() * 360) + 0.75;
    end

    self.texture:SetPoint("CENTER", self, "CENTER", 0, offset);
    self:SetAlpha(alpha);
end