local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");

local ActionSystem = GUI:GetOrNewWidget("ActionSystem");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Give the inheriting frame methods to have macro commands.

-- The downside is that a frame inheriting of this system will be
-- protected and locked during combat.

-- Adds:
-- ------------------------
-- ClearAction()
-- SetAction()
-- SetTargetAction()
-- UpdateAction()         -- needs to be called on each OnUpdate.

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Clear the macro to be executed when clicking on the widget.      *
-- ********************************************************************
local function ClearAction(self)
    self.secureAction = "";
    self.secureUpdate = true;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> macroText: macro text that will be run.                       *
-- ********************************************************************
-- * Set the macro to be executed when clicking on the widget.        *
-- ********************************************************************
local function SetAction(self, macroText)
    self.secureAction = macroText;
    self.secureUpdate = true;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> unitName: the name of the unit to target.                     *
-- ********************************************************************
-- * Set the target macro to be executed when clicking on the widget. *
-- ********************************************************************
local function SetTargetAction(self, unitName)
    self:SetAction("/targetexact "..unitName);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> unitName: the name of the unit to target.                     *
-- ********************************************************************
-- * Set the target macro to be executed when clicking on the widget. *
-- ********************************************************************
local function UpdateAction(self)
    if self.secureUpdate and not InCombatLockdown() then
        local button = self.secureActionButton;
        if not button then
            button = CreateFrame("Button", nil, self, "SecureActionButtonTemplate");
            button:SetAttribute("*type*", "macro");
            button:SetAttribute("*macrotext*", "");
            button:SetAllPoints(self);
            button:Show();
            self.secureActionButton = button;
        end
        button:SetAttribute("*macrotext*", self.secureAction);
        self.secureUpdate = false;
    end
end

-- --------------------------------------------------------------------
-- **                              API                               **
-- --------------------------------------------------------------------

function ActionSystem.Install(self)
    -- Install properties
    self.secureAction = "";
    self.secureUpdate = false;

    -- Install methods
    self.ClearAction = ClearAction;
    self.SetAction = SetAction;
    self.SetTargetAction = SetTargetAction;
    self.UpdateAction = UpdateAction;

    return true;
end