local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");

local MenuSystem = GUI:GetOrNewWidget("MenuSystem");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Give the inheriting frame a right-click menu dropdown.
-- Other systems can register their commands on it.

-- Adds:
-- ------------
-- RegisterCommandSet()

-- --------------------------------------------------------------------
-- **                             Locals                             **
-- --------------------------------------------------------------------

local NULL_FUNC = function() end;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> initFunction: the initialization function that will be called *
-- * when initializing the commands of this command set on dropdown.  *
-- ********************************************************************
-- * Add a command set to the frame's menu.                           *
-- * A command set is a dropdown init function which explicitly adds  *
-- * new buttons on the dropdown. They are executed in the order they *
-- * were registered.                                                 *
-- ********************************************************************
local function RegisterCommandSet(self, initFunction)
    if type(initFunction) ~= "function" then
        error("RegisterCommandSet: initFunction argument is not a function.");
    end
    tinsert(self.menuCmdSets, initFunction);
end

-- --------------------------------------------------------------------
-- **                              API                               **
-- --------------------------------------------------------------------

function MenuSystem.Install(self)
    if self:GetName() then
        -- Install properties
        self.menuCmdSets = { };

        -- Install methods
        self.RegisterCommandSet = RegisterCommandSet;

        -- Install handlers
        if self:GetScript("OnClick") then
            self:HookScript("OnClick", MenuSystem.OnClick);
      else
            self:SetScript("OnClick", MenuSystem.OnClick);
        end

        -- Install children
        self.dropDown = CreateFrame("Frame", self:GetName().."DropDown", self, "UIDropDownMenuTemplate");
        UIDropDownMenu_Initialize(self.dropDown, MenuSystem.InitializeDropDown, "MENU");

        -- Make sure the frame is clickable, so it can generate OnClick events.
        self:RegisterForClicks("RightButtonDown");
        self:Enable();
        return true;
  else
        Root:Debug("Tried to install Menu on a frame with no name.");
        return false;
    end
end

function MenuSystem:OnClick(button)
    if button == "RightButton" then
        HideDropDownMenu(1);
        ToggleDropDownMenu(1, nil, self.dropDown, "cursor");
        PlaySound("igMainMenuOpen");
    end
end

function MenuSystem.InitializeDropDown()
    local i, cmdSet;
    local frame = UIDropDownMenu_GetCurrentDropDown():GetParent();

    for i=1, #frame.menuCmdSets do
        cmdSet = frame.menuCmdSets[i];
        cmdSet(frame);
    end

    -- Always add the close command at the end
    if UIDROPDOWNMENU_MENU_LEVEL == 1 then
        local info = UIDropDownMenu_CreateInfo();
        info.text  = CLOSE;
        info.func  = NULL_FUNC;
        info.notCheckable = true;
        UIDropDownMenu_AddButton(info);
    end
end