local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");

local NotifySystem = GUI:GetOrNewWidget("NotifySystem");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Give the inheriting frame methods to call a handler from its parent
-- to notify a task has been completed etc.

-- Adds:
-- ------------------------
-- Notify(handlerName, ...)

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> handlerName: the name of the handler to call.                 *
-- * >> ...: arguments for the call.                                  *
-- ********************************************************************
-- * Invoke an handler from the parent of the object, if it exists.   *
-- ********************************************************************
local function Notify(self, handlerName, ...)
    local parent = self:GetParent();
    if parent then
        local handler = parent[handlerName];
        if handler then
            handler(parent, ...);
        end
    end
end

-- --------------------------------------------------------------------
-- **                              API                               **
-- --------------------------------------------------------------------

function NotifySystem.Install(self)
    -- Install methods
    self.Notify = Notify;

    return true;
end