local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");

local Save = Root:GetOrNewModule("Save");

local SaveSystem = GUI:GetOrNewWidget("SaveSystem");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Give the inheriting frame methods to save data.

-- Adds:
-- ------------
-- ReadValue()
-- WriteValue()

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> field: the field read.                                        *
-- ********************************************************************
-- * Read a saved value for the frame.                                *
-- ********************************************************************
local function ReadValue(self, field)
    return self._save[field];
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> field: the field read.                                        *
-- * >> value: the value we wish to write.                            *
-- ********************************************************************
-- * Write a saved value for the frame.                               *
-- ********************************************************************
local function WriteValue(self, field, value)
    self._save[field] = value;
end

-- --------------------------------------------------------------------
-- **                              API                               **
-- --------------------------------------------------------------------

function SaveSystem.Install(self)
    if not self._save then
        -- Check the frame is named
        local name = self:GetName();
        local guid = UnitGUID("player");
        if name and guid then
            local id = name..":"..guid;

            -- Install properties
            self._save = Save:Get("gui", id) or { };
            Save:Set("gui", id, self._save);

            -- Install methods
            self.ReadValue = ReadValue;
            self.WriteValue = WriteValue;

            return true;
        end
    end
    return false;
end