local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Resources = Root:GetOrNewModule("Resources");
local Audio = Root:GetOrNewModule("Audio");
local Utility = Root:GetOrNewModule("Utility");
local L = Root:GetOrNewModule("Localization");

local MusicRow = GUI:GetOrNewWidget("MusicRow");

MusicRow.TEMPLATE = "BossEncounter3_MusicRowTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- MusicRow is a GUI element allowing to enable/preview a plugin music.

-- --------------------------------------------------------------------
-- **                            Constants                           **
-- --------------------------------------------------------------------

local TITLE_SCALE = 0.5;
local DURATION_SCALE = 0.5;
local BUTTON_SCALE = 0.6;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> pluginIndex: the index of the music plugin.                   *
-- * >> musicIndex: the index of the music within the plugin.         *
-- ********************************************************************
-- * Set up the music row.                                            *
-- ********************************************************************
function MusicRow:Setup(pluginIndex, musicIndex)
    self.plugin = pluginIndex;
    self.music = musicIndex;

    self:Update();
end

-- ********************************************************************
-- * Update the music row.                                            *
-- ********************************************************************
function MusicRow:Update()
    local name, file, category, duration, _, enabled = Resources:GetPluginMusic(self.plugin, self.music);

    if name then
        if category == "HURRYUP" or category == "NEARVICTORY" or not enabled then
            self.playButton:Disable();
      else
            self.playButton:Enable();
        end

        self.title:SetText(name);
        self.title:SetColor(1, 1, 1);
        self.duration:SetText( Utility:FormatCountdown("%M'%S", duration) );
        self.duration:SetColor(1, 1, 1);
        self.playButton.name = name;
        self.playButton:Show();
        self.useButton:SetChecked(enabled);
        self.useButton:Show();
  else
        self.title:SetText(L:Get("Plugin-Title"));
        self.title:SetColor(0, 1, 1);
        self.duration:SetText(L:Get("Plugin-Duration"));
        self.duration:SetColor(0, 1, 1);
        self.playButton:Hide();
        self.useButton:Hide();
    end

    local titleOfs, durationOfs, playOfs, useOfs = self:GetParent():GetOffsets();

    self.title:SetPoint("LEFT", self, "LEFT", titleOfs / TITLE_SCALE, 0);
    self.duration:SetPoint("LEFT", self, "LEFT", durationOfs / DURATION_SCALE, 0);
    self.playButton:SetPoint("LEFT", self, "LEFT", playOfs / BUTTON_SCALE, 0);
    self.useButton:SetPoint("LEFT", self, "LEFT", useOfs / BUTTON_SCALE, 0);

    self:SetWidth(useOfs + self.useButton:GetWidth() / BUTTON_SCALE + titleOfs);
    self:SetHeight(16);
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function MusicRow:OnLoad()
    -- ** Children **
    self.title = GUI:RequestChild("MonoFont", nil, self, 1);
    self.title:SetScale(TITLE_SCALE);
    self.duration = GUI:RequestChild("MonoFont", nil, self, 1);
    self.duration:SetScale(DURATION_SCALE);
    self.playButton = CreateFrame("Button", nil, self, "OptionsButtonTemplate");
    self.playButton:SetWidth(80);
    self.playButton:SetHeight(24);
    self.playButton:SetScale(BUTTON_SCALE);
    self.playButton:SetText(L:Get("Plugin-Play"));
    self.playButton:SetScript("OnClick", self.OnPlayButtonClick);
    self.playButton:Show();
    self.useButton = CreateFrame("CheckButton", self:GetName().."_UseButton", self, "BossEncounter3_CheckButtonTemplate");
    self.useButton:SetScale(BUTTON_SCALE);
    self.useButton:SetText(L:Get("Plugin-Use"));
    self.useButton:SetScript("OnClick", self.OnUseButtonClick);
    self.useButton:Show();

    -- ** Properties **
    self.plugin = 0;
    self.music = 0;
end

function MusicRow.OnPlayButtonClick(button)
    Audio:PlayMusicOnce(button.name);
end

function MusicRow.OnUseButtonClick(button)
    local parent = button:GetParent();
    if button:GetChecked() then
        PlaySound("igMainMenuOptionCheckBoxOn");
        Resources:SetPluginMusicState(parent.plugin, parent.music, true);
  else
        PlaySound("igMainMenuOptionCheckBoxOff");
        Resources:SetPluginMusicState(parent.plugin, parent.music, false);
    end
    parent:Update();
end