local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Resources = Root:GetOrNewModule("Resources");
local Audio = Root:GetOrNewModule("Audio");

local Popup = GUI:GetOrNewWidget("Popup");

Popup.TEMPLATE = "BossEncounter3_PopupTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Display a popup.

-- --------------------------------------------------------------------
-- **                            Constants                           **
-- --------------------------------------------------------------------

local TITLE_SCALE = 1.00;
local SCROLL_TIMER = 1.00;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> type: the type of popup. Can be: NORMAL, WARNING.             *
-- * >> title: the title of the popup.                                *
-- * >> text: the content of the popup.                               *
-- ********************************************************************
-- * Open the popup.                                                  *
-- ********************************************************************
function Popup:Open(type, title, text)
    if type == "WARNING" then
        local _, filename, width, height = Resources:GetTextureInfo("WARNING");
        self.icon:SetTexture(filename);
        self.icon:SetWidth(width);
        self.icon:SetHeight(height);
        self.icon:Show();
        self.text:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 72, 8);
  else
        self.icon:Hide();
        self.text:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 40, 8);
    end

    self.title:SetText(title);
    self.text:SetText(text);

    self:Show();
    Audio:PlaySound("POPUP");
end

-- ********************************************************************
-- * Close the popup.                                                 *
-- ********************************************************************
function Popup:Close()
    self:Hide();
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function Popup:OnLoad()
    -- ** Children **
    self.title = GUI:RequestChild("MonoFont", nil, self, 1);
    self.title:SetPoint("TOP", self, "TOP", 0, 0);
    self.title:SetScale(TITLE_SCALE);
    self.icon = self:CreateTexture(nil, "ARTWORK");
    self.icon:ClearAllPoints();
    self.icon:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 8, 0);
    self.text = _G[self:GetName().."_Text"];
    self.text:ClearAllPoints();
    self.band = GUI:RequestChild("Band", nil, self, 0);
    self.band:SetAllPoints(self);
    self.band:SetGraphic("BAND_HS");
    self.closeButton = CreateFrame("Button", nil, self, "UIPanelCloseButton");
    self.closeButton:SetScript("OnClick", self.OnCloseButtonClick);
    self.closeButton:SetPoint("TOPRIGHT", self, "TOPRIGHT", -4, 4);

    -- ** Properties **
    self.scroll = 0;
end

function Popup:OnShow()
    self.scroll = SCROLL_TIMER;
end

function Popup:OnUpdate(elapsed)
    self.scroll = max(0, self.scroll - elapsed);

    local parent = self:GetParent();
    local progression = self.scroll / SCROLL_TIMER;
    local alpha = sin( math.fmod( 90 + (progression) * 450 , 180) );

    self:SetPoint("BOTTOM", parent, "BOTTOM", 128 * progression, parent:GetHeight() * 0.25);
    self:SetAlpha(alpha);
end

function Popup:OnCloseButtonClick()
    self:GetParent():Close();
end