local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");

local List = GUI:GetOrNewWidget("List");

List.TEMPLATE = "BossEncounter3_ListTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- List is a GUI primitive element that allows one to display rows
-- of similar slave elements.

-- Number of rows is not limited in itself.
-- It is very important to note that elements should be
-- of the same type, or at the very least, the same dimensions.

-- --------------------------------------------------------------------
-- **                            Constants                           **
-- --------------------------------------------------------------------

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Check if the size is empty.                                      *
-- ********************************************************************
function List:IsEmpty()
    return (self:GetSize() == 0);
end

-- ********************************************************************
-- * Get the size of the list.                                        *
-- ********************************************************************
function List:GetSize()
    return #self.order;
end

-- ********************************************************************
-- * Insert an element into the list.                                 *
-- ********************************************************************
function List:Insert(element)
    tinsert(self.order, element);
    GUI:Show(element);
    self:Update();
end

-- ********************************************************************
-- * Remove an element from the list.                                 *
-- ********************************************************************
function List:Remove(element)
    local i;
    for i=#self.order, 1, -1 do
        if self.order[i] == element then
            tremove(self.order, i);
        end
    end
    GUI:Hide(element);
    self:Update();
end

-- ********************************************************************
-- * Clear elements of the list.                                      *
-- ********************************************************************
function List:Clear()
    local i;
    for i=#self.order, 1, -1 do
        GUI:Hide(self.order[i]);
    end
    wipe(self.order);
    self:Update();
end

-- ********************************************************************
-- * This method updates the ordering of "order" field, to readjust   *
-- * elements within the list.                                        *
-- ********************************************************************
function List:Sort(sortFunc)
    table.sort(self.order, sortFunc);
    self:Update();
end

-- ********************************************************************
-- * Update the position of each element to match the "order" field.  *
-- ********************************************************************
function List:Update()
    local h = 0;
    local i, element;
    for i=1, #self.order do
        element = self.order[i];
        element:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -h);
        h = h + self.spacing;
        h = h + element:GetHeight() + self.spacing;
    end
    self:UpdateSize();
end

-- ********************************************************************
-- * Update the size of the list.                                     *
-- ********************************************************************
function List:UpdateSize()
    local w, h = 0, 0;
    local i, element;
    for i=1, #self.order do
        element = self.order[i];
        w = max(element:GetWidth(), w);
        if i > 1 then
            h = h + self.spacing;
        end
        h = h + element:GetHeight();
    end
    self:SetWidth(w);
    self:SetHeight(h);
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function List:OnLoad()
    -- ** Children **
    self.order = { };

    -- ** Properties **
    self.spacing = 1;

    -- ** Initial config **
    self:Update();
    self:Show();
end

function List:OnUpdate(elapsed)

end