local Root = BossEncounter3;
local GUI = Root:GetOrNewModule("GUI");
local Utility = Root:GetOrNewModule("Utility");

local SizeSlider = GUI:GetOrNewWidget("SizeSlider");

SizeSlider.TEMPLATE = "BossEncounter3_SizeSliderTemplate";

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Size slider is the widget that is invoked when the user chooses the
-- resize command on widgets' dropdown menu.

-- --------------------------------------------------------------------
-- **                            Constants                           **
-- --------------------------------------------------------------------

local MIN_VALUE  = 0.50;
local MAX_VALUE  = 1.50;
local STEP_VALUE = 0.05;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> target: the frame whose size is edited.                       *
-- * >> ref: the frame that invoked the size slider.                  *
-- *         Typically this will be a dropdown. Can be nil.           *
-- ********************************************************************
-- * Open the size slider to alter a compatible frame.                *
-- ********************************************************************
function SizeSlider:Open(target, ref)
    if self:SetTarget(target) then
        if ref then
            self:SetPoint("CENTER", UIParent, "BOTTOMLEFT", ref:GetCenter());
      else
            self:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
        end
        self:Show();
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> target: the new target.                                       *
-- ********************************************************************
-- * Designate the frame that will be resized by the size slider.     *
-- ********************************************************************
function SizeSlider:SetTarget(target)
    if target and target.ChangeScale then
        self.target = nil;
        self.scaleSlider:SetValue(target:GetScale());
        self.target = target;
        return true;
  else
        return false;
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> value: the new value.                                         *
-- ********************************************************************
-- * Set the new scale value.                                         *
-- ********************************************************************
function SizeSlider:SetScaleValue(value)
    if self.target then
        self.target:ChangeScale(value);
    end
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function SizeSlider:OnLoad()
    -- ** Children **
    self.scaleSlider = CreateFrame("Slider", self:GetName().."_ScaleSlider", self, "BossEncounter3_SliderTemplate");
    self.scaleSlider:SetMinMaxValues(MIN_VALUE, MAX_VALUE);
    self.scaleSlider:SetValueStep(STEP_VALUE);
    self.scaleSlider:SetLowText((MIN_VALUE*100).."%");
    self.scaleSlider:SetHighText((MAX_VALUE*100).."%");
    self.scaleSlider:SetScript("OnValueChanged", self.OnScaleSliderValueChanged);
    self.scaleSlider:SetValue(0);
    self.scaleSlider:SetPoint("TOP", self, "TOP", 0, -8);
    self.closeButton = CreateFrame("Button", nil, self, "OptionsButtonTemplate");
    self.closeButton:SetPoint("BOTTOM", self, "BOTTOM", 0, 8);
    self.closeButton:SetWidth(64);
    self.closeButton:SetHeight(24);
    self.closeButton:SetText(CLOSE);
    self.closeButton:SetScript("OnClick", self.OnCloseButtonClick);
    self.closeButton:Show();

    -- ** Properties **
    self.target = nil;

    -- ** Initial config **
    Utility:SetBackdrop(self, "OPTION");
    self:SetClampedToScreen(true);
end

function SizeSlider:OnShow()
    -- TODO
end

function SizeSlider.OnCloseButtonClick(button)
    button:GetParent():Hide();
end

function SizeSlider.OnScaleSliderValueChanged(slider)
    local value = slider:GetValue();
    local valueText = ("%d%%"):format(value * 100);
    slider:SetValueText(valueText);
    slider:GetParent():SetScaleValue(value);
end
