﻿-- --------------------------------------------------------------------
-- ////////////////////////////////////////////////////////////////////
-- --                           INSTALLER                            --
-- \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
-- --------------------------------------------------------------------

-- /!\ This file has to be loaded first.

BossEncounter3 = CreateFrame("Frame", "BossEncounter3");
BossEncounter3:SetScript("OnEvent", function(self, ...) self:OnEvent(...) end);
BossEncounter3:SetScript("OnUpdate", function(self, ...) self:OnUpdate(...) end);
BossEncounter3:Show();

local Root = BossEncounter3;

-- These properties are present in all cores.
Root.moduleList = { };
Root.coreList = { };
Root.printPrefix = "|cff00ff00BossEncounter3|r - ";
Root.debug = false;

-- These are specific to the main core.
Root.folder = "Interface\\AddOns\\BossEncounter3\\";
Root.started = false;
Root.entered = false;
Root.booted = false;
Root.invalid = false;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Return true if the mod should not be working.                    *
-- ********************************************************************
function Root:IsInvalid()
    return self.invalid or false;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> moduleName: the name of the module.                           *
-- * >> debug: set whether to enable debugging or not.                *
-- ********************************************************************
-- * Build a sub-core object, getting the same methods as core (minus *
-- * some methods and properties). Sub-cores will automatically       *
-- * propagate events on their own modules.                           *
-- ********************************************************************
function Root:NewSubCore(moduleName, debug)
    local t = self:GetOrNewModule(moduleName);
    tinsert(self.coreList, moduleName);

    -- Methods
    t.GetOrNewModule = self.GetOrNewModule;
    t.InvokeHandler = self.InvokeHandler;
    t.InsertToModule = self.InsertToModule;
    t.Print = self.Print;
    t.PrintFmt = self.PrintFmt;
    t.Debug = self.Debug;
    t.DebugFmt = self.DebugFmt;
    t.EnableDebug = self.EnableDebug;
    t.DisableDebug = self.DisableDebug;

    t._delegateEvent = function(self, handlerName, ...) self:InvokeHandler(handlerName, ...) end;

    -- Properties
    t.moduleList = { };
    t.coreList = { };
    t.printPrefix = self.printPrefix;
    t.debug = self.debug;
    if debug ~= nil then
        t.debug = debug;
    end

    return t;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> moduleName: the name of the module.                           *
-- ********************************************************************
-- * Gets an existing module's table object or create it if it does   *
-- * not exist. Whatever this function does, it'll return the table.  *
-- ********************************************************************
function Root:GetOrNewModule(moduleName)
    if type(self[moduleName]) == "table" then
        return self[moduleName];
  else
        self[moduleName] = { };
        tinsert(self.moduleList, moduleName);
        return self[moduleName];
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> handlerName: the name of the handler to call.                 *
-- * >> ...: arguments to pass to the handler.                        *
-- ********************************************************************
-- * Invoke a given handler on all registered modules.                *
-- ********************************************************************
function Root:InvokeHandler(handlerName, ...)
    local coreList = self.coreList;
    local moduleList = self.moduleList;
    local i, m;
    for i=1, #moduleList do
        m = self[moduleList[i]];
        if m then
            h = m[handlerName];
            if h then
                h(m, ...);
            end
        end
    end
    for i=1, #coreList do
        m = self[coreList[i]];
        if m then
            m:_delegateEvent(handlerName, ...);
        end
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> insert: the table that has its content inserted into target.  *
-- * >> moduleName: the name of the target module.                    *
-- ********************************************************************
-- * Add the content of a table into a module's table.                *
-- ********************************************************************
function Root:InsertToModule(insert, moduleName)
    local module = self:GetOrNewModule(moduleName);
    local k, v;
    for k, v in pairs(insert) do
        if not module[k] then
            module[k] = v;
        end
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> txt: the message to print.                                    *
-- ********************************************************************
-- * Print a message.                                                 *
-- ********************************************************************
function Root:Print(txt)
    local partString = self.printPrefix or '';
    local text = partString .. txt;
    if ( not DEFAULT_CHAT_FRAME ) then
        print(text);
  else
        DEFAULT_CHAT_FRAME:AddMessage(text);
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> txt: the message to print.                                    *
-- * >> ...: format arguments.                                        *
-- ********************************************************************
-- * Print a message with format.                                     *
-- ********************************************************************
function Root:PrintFmt(txt, ...)
    self:Print(txt:format(...));
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> txt: the debug message to print.                              *
-- ********************************************************************
-- * Print a debug message.                                           *
-- ********************************************************************
function Root:Debug(txt)
    if self.debug then
        self:Print(txt);
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> txt: the debug message to print.                              *
-- * >> ...: format arguments.                                        *
-- ********************************************************************
-- * Print a debug message with format.                               *
-- ********************************************************************
function Root:DebugFmt(txt, ...)
    if self.debug then
        self:PrintFmt(txt, ...);
    end
end

-- ********************************************************************
-- * Get the mod folder.                                              *
-- ********************************************************************
function Root:GetFolder()
    return self.folder;
end

-- ********************************************************************
-- * Set the debug flag.                                              *
-- ********************************************************************
function Root:EnableDebug(echo)
    if echo and (not self.debug) then
        Root:Print("Debug enabled.");
    end
    self.debug = true;

    local coreList = self.coreList;
    local i, m;
    for i=1, #coreList do
        m = self[coreList[i]];
        m:EnableDebug(false);
    end
end

-- ********************************************************************
-- * Clear the debug flag.                                            *
-- ********************************************************************
function Root:DisableDebug(echo)
    if echo and self.debug then
        Root:Print("Debug disabled.");
    end
    self.debug = false;

    local coreList = self.coreList;
    local i, m;
    for i=1, #coreList do
        m = self[coreList[i]];
        m:DisableDebug(false);
    end
end

-- --------------------------------------------------------------------
-- **                            Handlers                            **
-- --------------------------------------------------------------------

function Root:OnLoad()
    self:RegisterEvent("VARIABLES_LOADED");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");
    self:RegisterEvent("PLAYER_LOGOUT");

    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");

    self:RegisterEvent("CHAT_MSG_MONSTER_YELL");
    self:RegisterEvent("CHAT_MSG_MONSTER_WHISPER");
    self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE");
    self:RegisterEvent("CHAT_MSG_RAID_BOSS_EMOTE");
    self:RegisterEvent("CHAT_MSG_RAID_BOSS_WHISPER");

    self:RegisterEvent("PLAYER_TARGET_CHANGED");
    self:RegisterEvent("PLAYER_FOCUS_CHANGED");
    self:RegisterEvent("UNIT_TARGET");
    self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT");
    self:RegisterEvent("UNIT_ENTERED_VEHICLE");
    self:RegisterEvent("UNIT_EXITED_VEHICLE");

    self:RegisterEvent("CHAT_MSG_ADDON");
    self:RegisterEvent("CHAT_MSG_WHISPER");
    self:RegisterEvent("CHAT_MSG_BN_WHISPER");

    self:RegisterEvent("ZONE_CHANGED");
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
    self:RegisterEvent("ZONE_CHANGED_INDOORS");
    self:RegisterEvent("WORLD_MAP_UPDATE");

    self:RegisterEvent("IGNORELIST_UPDATE");
end

function Root:OnEvent(event, ...)
    local Unit = self:GetOrNewModule("Unit");

    if event == "COMBAT_LOG_EVENT_UNFILTERED" then
        local Combat = Root.System.Combat;
        Combat:OnRawCombatEvent(...); -- Performance hack.

elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
        local unit, spellName, _, _, spellId = ...
        if unit and unit:sub(1, 4) == "boss" and spellId then
            Root:InvokeHandler("OnBossCast", unit, spellId, spellName);
        end

elseif event == "PLAYER_TARGET_CHANGED" then
        Unit:RebuildList();
        Root:InvokeHandler("OnTargetChange");

elseif event == "PLAYER_FOCUS_CHANGED" or event == "UNIT_TARGET" or event == "INSTANCE_ENCOUNTER_ENGAGE_UNIT" then
        Unit:RebuildList();

elseif event == "UNIT_ENTERED_VEHICLE" then
        Unit:RebuildList();
        local unit = ...
        if unit == "player" then
            Root:InvokeHandler("OnVehicleEnter");
        end

elseif event == "UNIT_EXITED_VEHICLE" then
        Unit:RebuildList();
        local unit = ...
        if unit == "player" then
            Root:InvokeHandler("OnVehicleLeave");
        end

elseif event == "CHAT_MSG_MONSTER_YELL" or event == "CHAT_MSG_MONSTER_WHISPER" or event == "CHAT_MSG_RAID_BOSS_EMOTE" or event == "CHAT_MSG_RAID_BOSS_WHISPER" then
        local text, npc, _, _, target = ...
        local channel = "YELL";
        if event == "CHAT_MSG_MONSTER_WHISPER"   then channel = "WHISPER"; end
        if event == "CHAT_MSG_RAID_BOSS_EMOTE"   then channel = "RAID_EMOTE"; end
        if event == "CHAT_MSG_RAID_BOSS_WHISPER" then channel = "RAID_WHISPER"; end
        Root:InvokeHandler("OnMobYell", text, npc, channel, target);

elseif event == "CHAT_MSG_MONSTER_EMOTE" then
        local text, npc, _, _, target = ...
        Root:InvokeHandler("OnMobYell", text:format(npc), npc, "EMOTE", target);

elseif event == "CHAT_MSG_ADDON" then
        Root:InvokeHandler("OnAddonMessage", ...);

elseif event == "CHAT_MSG_WHISPER" then
        Root:InvokeHandler("OnWhisper", ...);

elseif event == "CHAT_MSG_BN_WHISPER" then
        Root:InvokeHandler("OnBNWhisper", ...);

elseif event == "PLAYER_REGEN_ENABLED" then
        self:InvokeHandler("OnCombatLeave");

elseif event == "VARIABLES_LOADED" then
        self:OnStart();

elseif event == "PLAYER_ENTERING_WORLD" then
        self:OnWorldEnter();

elseif event == "PLAYER_LOGOUT" then
        Root:InvokeHandler("OnLogout");

elseif event:sub(1, 12) == "ZONE_CHANGED" then
        Root:InvokeHandler("OnZoneChange");

elseif event == "WORLD_MAP_UPDATE" then
        Root:InvokeHandler("OnMapUpdate", ...);

elseif event == "IGNORELIST_UPDATE" then
        Root:InvokeHandler("OnIgnoreChange");
    end
end

function Root:OnUpdate(elapsed)
    if not self:IsInvalid() then
        self:InvokeHandler("OnUpdate", elapsed);
    end
end

function Root:OnStart()
    self.started = true;
    self:CheckBoot();
end

function Root:OnWorldEnter()
    self.entered = true;
    self:CheckBoot();

    if self.booted then
        self:InvokeHandler("OnWorldEnter");
    end
end

function Root:CheckBoot()
    if self:IsInvalid() then
        -- Nothing for now.

elseif self.started and self.entered and not self.booted then
        self.booted = true;

        self:InvokeHandler("OnBootCompleteImportant");
        self:InvokeHandler("OnBootComplete");

        self:OnBootComplete();
    end
end

function Root:OnBootComplete()
    local L = self:GetOrNewModule("Localization");
    local Save = self:GetOrNewModule("Save");
    local Resources = self:GetOrNewModule("Resources");
    local Version = self:GetOrNewModule("Version");
    local verString = Version:GetString();
    local numPlugins = Resources:GetNumPlugins();

    self:Print(L:FormatGet("BootComplete", verString, numPlugins));

    -- Popup for first-time run.

    local key = "Known:"..verString;
    hasRead = Save:Get("system", key) or false;

    if (not hasRead) then
        Save:Set("system", key, true);

        StaticPopupDialogs["BE3_WELCOME_POPUP"] = {
            text = L:FormatLoc("FirstTimeNotice", verString),
            button1 = OKAY,
            timeout = 0,
            whileDead = 1,
            hasEditBox = 1,
            maxLetters = 32,

            OnShow = function(self)
                self.button1:Disable();
                self.editBox:SetFocus();
            end,
            OnHide = function(self)
                ChatEdit_FocusActiveWindow();
                self.editBox:SetText("");
            end,
            EditBoxOnEnterPressed = function(self)
                local parent = self:GetParent();
                if parent.button1:IsEnabled() then
                    parent:Hide();
                end
            end,
            EditBoxOnTextChanged = function(self)
                local parent = self:GetParent();
                local text = strupper(parent.editBox:GetText());
                if text == "/BE3" or text == "BE3" then -- BE3 for n00bs that don't know how to type '/' x]
                    parent.button1:Enable();
                else
                    parent.button1:Disable();
                end
            end,
        };

        StaticPopup_Show("BE3_WELCOME_POPUP");
    end

    -- Refresh plugins

    Resources:RefreshPlugins();
end

-- --------------------------------------------------------------------
-- **                            Startup                             **
-- --------------------------------------------------------------------

-- Fire the OnLoad handler.

do
    Root:OnLoad();
end