﻿local Root = BossEncounter3;
local QuickModule = Root:GetOrNewModule("QuickModule");

-- ================================
-- Define here extra quick modules.
-- ================================

-- Boss target dummy
QuickModule:AddDescriptionTable( {
    boss = 31146,
    environment = "SOLO",

    timeout = 45,
    ignoreStandardClear = true,
    ignoreLeaveCombat = true,
    ignoreWipe = true,
    ignoreScore = true,

    timers = {
        { id = "TrainingEnd",
          category = "ooc",
          default = 60, -- will be overriden
          finish = true,
          callback = "OnCleared",
        },
    },

    settings = {
        {
          id = "Duration",
          type = "NUMBER",
          default = 180,

          min = 15,
          max = 900,
          step = 15,

          label = {
              ["enUS"] = "Duration",
              ["frFR"] = "Durée",
          },
          explain = {
              ["enUS"] = "How long the training will last.",
              ["frFR"] = "Spécifie la durée de l'entraînement.",
          },
        },
    },

    locale = {
        ["enUS"] = {
            ["Title"] = "Training",
            ["TrainingEnd"] = "End",
        },
        ["frFR"] = {
            ["Title"] = "Entraînement",
            ["TrainingEnd"] = "Fin",
        },
    },

    extra = {
        OnEngaged = function(self)
            self:StartTimer("TrainingEnd", self:GetSetting("Duration"));
        end,

        OnUpdate = function(self, elapsed)
            -- Deal with the unusual way training dummies are engaged.
            if self:GetState() == "STANDBY" and UnitAffectingCombat("player") then
                self:OnEngaged();
            end
        end,
    },
} );

-- Burst test
--[[
QuickModule:AddDescriptionTable( {
    boss = 31144,
    environment = "SOLO",

    timeout = 30,
    ignoreStandardClear = true,
    ignoreLeaveCombat = true,
    ignoreWipe = true,
    ignoreScore = true,

    timers = {
        { id = "TrainingEnd",
          category = "ooc",
          first = 10,
          default = 10,
          finish = true,
          callback = "OnCleared",
        },
    },

    locale = {
        ["enUS"] = {
            ["Title"] = "Burst",
            ["TrainingEnd"] = "End",
        },
        ["frFR"] = {
            ["Title"] = "Burst",
            ["TrainingEnd"] = "Fin",
        },
    },

    extra = {
        OnUpdate = function(self, elapsed)
            -- Deal with the unusual way training dummies are engaged.
            if self:GetState() == "STANDBY" and UnitAffectingCombat("player") then
                self:OnEngaged();
            end
        end,
    },
} );
]]

-- Whale Shark
QuickModule:AddDescriptionTable( {
    boss = 40728,
    environment = "SOLO",
    timeout = 45,
    importance = 90,
} );

-- Debug module
--[[
QuickModule:AddDescriptionTable( {
    boss = 116,
    importance = 5,

    timers = {
        { id = "Blabla",
          category = "all/critical",
          first = 8,
          default = 12,
          loop = true,
          autoTrigger = true,
          callback = "OnSomething",
        },
    },

    zones = {
        { id = "Stuff",
          category = "all/critical",
          color = "red",
          duration = 4,
          radius = 8,
          type = "aimed",
        },
    },

    extra = {
        OnSomething = function(self)
            self:ShowZone("Stuff", "player");
        end,
    },
} );
]]