﻿local Root = BossEncounter3;
local QuickModule = Root:GetOrNewModule("QuickModule");

-- =======================================
-- Define here quick modules of Cataclysm.
-- =======================================

-- I. Throne of the Tides
--   1. Lady Naz'jar
QuickModule:AddDescriptionTable( {
    boss = 40586,
    mooks = {40633}, -- Naz'jar Honor Guard
    timeCoeff = 1.60,
    importance = 25,

    thresholds = {
        { value = 0.600,
          description = "Shield",
        },
        { value = 0.300,
          description = "Shield",
        },
    },

    timers = {
        { id = "ShieldEnd",
          category = "all/high",
          default = 60,
        },
    },

    zones = {
        { id = "LightningSurge",
          category = "all/high",
          color = "red",
          duration = 5,
          radius = 10,
          type = "anchored",
        },
    },

    spells = {
        { id = 76008, -- Shock Blast
          trigger = "cast",
          cooldown = 5,
          callback = "OnShock",
        },

        -- Lightning Surge
        { id = 75992,
          trigger = "effectgain",
          zone = "LightningSurge",
          callback = "OnLightningSurge",
        },
        { id = 75992,
          trigger = "effectfade",
          zone = "LightningSurge",
        },

        -- Water Shield
        { id = 75683,
          trigger = "effectgain",
          startTimer = "ShieldEnd",
        },
        { id = 75683,
          trigger = "effectfade",
          finishTimer = "ShieldEnd",
        },
    },

    locale = {
        ["enUS"] = {
            ["Shield"] = "Shield",
            ["ShieldEnd"] = "Shield: end",
        },
        ["frFR"] = {
            ["Shield"] = "Bouclier",
            ["ShieldEnd"] = "Fin du bouclier",
        },
    },

    extra = {
        OnShock = function(self)
            self:InterruptMob(nil, "critical");
        end,

        OnLightningSurge = function(self, guid)
            self:Bomb(guid);
        end
    },
} );

--   2. Commander Ulthok
QuickModule:AddDescriptionTable( {
    boss = 40765,
    importance = 20,

    timers = {
        { id = "EnrageEnd",
          category = "all/normal",
          default = 10,
        },
    },

    spells = {
        { id = 76047, -- Fissure
          trigger = "cast",
          callback = "OnFissure",
        },
        { id = 76026, -- Squeeze
          trigger = "effectgain",
          cooldown = 3,
          callback = "OnSqueeze",
        },

        -- Enrage
        { id = 76100,
          trigger = "effectgain",
          cooldown = 8,
          startTimer = "EnrageEnd",
          callback = "OnEnrage",
        },
        { id = 76100,
          trigger = "effectfade",
          finishTimer = "EnrageEnd",
        },
    },

    settings = {
        {
          id = "Squeeze",
          type = "BOOLEAN",
          action = "SYMBOL",
          default = false,
          exclusive = true,
          meta = { spell = 76026 },
        },
    },

    locale = {
        ["enUS"] = {
            ["EnrageEnd"] = "Enrage: end",
            ["Enrage"] = "Boss enraged!",
        },
        ["frFR"] = {
            ["EnrageEnd"] = "Fin de l'enrager",
            ["Enrage"] = "Le boss est enragé !",
        },
    },

    extra = {
        OnEnrage = function(self, guid)
            self:AlertCategorized("shield/normal", self:Localize("Enrage"));
        end,

        OnSqueeze = function(self, guid)
            if self:GetSetting("Squeeze") then
                self:AnnounceUnitSpell(76026, guid);
            end
        end,

        OnFissure = function(self)
            self:AlertIncomingSpell("tank/normal", 76047);
        end,
    },
} );

--   3. Erunak Stonespeaker
QuickModule:AddDescriptionTable( {
    boss = 40825, -- Erunak Stonespeaker
    adds = {40788}, -- Mindbender Ghur'sha
    timeCoeff = 2.00,
    importance = 35,

    ignoreStandardClear = true,

    thresholds = {
        { value = 0.505,
          description = "ErunakReleased",
        },
    },

    spells = {
        { id = 76207,
          trigger = "effectgain",
          cooldown = 5,
          callback = "OnEnslave",
        },
    },

    settings = {
        {
          id = "Enslave",
          type = "BOOLEAN",
          action = "SYMBOL",
          default = false,
          exclusive = true,
          meta = { spell = 76207 },
        },
    },

    locale = {
        ["enUS"] = {
            ["ErunakReleased"] = "Erunak released",
        },
        ["frFR"] = {
            ["ErunakReleased"] = "Erunak libéré",
        },
    },

    extra = {
        OnEnslave = function(self, guid)
            if self:GetSetting("Enslave") then
                self:AnnounceUnitSpell(76207, guid);
            end
        end,

        OnAddDeath = function(self, guid, id, name)
            if id == 40788 then
                self:OnCleared(); -- Success!
            end
        end,
    },
} );

--   4. Ozumat
QuickModule:AddDescriptionTable( {
    boss = 44566, -- Ozumat
    adds = {44648, 40792}, -- Behemoth, Neptulon
    importance = 50,

    bypass = true, -- Neptulon triggers the module, he is friendly.
    timeout = 300,
    ignoreStandardClear = true,
    ignoreLeaveCombat = true,
    greatTime = 240,
    poorTime = 320,
 
    spells = {
        { id = 83463,
          trigger = "effectgain",
          callback = "OnPhaseTwo",
        },
        { id = 76133,
          trigger = "effectgain",
          callback = "OnPhaseThree",
        },
    },

    locale = {
        ["enUS"] = {
            ["Title"] = "Ozumat",
            ["EngageTrigger"] = "As I purify these waters",
        },
        ["frFR"] = {
            ["Title"] = "Ozumat",
            ["EngageTrigger"] = "La purification de ces eaux",
        },
    },

    extra = {
        OnEngaged = function(self)
            self:SetPhase(1);
        end,

        OnPhaseTwo = function(self)
            self:SetIncreasingPhase(2);
        end,

        OnPhaseThree = function(self)
            self:SetIncreasingPhase(3);
        end,

        OnUpdate = function(self, elapsed)
            if self:GetState() == "STANDBY" then
                local guid, uid = self:GetMobGUID(40792);
                if uid then
                    if UnitHealth(uid) < UnitHealthMax(uid) then
                        self:OnEngaged(); -- Saving throw for unsupported languages.
                    end
                end
          else
                local current, max, ratio = self:GetBossHealth();
                if ratio <= 0.10 and max > 0 then
                    self:OnCleared(); -- Success!
                end
            end
        end,
    },
} );

-- II. Blackrock Caverns
--   1. Rom'ogg Bonecrusher
QuickModule:AddDescriptionTable( {
    boss = 39665,
    importance = 10,

    timers = {
        { id = "Skullcracker",
          category = "all/critical",
          spell = 75543,
          default = 12,
        },
    },

    zones = {
        { id = "Impact",
          category = "all/critical",
          color = "red",
          duration = 12,
          radius = 8,
          type = "aimed",
        },
    },

    spells = {
        { id = 75543, -- Skullcracker
          trigger = "cast",
          callback = "OnSkullcracker",
        },
    },

    extra = {
        OnSkullcracker = function(self)
            local duration;
            if self:IsHeroic() then
                duration = 8;
          else
                duration = 12;
            end
            self:ShowZone("Impact", "player", duration);
            self:StartTimer("Skullcracker", duration);
        end,
    },
} );

--   2. Corla, Herald of Twilight
QuickModule:AddDescriptionTable( {
    boss = 39679,
    importance = 20,

    spells = {
        { id = 75823, -- Dark Command
          trigger = "cast",
          cooldown = 5,
          callback = "OnDarkCommand",
        },
        { id = 75697, -- Evolution
          trigger = "effectgain",
          onSelf = true,
          callback = "OnEvolutionStack",
        },
    },

    locale = {
        ["enUS"] = {
            ["Stack"] = "%d stacks of Evolution debuff!",
        },
        ["frFR"] = {
            ["Stack"] = "%d empilements de l'effet Evolution !",
        },
    },

    extra = {
        OnDarkCommand = function(self)
            self:InterruptMob(nil, "high");
        end,

        OnEvolutionStack = function(self, guid, count)
            local category = nil;
            if count == 80 then
                category = "all/critical";

        elseif count == 70 then
                category = "all/high";
   
        elseif count == 50 then
                category = "all/normal";
            end
            if category then
                self:AlertCategorized(category, self:FormatLoc("Stack", count));
            end
        end,
    },
} );

--   3. Karsh Steelbender
QuickModule:AddDescriptionTable( {
    boss = 39698,
    importance = 30,

    spells = {
        -- Superheated armor
        { id = 75846,
          trigger = "effectgain",
          callback = "OnHeatStack",
        },
        { id = 75846,
          trigger = "effectfade",
          callback = "OnHeatStack",
        },
    },

    extra = {
        heatCount = 0,

        algorithm = {
            label = {
                ["enUS"] = "Count",
                ["frFR"] = "Compt",
            },
            color = {1, 0.5, 0},
            digits = 2,
            useMultiples = false,

            GetValue = function(self, uid)
                return self.module:GetHeatCount();
            end,

            GetValueMasked = function(self, guid)
                return self.module:GetHeatCount();
            end,
        },

        OnEnable = function(self)
            local EnemyList = self:GetWidget("EnemyList");
            if EnemyList then
                EnemyList:SetExtraAlgorithm(self.algorithm); -- Enable the secondary bar in the enemy list.
            end
        end,

        OnHeatStack = function(self, guid, count)
            self.heatCount = count;
        end,

        GetHeatCount = function(self)
            return self.heatCount, 15;
        end,
    },
} );

--   4. Beauty
QuickModule:AddDescriptionTable( {
    boss = 39700, -- Beauty
    mooks = {40008, 40011, 40013, 40015}, -- Puppies (Runty is last)
    importance = 10,

    zones = {
        { id = "Spit",
          category = "all/normal",
          color = "yellow",
          duration = 9,
          radius = 8,
          type = "anchored",
          multiple = true,
        },
    },

    spells = {
        -- Magma spit
        { id = 76031,
          trigger = "effectgain",
          zone = "Spit",
        },
        { id = 76031,
          trigger = "effectfade",
          zone = "Spit",
        },
    },
} );

--   5. Ascendant Lord Obsidius
QuickModule:AddDescriptionTable( {
    boss = 39705,
    safeEngage = true,
    importance = 50,
} );

-- III. Vortex Pinnacle
--   1. Grand Vizier Ertan
QuickModule:AddDescriptionTable( {
    boss = 43878,
    importance = 20,
} );

--   2. Altairus
QuickModule:AddDescriptionTable( {
    boss = 43873,
    importance = 40,

    spells = {
        { id = 88286, -- Downwind of Altairus
          trigger = "effectgain",
          cooldown = 5,
          onSelf = true,
          callback = "OnDownwind",
        },
    },

    locale = {
        ["enUS"] = {
            ["Downwind"] = "You are standing downwind!",
        },
        ["frFR"] = {
            ["Downwind"] = "Vous êtes dans le vent descendant !",
        },
    },

    extra = {
        OnDownwind = function(self)
            self:AlertCategorized("all/high", self:Localize("Downwind"));
        end,
    },
} );

--   3. Asaad
QuickModule:AddDescriptionTable( {
    boss = 43875,
    importance = 50,
    range = 8,

    timers = {
        { id = "GroundingField",
          category = "all/high",
          spell = 86911,
          default = 45,
          first = 45,
          loop = true,
        },
    },

    spells = {
        { id = 86911, -- Unstable Grounding Field
          trigger = "cast",
          finishTimer = "GroundingField",
          callback = "OnField",
        },
    },

    locale = {
        ["enUS"] = {
            ["GroundingFieldAlert"] = "Get ready to get into the safe zone!",
        },
        ["frFR"] = {
            ["GroundingFieldAlert"] = "Préparez-vous à entrer dans la zone sûre !",
        },
    },

    extra = {
        OnField = function(self)
            self:AlertCategorized("all/high", self:Localize("GroundingFieldAlert"));
        end,
    },
} );

-- IV. Stonecore
--   1. Corborus
QuickModule:AddDescriptionTable( {
    boss = 43438,
    timeout = 120, -- The boss vanishes.
    timeCoeff = 1.20,
    importance = 25,

    spells = {
        { id = 86881, -- Crystal Barrage
          trigger = "damage",
          cooldown = 2.5,
          onSelf = true,
          callback = "OnCrystalBarrage",
        },
    },

    locale = {
        ["enUS"] = {
            ["CrystalBarrage"] = "Get out!",
        },
        ["frFR"] = {
            ["CrystalBarrage"] = "Evitez la zone !",
        },
    },

    extra = {
        OnCrystalBarrage = function(self)
            self:AlertCategorized("all/high", self:Localize("CrystalBarrage"));
        end,
    },
} );

--   2. Slabhide
QuickModule:AddDescriptionTable( {
    boss = 43214,
    timeCoeff = 1.20,
    importance = 35,

    spells = {
        { id = 80801, -- Lava Pool
          trigger = "damage",   
          cooldown = 2.5,
          onSelf = true,
          callback = "OnLavaPool",
        },
        { id = 92265, -- Crystal Storm
          trigger = "cast",
          cooldown = 5,
          callback = "OnCrystalStorm",
        },
    },

    locale = {
        ["enUS"] = {
            ["LavaPool"] = "You're burning!",
            ["CrystalStorm"] = "Hide yourself!",
        },
        ["frFR"] = {
            ["LavaPool"] = "Vous brûlez !",
            ["CrystalStorm"] = "Cachez-vous !",
        },
    },

    extra = {
        OnLavaPool = function(self)
            self:AlertCategorized("all/high", self:Localize("LavaPool"));
        end,

        OnCrystalStorm = function(self)
            self:AlertCategorized("all/critical", self:Localize("CrystalStorm"));
        end,
    },
} );

--   3. Ozruk
QuickModule:AddDescriptionTable( {
    boss = 42188,
    timeCoeff = 1.20,
    importance = 45,

    zones = {
        { id = "Impact",
          category = "all/critical",
          color = "red",
          duration = 4,
          radius = 8,
          type = "aimed",
        },
    },

    spells = {
        { id = 78835, -- Elementium Spike Shield
          trigger = "effectgain",
          cooldown = 11,
          callback = "OnSpikeShield",
        },
        { id = 78939, -- Elementium Bulwark
          trigger = "effectgain",
          cooldown = 11,
          callback = "OnBulwark",
        },
        { id = 78903, -- Ground Slam
          trigger = "cast",
          cooldown = 5,
          callback = "OnGroundSlam",
        },
    },

    locale = {
        ["enUS"] = {
            ["StopMelee"] = "Don't melee too much!",
            ["StopCast"] = "Don't cast too much!",
        },
        ["frFR"] = {
            ["StopMelee"] = "Modérez vos coups !",
            ["StopCast"] = "Modérez vos incantations !",
        },
    },

    extra = {
        OnSpikeShield = function(self)
            self:AlertCategorized("melee/high", self:Localize("StopMelee"));
        end,

        OnBulwark = function(self)
            self:AlertCategorized("caster/normal", self:Localize("StopCast"));
        end,

        OnGroundSlam = function(self)
            local guid = self:GetTankUnit(); -- Place the zone on the tank.
            if guid then
                self:ShowZone("Impact", guid);
            end
        end,
    },
} );

--   4. High Priestess Azil
QuickModule:AddDescriptionTable( {
    boss = 42333,
    timeCoeff = 2.40,
    importance = 50,
} );

-- V. Lost City of the Tol'vir
--   1. General Husam
QuickModule:AddDescriptionTable( {
    boss = 44577,
    importance = 25,

    spells = {
        { id = 83445, -- Shockwave
          trigger = "cast",
          cooldown = 5,
          callback = "OnShockwave",
        },
    },

    extra = {
        OnShockwave = function(self)
            self:AlertIncomingSpell("shield/high", 83445);
        end,
    },
} );

--   2. A) Lockmaw
QuickModule:AddDescriptionTable( {
    boss = 43614,
    range = 5,
    timeCoeff = 1.15,
    importance = 20,

    thresholds = {
        { value = 0.300,
          description = "Enrage",
        },
    },

    spells = {
        { id = 81690, -- Scent of Blood
          trigger = "effectgain",
          cooldown = 5,
          callback = "OnScent",
        },
    },

    settings = {
        {
          id = "Scent",
          type = "BOOLEAN",
          action = "SYMBOL",
          default = false,
          exclusive = true,
          meta = { spell = 81690,
                   symbol = 1 },
        },
    },

    locale = {
        ["enUS"] = {
            ["Enrage"] = "Enrage",
        },
        ["frFR"] = {
            ["Enrage"] = "Enrager",
        },
    },

    extra = {
        OnScent = function(self, guid)
            if self:IsMe(guid) then
                self:AlertSpell("all/high", 81690, guid);
            end
            if self:GetSetting("Scent") then
                self:AnnounceUnitSpell(81690, guid, 1, 12);
            end
        end,
    },
} );

--   2. B) Augh
QuickModule:AddDescriptionTable( {
    boss = 49045,
    importance = 15,
} );

--   3. High Prophet Barim
QuickModule:AddDescriptionTable( {
    boss = 43612, -- High Prophet Barim
    mooks = {43927}, -- Harbinger of Darkness
    range = 8,
    timeCoeff = 2.00,
    importance = 40,

    thresholds = {
        { value = 0.500,
          description = "Repentance",
        },
    },

    zones = {
        { id = "Plague",
          category = "all/high",
          color = "red",
          duration = 8,
          radius = 8,
          type = "anchored",
        },
    },

    spells = {
        { id = 82506,
          trigger = "effectgain",
          cooldown = 3,
          callback = "OnLash",
        },

        -- Plague of Ages
        { id = 82622,
          trigger = "effectgain",
          zone = "Plague",
        },
        { id = 82622,
          trigger = "effectfade",
          zone = "Plague",
        },
    },

    locale = {
        ["enUS"] = {
            ["Repentance"] = "Repentance",
        },
        ["frFR"] = {
            ["Repentance"] = "Repentir",
        },
    },

    extra = {
        OnLash = function(self)
            self:AlertIncomingSpell("shield/high", 82506);
        end,
    },
} );

--   4. Siamat
QuickModule:AddDescriptionTable( {
    boss = 44819,
    range = 5,
    timeCoeff = 2.25,
    importance = 50,

    spells = {
        { id = 84589,
          trigger = "effectfade",
          callback = "OnPhaseTwo",
        },
    },

    extra = {
        OnEngaged = function(self)
            self:SetPhase(1);
        end,

        OnPhaseTwo = function(self)
            if self:GetEngageTime() > 0 then
                self:SetIncreasingPhase(2);
            end
        end,
    },
} );

-- VI. Halls of Origination
--   1. Temple Guardian Anhuur
QuickModule:AddDescriptionTable( {
    boss = 39425,
    range = 15,
    timeCoeff = 2.00,
    importance = 30,

    thresholds = {
        { value = 0.666,
          description = "Hymn",
        },
        { value = 0.333,
          description = "Hymn",
        },
    },

    zones = {
        { id = "Reckoning",
          category = "all/high",
          color = "red",
          duration = 8,
          radius = 15,
          type = "anchored",
        },
    },

    spells = {
        { id = 74938, -- Shield of Light
          trigger = "effectgain",
          cooldown = 3,
          callback = "OnShield",
        },
        { id = 75117, -- Burning Light
          trigger = "damage",
          onSelf = true,
          cooldown = 3,
          callback = "OnFlame",
        },

        -- Divine Reckoning
        { id = 75592,
          trigger = "effectgain",
          zone = "Reckoning",
        },
        { id = 75592,
          trigger = "effectfade",
          zone = "Reckoning",
        },
    },

    locale = {
        ["enUS"] = {
            ["Hymn"] = "Hymn",
            ["Shield"] = "The shield is enabled!",
            ["Flame"] = "You're burning!",
        },
        ["frFR"] = {
            ["Hymn"] = "Hymne",
            ["Shield"] = "Le bouclier est activé !",
            ["Flame"] = "Vous brûlez !",
        },
    },

    extra = {
        OnShield = function(self)
            self:AlertCategorized("all/high", self:Localize("Shield"));
        end,

        OnFlame = function(self)
            self:AlertCategorized("all/high", self:Localize("Flame"));
        end,
    },
} );

--   2. Earthrager Ptah
QuickModule:AddDescriptionTable( {
    boss = 39428,
    timeout = 120, -- The boss vanishes.
    timeCoeff = 1.40,
    importance = 15,

    thresholds = {
        { value = 0.500,
          description = "Adds",
        },
    },

    spells = {
        { id = {75547, 75548, 89396, 89648}, -- Quicksand
          trigger = "effectgain",
          onSelf = true,
          cooldown = 3,
          callback = "OnQuicksand",
        },
    },

    locale = {
        ["enUS"] = {
            ["Adds"] = "Reinforcements",
            ["Quicksand"] = "You're in a quicksand!",
        },
        ["frFR"] = {
            ["Adds"] = "Renforts",
            ["Quicksand"] = "Sortez des sables mouvants !",
        },
    },

    extra = {
        bossDiedOnce = false,

        OnEnable = function(self)
            self:EnableBossMultiDeath(); -- Prevents the module from ending on the first death of the boss.
        end,

        OnQuicksand = function(self)
            self:AlertCategorized("all/high", self:Localize("Quicksand"));
        end,

        OnBossDeath = function(self, guid, id, name)
            if self.bossDiedOnce then
                self:FlagBossAsDead();
          else
                self.bossDiedOnce = true;
            end
        end,
    },
} );

--   3. Anraphet
QuickModule:AddDescriptionTable( {
    boss = 39788,
    range = 6,
    safeEngage = true,
    importance = 25,

    timers = {
        { id = "OmegaStance",
          category = "healer/high",
          spell = 75622,
          default = 47.5,
          first = 35,
          loop = true,
        },
    },

    zones = {
        { id = "Beam",
          category = "all/high",
          color = "red",
          duration = 3,
          radius = 6,
          type = "aimed",
          multiple = true,
        },
    },

    spells = {
        { id = 76184, -- Alpha Beams
          trigger = "cast",
          cooldown = 5,
          callback = "OnBeam",
        },
        { id = {76912, 76956, 91205}, -- Alpha Beams (damage)
          trigger = "damage",
          cooldown = 3,
          callback = "OnBeamDamage",
        },
        { id = 75622, -- Omega Stance
          trigger = "cast",
          finishTimer = "OmegaStance",
          callback = "OnOmegaStance",
        },
    },

    extra = {
        beamChecklist = { },

        OnBeam = function(self)
            wipe(self.beamChecklist);
            self:AlertIncomingSpell("all/high", 76184);
        end,

        OnBeamDamage = function(self, guid) -- This approximates the position of void zones.
            if not self.beamChecklist[guid] then
                if self:IsHeroic() then
                    self:ShowZone("Beam", guid, 300); -- Make the zone last the full fight.
              else
                    self:ShowZone("Beam", guid);
                end
                self.beamChecklist[guid] = true;
            end
        end,

        OnOmegaStance = function(self)
            self:AlertIncomingSpell("healer/high", 75622);
        end,
    },
} );

--   4. Isiset
QuickModule:AddDescriptionTable( {
    boss = 39587,
    timeout = 120, -- The boss vanishes.
    timeCoeff = 1.20,
    importance = 30,

    thresholds = {
        { value = 0.666,
          description = "Vanish",
        },
        { value = 0.333,
          description = "Vanish",
        },
    },

    spells = {
        { id = 74136, -- Supernova
          trigger = "cast",
          cooldown = 3,
          callback = "OnSupernova",
        },
    },

    locale = {
        ["enUS"] = {
            ["Vanish"] = "Vanish",
            ["Supernova"] = "Don't look at the boss!",
        },
        ["frFR"] = {
            ["Vanish"] = "Disparaît",
            ["Supernova"] = "Ne regardez pas le boss !",
        },
    },

    extra = {
        OnSupernova = function(self)
            self:AlertCategorized("all/critical", self:Localize("Supernova"));
        end,
    },
} );

--   5. Ammunae
QuickModule:AddDescriptionTable( {
    boss = 39731,
    timeCoeff = 1.10,
    importance = 35,

    spells = {
        { id = 75790, -- Rampant Growth
          trigger = "cast",
          callback = "OnRampantGrowth",
        },
    },

    extra = {
        OnRampantGrowth = function(self)
            self:AlertIncomingSpell("all/high", 75790);
        end,
    },
} );

--   6. Setesh
QuickModule:AddDescriptionTable( {
    boss = 39732,
    mooks = {41148, 41208},
    timeCoeff = 1.10,
    importance = 45,

    spells = {
        { id = 76903, -- Anti-Magic Prison
          trigger = "effectgain",
          callback = "OnAntiMagicPrison",
        },
    },

    locale = {
        ["enUS"] = {
            ["AMP"] = "Anti-Magic Prison on %s!",
        },
        ["frFR"] = {
            ["AMP"] = "Prison anti-magie sur %s !",
        },
    },

    extra = {
        OnAntiMagicPrison = function(self, guid)
            self:AlertUnitCategorized("all/normal", self:Localize("AMP"), guid);
        end,
    },
} );

--   7. Rajh
QuickModule:AddDescriptionTable( {
    boss = 39378,
    range = 8,
    timeCoeff = 0.65,
    importance = 50,

    timers = {
        { id = "Blessing",
          category = "all/high",
          spell = 76352,
          default = 20,
          loop = true,
        },
    },

    zones = {
        { id = "Leap",
          category = "all/high",
          color = "red",
          duration = 4,
          radius = 8,
          type = "aimed",
        },
    },

    spells = {
        { id = 80352, -- Summon Sun Orb
          trigger = "cast",
          cooldown = 5,
          callback = "OnSunOrb",
        },
        { id = 87657, -- Burning Adrenaline Rush
          trigger = "effectgain",
          cooldown = 3,
          zone = "Leap",
          callback = "OnLeap",
        },
        { id = 76352, -- Blessing of the Sun
          trigger = "effectgain",
          cooldown = 3,
          startTimer = "Blessing",
        },
    },

    locale = {
        ["enUS"] = {
            ["Leap"] = "Dodge!",
        },
        ["frFR"] = {
            ["Leap"] = "Esquivez !",
        },
    },

    extra = {
        OnSunOrb = function(self)
            self:InterruptMob(nil, "normal");
        end,

        OnLeap = function(self)
            self:AlertCategorized("all/high", self:Localize("Leap"));
        end
    },
} );

-- VII. Grim Batol
--   1. General Umbriss
QuickModule:AddDescriptionTable( {
    boss = 39625,
    range = 8,
    timeCoeff = 1.20,
    importance = 30,

    thresholds = {
        { value = 0.300,
          description = "Frenzy",
        },
    },

    zones = {
        { id = "Malady",
          category = "all/critical",
          color = "red",
          duration = 10,
          radius = 8,
          type = "anchored",
          multiple = true,
        },
    },

    spells = {
        { id = 74846,
          trigger = "effectgain",
          cooldown = 3,
          callback = "OnBleedingWound",
        },
        { id = 74670,
          trigger = "cast",
          callback = "OnBlitz",
        },

        -- Modgud's Malady
        { id = 74837,
          trigger = "effectgain",
          zone = "Malady",
        },
        { id = 74837,
          trigger = "effectfade",
          zone = "Malady",
        },
    },

    locale = {
        ["enUS"] = {
            ["Frenzy"] = "Frenzy",
            ["BleedingWound"] = "Bleeding Wound!",
        },
        ["frFR"] = {
            ["Frenzy"] = "Frénésie",
            ["BleedingWound"] = "Plaie hémorragique !",
        },
    },

    extra = {
        OnBleedingWound = function(self)
            self:AlertCategorized("shield/normal", self:Localize("BleedingWound"));
        end,

        OnBlitz = function(self)
            self:AlertIncomingSpell("caster/high", 74670);
        end,
    },
} );

--   2. Forgemaster Throngus
QuickModule:AddDescriptionTable( {
    boss = 40177,
    importance = 25,

    timers = {
        { id = "WeaponSwitch",
          category = "all/normal",
          spell = 75000,
          default = 30,
          loop = true,
        },
    },

    spells = {
        { id = 75000,
          trigger = "cast",
          finishTimer = "WeaponSwitch",
        },
        { id = 75007,
          trigger = "effectgain",
          callback = "OnMace",
        },
        { id = 74984,
          trigger = "cast",
          cooldown = 2.5,
          callback = "OnCaveIn",
        },
        { id = 74987,
          trigger = "damage",
          onSelf = true,
          cooldown = 5,
          callback = "OnCaveInDamage",
        },
        { id = 75056,
          trigger = "effectgain",
          cooldown = 3,
          callback = "OnImpalingSlam",
        },
    },

    locale = {
        ["enUS"] = {
            ["Mace"] = "Avoid the boss!",
            ["CaveIn"] = "Cave In incoming!",
            ["CaveInDamage"] = "Move away!",
            ["ImpalingSlam"] = "%s is impaled!",
        },
        ["frFR"] = {
            ["Mace"] = "Fuyez le boss !",
            ["CaveIn"] = "Eboulement imminent !",
            ["CaveInDamage"] = "Déplacez-vous !",
            ["ImpalingSlam"] = "%s est empalé(e) !",
        },
    },

    extra = {
        OnMace = function(self)
            self:AlertCategorized("tank/critical", self:Localize("Mace"));
        end,

        OnCaveIn = function(self)
            self:AlertCategorized("all/normal", self:Localize("CaveIn"));
        end,

        OnCaveInDamage = function(self)
            self:AlertCategorized("all/high", self:Localize("CaveInDamage"));
        end,

        OnImpalingSlam = function(self, guid)
            self:AlertUnitCategorized("healer/high", self:Localize("ImpalingSlam"), guid);
        end,
    },
} );

--   3. Drahga Shadowburner
QuickModule:AddDescriptionTable( {
    boss = 40319, -- Drahga Shadowburner
    mooks = {40320}, -- Valiona
    timeCoeff = 6.00,
    importance = 40,

    extra = {
        OnEnable = function(self)
            self:SetRelevantDamagePending(false);
            self:AddRelevantTargets(40320);
        end,
    },
} );

--   4. Erudax
QuickModule:AddDescriptionTable( {
    boss = 40484,
    mooks = {40600},
    timeCoeff = 1.25,
    importance = 50,

    spells = {
        { id = 75664,
          trigger = "cast",
          callback = "OnShadowGale",
        },
    },

    locale = {
        ["enUS"] = {
            ["ShadowGale"] = "Get into the eye of the gale!",
        },
        ["frFR"] = {
            ["ShadowGale"] = "Placez-vous dans l'oeil de la trombe !",
        },
    },

    extra = {
        OnShadowGale = function(self)
            self:AlertCategorized("all/high", self:Localize("ShadowGale"));
        end,
    },
} );

-- VIII. Deadmines
--   1. Glubtok
QuickModule:AddDescriptionTable( {
    boss = 47162,
    importance = 10,

    thresholds = {
        { value = 0.500,
          description = "PhaseTwo",
        },
    },

    spells = {
        { id = 91397,
          trigger = "damage",
          onSelf = true,
          callback = "OnFireWallDamage",
        },
    },

    locale = {
        ["enUS"] = {
            ["FireWall"] = "You are touching the Fire Wall!",
        },
        ["frFR"] = {
            ["FireWall"] = "Vous touchez le mur de feu !",
        },
    },

    extra = {
        OnFireWallDamage = function(self)
            self:AlertCategorized("all/high", self:Localize("FireWall"));
        end,
    },
} );

--   2. Helix
QuickModule:AddDescriptionTable( {
    boss = 47297, -- Mount of Helix
    adds = {47296}, -- Helix
    timeCoeff = 1.25,
    importance = 20,

    zones = {
        { id = "Bomb",
          category = "all/critical",
          color = "red",
          duration = 10,
          radius = 8,
          type = "anchored",
        },
    },

    spells = {
        { id = 88352,
          trigger = "effectgain",
          zone = "Bomb",
          callback = "OnBomb",
        },
    },

    locale = {
        ["enUS"] = {
            ["Title"] = "Helix",
        },
        ["frFR"] = {
            ["Title"] = "Helix",
        },
    },

    extra = {
        OnBomb = function(self, guid)
            self:Bomb(guid);
        end,
    },
} );

--   3. Foe Reaper
QuickModule:AddDescriptionTable( {
    boss = 43778,
    timeCoeff = 1.40,
    importance = 25,

    thresholds = {
        { value = 0.400,
          description = "Enrage",
        },
    },

    spells = {
        { id = 88481,
          trigger = "cast",
          callback = "OnOverdrive",
        },

        { id = 88495,
          trigger = "cast",
          callback = "OnHarvest",
        },
    },

    locale = {
        ["enUS"] = {
            ["Enrage"] = "Enrage",
        },
        ["frFR"] = {
            ["Enrage"] = "Enrager",
        },
    },

    extra = {
        OnOverdrive = function(self)
            self:AlertIncomingSpell("all/normal", 88481);
        end,

        OnHarvest = function(self)
            self:AlertIncomingSpell("all/high", 88495);
        end,
    },
} );

--   4. Ripsnarl
QuickModule:AddDescriptionTable( {
    boss = 47626,
    timeout = 120, -- The boss vanishes.
    timeCoeff = 1.75,
    importance = 40,

    thresholds = {
        { value = 0.750,
          description = "Vanish",
        },
        { value = 0.500,
          description = "Vanish",
        },
        { value = 0.250,
          description = "Vanish",
        },
    },

    locale = {
        ["enUS"] = {
            ["Vanish"] = "Vanish",
        },
        ["frFR"] = {
            ["Vanish"] = "Disparaît",
        },
    },
} );

--   5. Cookie
QuickModule:AddDescriptionTable( {
    boss = 47739,
    importance = 0,
} );

--   6. Vanessa
QuickModule:AddDescriptionTable( {
    boss = 49541,
    timeout = 90, -- Just in case.
    timeCoeff = 1.80,
    importance = 50,

    thresholds = {
        { value = 0.500,
          description = "Escape",
        },
        { value = 0.250,
          description = "Escape",
        },
    },

    timers = {
        { id = "Deflection",
          category = "dps/high",
          spell = 92614,
          default = 10,
        },
        { id = "DeadlyBlades",
          category = "all/high",
          spell = 92622,
          default = 5,
        },
    },

    spells = {
        { id = 92614,
          trigger = "effectgain",
          startTimer = "Deflection",
        },
        { id = 92622,
          trigger = "effectgain",
          startTimer = "DeadlyBlades",
        },
    },

    locale = {
        ["enUS"] = {
            ["Escape"] = "Escape",
        },
        ["frFR"] = {
            ["Escape"] = "Fuite",
        },
    },
} );

-- IX. Shadowfang Keep
--   1. Ashbury
QuickModule:AddDescriptionTable( {
    boss = 46962,
    importance = 40,

    thresholds = {
        { value = 0.250,
          description = "Calamity",
        },
    },

    timers = {
        { id = "Asphyxiate",
          category = "all/critical",
          spell = 93423,
          first = 20,
          default = 40,
          loop = true,
        },
    },

    spells = {
        { id = 93423,
          trigger = "cast",
          finishTimer = "Asphyxiate",
        },
    },

    locale = {
        ["enUS"] = {
            ["Calamity"] = "Calamity",
        },
        ["frFR"] = {
            ["Calamity"] = "Calamité",
        },
    },
} );

--   2. Silverlaine
QuickModule:AddDescriptionTable( {
    boss = 3887,
    mooks = {50834, 50851, 50857, 50869},
    timeCoeff = 1.25,
    importance = 20,
} );

--   3. Springvale
QuickModule:AddDescriptionTable( {
    boss = 4278,
    mooks = {50613, 50615},
    timeCoeff = 1.30,
    importance = 35,
} );

--   4. Walden
QuickModule:AddDescriptionTable( {
    boss = 46963,
    importance = 25,

    spells = {
        { id = 93697,
          trigger = "effectgain",
          onSelf = true,
          cooldown = 2.5,
          callback = "OnMixture",
        },
    },

    locale = {
        ["enUS"] = {
            ["Move"] = "Keep moving!",
        },
        ["frFR"] = {
            ["Move"] = "Restez en mouvement !",
        },
    },

    extra = {
        OnMixture = function(self)
            self:AlertCategorized("all/normal", self:Localize("Move"));
        end,
    },
} );

--   5. Godfrey
QuickModule:AddDescriptionTable( {
    boss = 46964,
    importance = 50,

    timers = {
        { id = "Ghouls",
          category = "all/normal",
          spell = 93707,
          default = 30,
        },
        { id = "PistolBarrage",
          category = "all/high",
          spell = 93520,
          default = 30,
        },
    },

    locale = {
        ["enUS"] = {
            ["PistolDamage"] = "Pewpewpew!",
        },
        ["frFR"] = {
            ["PistolDamage"] = "Pioupioupiou !",
        },
    },

    spells = {
        { id = 93707,
          trigger = "cast",
          startTimer = "Ghouls",
        },
        { id = 93520,
          trigger = "cast",
          startTimer = "PistolBarrage",
        },
        { id = 93564,
          trigger = "damage",
          onSelf = true,
          cooldown = 2.5,
          callback = "OnPistolBarrageDamage",
        },
    },

    extra = {
        OnPistolBarrageDamage = function(self)
            self:AlertCategorized("all/high", self:Localize("PistolDamage"));
        end,
    },
} );

-- X. Zul'Aman

-- 1. Nalorakk
QuickModule:AddDescriptionTable( {
    boss = 23576,
    importance = 30,
} );

-- 2. Akil'zon
QuickModule:AddDescriptionTable( {
    boss = 23574,
    range = 12,
    importance = 35,

    timers = {
        { id = "Storm",
          category = "all/high",
          spell = 43648,
          default = 55,
          first = 47.5,
          loop = true,
        },
    },

    zones = {
        { id = "Safe",
          category = "all/high",
          color = "green",
          duration = 8,
          radius = 10,
          type = "anchored",
        },
    },

    spells = {
        { id = 43648,
          trigger = "effectgain",
          zone = "Safe",
          finishTimer = "Storm",
          callback = "OnStorm",
        },
        { id = 43648,
          trigger = "effectfade",
          zone = "Safe",
          callback = "OnStormFade",
        },
    },

    extra = {
        OnStorm = function(self)
            self:ChangeMinRange(0);
        end,

        OnStormFade = function(self)
            self:ChangeMinRange(12);
        end,
    },
} );

-- 3. Jan'alai
QuickModule:AddDescriptionTable( {
    boss = 23578,
    importance = 40,

    thresholds = {
        { value = 0.500,
          description = "Hatch",
        },
        { value = 0.200,
          description = "Frenzy",
        },
    },

    locale = {
        ["enUS"] = {
            ["Hatch"] = "Eggs hatch",
            ["Frenzy"] = "Frenzy",
        },
        ["frFR"] = {
            ["Hatch"] = "Eclosion",
            ["Frenzy"] = "Frénésie",
        },
    },
} );

-- 4. Halazzi
QuickModule:AddDescriptionTable( {
    boss = 23577,
    importance = 40,
    timeCoeff = 1.75,

    thresholds = {
        { value = 0.665,
          description = "Lynx",
        },
        { value = 0.335,
          description = "Lynx",
        },
        { value = 0.150,
          description = "Frenzy",
        },
    },

    locale = {
        ["enUS"] = {
            ["Lynx"] = "Lynx",
            ["Frenzy"] = "Frenzy",
        },
        ["frFR"] = {
            ["Lynx"] = "Lynx",
            ["Frenzy"] = "Frénésie",
        },
    },
} );

-- 5. Malacrass
QuickModule:AddDescriptionTable( {
    boss = 24239,
    importance = 45,

    timers = {
        { id = "SpiritBolts",
          category = "all/normal",
          spell = 43383,
          default = 36,
          loop = true,
        },
    },

    spells = {
        { id = 43383,
          trigger = "effectgain",
          finishTimer = "SpiritBolts",
        },
    },
} );

-- 6. Daakara
QuickModule:AddDescriptionTable( {
    boss = 23863,
    importance = 50,

    thresholds = {
        { value = 0.800,
          description = "Transformation",
        },
        { value = 0.400,
          description = "Transformation",
        },
    },

    locale = {
        ["enUS"] = {
            ["Transformation"] = "Transformation",
        },
        ["frFR"] = {
            ["Transformation"] = "Transformation",
        },
    },
} );

-- XI. Zul Gurub

-- 1. Venoxis
QuickModule:AddDescriptionTable( {
    boss = 52155,
    importance = 35,

    spells = {
        { id = 96477,
          trigger = "effectgain",
          onSelf = true,
          callback = "OnToxicLink",
        },
    },

    extra = {
        OnToxicLink = function(self, guid)
            self:AlertSpell("all/high", 96477, guid);
        end,
    },
} );

-- 2. Mandokir
QuickModule:AddDescriptionTable( {
    boss = 52151,
    importance = 35,

    thresholds = {
        { value = 0.200,
          description = "Frenzy",
        },
    },

    timers = {
        { id = "Decapitate",
          category = "all/high",
          spell = 96684,
          default = 30,
        },
        { id = "Bloodletting",
          category = "healer/high",
          spell = 96776,
          default = 25,
        },
    },

    spells = {
        { id = 96684,
          trigger = "cast",
          startTimer = "Decapitate",
        },
        { id = 96776,
          trigger = "effectgain",
          startTimer = "Bloodletting",
        },
        { id = 96484,
          trigger = "cast",
          callback = "RemoveGhost",
        },
        { id = 96740,
          trigger = "cast",
          callback = "OnDevastatingSlam",
        },
        { id = 96724,
          trigger = "cast",
          callback = "OnOhganRevive",
        },
    },

    locale = {
        ["enUS"] = {
            ["Frenzy"] = "Frenzy",
        },
        ["frFR"] = {
            ["Frenzy"] = "Frénésie",
        },
    },

    extra = {
        ghosts = 8,

        algorithm = {
            label = {
                ["enUS"] = "Spirit",
                ["frFR"] = "Esprit",
            },
            color = {0, 1, 1},
            digits = 1,
            useMultiples = false,

            GetValue = function(self, uid)
                return self.module:GetGhosts();
            end,

            GetValueMasked = function(self, guid)
                return self.module:GetGhosts();
            end,
        },

        OnEnable = function(self)
            local EnemyList = self:GetWidget("EnemyList");
            if EnemyList then
                EnemyList:SetExtraAlgorithm(self.algorithm); -- Enable the secondary bar in the enemy list.
            end
        end,

        OnDeath = function(self, guid)
            if self:GetMobID(guid) == 52156 then
                self:RemoveGhost();
            end
        end,

        GetGhosts = function(self)
            return self.ghosts, 8;
        end,

        RemoveGhost = function(self)
            self.ghosts = max(0, self.ghosts - 1);
        end,

        OnDevastatingSlam = function(self)
            self:AlertIncomingSpell("tank/critical", 96740);
        end,

        OnOhganRevive = function(self)
            self:AlertIncomingSpell("dps/high", 96724);
        end,
    },
} );

-- 3. Kilnara
QuickModule:AddDescriptionTable( {
    boss = 52059,
    importance = 40,

    thresholds = {
        { value = 0.500,
          description = "PhaseTwo",
        },
    },

    timers = {
        { id = "WaveAgony",
          category = "all/high",
          spell = 96457,
          default = 30,
        },
    },

    spells = {
        { id = 96457,
          trigger = "cast",
          startTimer = "WaveAgony",
        },
        { id = 97380,
          trigger = "effectgain",
          callback = "OnPhaseTwo",
        },
        { id = 96435,
          trigger = "cast",
          callback = "OnTears",
        },
        { id = 96423,
          trigger = "cast",
          callback = "OnLash",
        },
    },

    extra = {
        OnEngaged = function(self)
            self:SetPhase(1);
        end,

        OnPhaseTwo = function(self)
            self:SetIncreasingPhase(2);
        end,

        OnTears = function(self)
            self:InterruptMob();
        end,

        OnLash = function(self, guid)
            self:AlertSpell("healer/high", 96423, guid);
        end,
    },
} );

-- 4. Zanzil
QuickModule:AddDescriptionTable( {
    boss = 52053,
    importance = 45,
    timeCoeff = 1.25,

    spells = {
        { id = 96338,
          trigger = "cast",
          callback = "OnGas",
        },
        { id = 96914,
          trigger = "damage",
          onSelf = true,
          cooldown = 3,
          callback = "OnFire",
        },
    },

    locale = {
        ["enUS"] = {
            ["Fire"] = "You're burning!",
        },
        ["frFR"] = {
            ["Fire"] = "Vous brûlez !",
        },
    },

    extra = {
        OnGas = function(self)
            self:AlertIncomingSpell("all/high", 96338);
        end,

        OnFire = function(self)
            self:AlertCategorized("all/high", self:Localize("Fire"));
        end,
    },
} );

-- 5. Jindo
QuickModule:AddDescriptionTable( {
    boss = 52148,
    timeout = 300,
    timeCoeff = 1.25,
    importance = 50,

    thresholds = {
        { value = 0.700,
          description = "PhaseTwo",
        },
    },

    timers = {
        { id = "Deadzone",
          category = "all/high",
          spell = 97170,
          default = 20,
          first = 15,
          loop = true,
        },
        { id = "Shadows",
          category = "all/high",
          spell = 97172,
          default = 20,
        },
    },

    spells = {
        { id = 97170,
          trigger = "cast",
          finishTimer = "Deadzone",
        },
        { id = 97172,
          trigger = "effectgain",
          startTimer = "Shadows",
        },
        { id = 97158,
          trigger = "cast",
          callback = "OnPhaseTwo",
        },
        { id = 97198,
          trigger = "cast",
          callback = "OnSlam",
        },
    },

    extra = {
        OnEngaged = function(self)
            self:SetPhase(1);
        end,

        OnPhaseTwo = function(self)
            self:SetIncreasingPhase(2);
        end,

        OnSlam = function(self)
            if self:GetPhase() >= 1 then
                self:AlertIncomingSpell("all/high", 97198);
            end
        end,
    },
} );

-- Extra 1. Gri'lek
QuickModule:AddDescriptionTable( {
    boss = 15082,
    importance = 25,
} );

-- Extra 2. Hazza'rah
QuickModule:AddDescriptionTable( {
    boss = 52271,
    importance = 25,

    spells = {
        { id = 96670,
          trigger = "cast",
          callback = "OnNightmare",
        },
    },

    extra = {
        OnNightmare = function(self)
            self:AlertIncomingSpell("all/high", 96670);
        end,
    },
} );

-- Extra 3. Renataki
QuickModule:AddDescriptionTable( {
    boss = 15084,
    importance = 25,
} );

-- Extra 4. Wushoolay
QuickModule:AddDescriptionTable( {
    boss = 52286,
    importance = 25,
} );

-- XII. End of Time

-- 1. Jaina
QuickModule:AddDescriptionTable( {
    boss = 54445,
    importance = 40,
} );

-- 2. Baine
QuickModule:AddDescriptionTable( {
    boss = 54431,
    importance = 40,
} );

-- 3. Sylvanas
QuickModule:AddDescriptionTable( {
    boss = 54123,
    importance = 40,
} );

-- 4. Tyrande
QuickModule:AddDescriptionTable( {
    boss = 54544,
    importance = 40,
} );

-- 5. Murozond
QuickModule:AddDescriptionTable( {
    boss = 54432,
    importance = 60,

    locale = {
        ["enUS"] = {
            ["ClearTrigger"] = "You know not",
        },
        ["frFR"] = {
            ["ClearTrigger"] = "Vous ne savez pas",
        },
    },
} );

-- XIII. Well of Eternity

-- 1. Perotharn
QuickModule:AddDescriptionTable( {
    boss = 55085,
    importance = 40,
} );

-- 2. Azshara
QuickModule:AddDescriptionTable( {
    boss = 54853,
    mooks = {54884, 54882, 54883},
    timeout = 300,
    timeCoeff = 4,
    importance = 50,

    locale = {
        ["enUS"] = {
            ["ClearTrigger"] = "Enough",
        },
        ["frFR"] = {
            ["ClearTrigger"] = "Assez",
        },
    },
} );

-- 3. Mannoroth
QuickModule:AddDescriptionTable( {
    boss = 54969,
    adds = {55419}, -- Captain Varo'then
    timeout = 180,
    timeCoeff = 0.7,
    importance = 60,

    locale = {
        ["enUS"] = {
            ["ClearTrigger"] = "This victory",
        },
        ["frFR"] = {
            ["ClearTrigger"] = "Cette victoire",
        },
    },
} );

-- XIV. Hour of Twilight

-- 1. Arcurion
QuickModule:AddDescriptionTable( {
    boss = 54590,
    importance = 40,

    thresholds = {
        { value = 0.30,
          description = "TorrentOfFrost" }
    },

    locale = {
        ["enUS"] = {
            ["TorrentOfFrost"] = "Torrent of Frost",
        },
        ["frFR"] = {
            ["TorrentOfFrost"] = "Torrent de givre",
        },
    },
} );

-- 2. Asira
QuickModule:AddDescriptionTable( {
    boss = 54968,
    importance = 50,
} );

-- 3. Benedictus
QuickModule:AddDescriptionTable( {
    boss = 54938,
    importance = 60,

    thresholds = {
        { value = 0.60,
          description = "TwilightEpiphany" }
    },

    locale = {
        ["enUS"] = {
            ["TwilightEpiphany"] = "Twilight Epiphany",
        },
        ["frFR"] = {
            ["TwilightEpiphany"] = "Epiphanie du Crépuscule",
        },
    },
} );