﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Firestorm"] = "Firestorm %d",
        ["AvoidFlames"] = "Avoid flames!",
        ["DamageUndivided"] = "has taken undivided damage.",
    },
    ["frFR"] = {
        ["Firestorm"] = "Tempête de feu %d",
        ["AvoidFlames"] = "Evitez les flammes !",
        ["DamageUndivided"] = "a subi des dégâts non divisés.",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Argaloth", nil, myLocalization);

Module:InstallExtra(M, {
    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(50);
        self:SetScoreBenchmarks(150, 300);
    end,

    OnEngaged = function(self)
        self:TriggerTimer("TankZone");
    end,

    OnBossFound = function(self, uid)
        self:RegisterHealthThreshold(0.6666, self:FormatLoc("Firestorm", 1));
        self:RegisterHealthThreshold(0.3333, self:FormatLoc("Firestorm", 2));
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnFirestorm = function(self)
        self:AlertCategorized("all/normal", self:Localize("AvoidFlames"));

        self:InterruptZone("Tank");
        self:InterruptTimer("TankZone");
        self:StartTimer("TankZone", 18);
    end,

    OnTankZoneRefresh = function(self)
        self:InterruptZone("Tank");
        local guid = self:GetTankUnit();
        if guid then
            self:ShowZone("Tank", guid);
        end
    end,

    OnMeteorSlash = function(self, name, amount)
        if amount > 100000 then
            self:TriggerError("DamageUndivided", name);
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(47120, false, false); -- ID, volatile, bypass
M:DefineBerserk(300);

-- Events

M:RegisterCastEvent(88972, "Firestorm");

M:RegisterDamageEvent(88942, "MeteorSlash");

-- Timers

M:DefineRepeatingTimer("TankZone", "invisible", 3, nil, "TankZone", true);

-- Zones

M:DefineAnchoredZone("Tank", "all/normal", "green", 10, 10, false);

-- Actions

M:DeclareAction("TankZone");
M:AddCallbackToAction("TankZone", "OnTankZoneRefresh");

M:DeclareAction("Firestorm");
M:AddCallbackToAction("Firestorm", "OnFirestorm");

M:DeclareAction("MeteorSlash");
M:AddCallbackToAction("MeteorSlash", "OnMeteorSlash", "targetname", "amount");

-- Errors

M:DefineBossAggroError();
M:DefineCustomError("DamageUndivided");
M:DefineAvoidDamageError(88954, 8); -- Dot