﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local FELUDIUS_ID    = 43687;
local IGNACIOUS_ID   = 43686;
local ARION_ID       = 43688;
local TERRASTRA_ID   = 43689;
local MONSTROSITY_ID = 43735;

local PHASE_MOB = {
    [FELUDIUS_ID]    = 1,
    [IGNACIOUS_ID]   = 1,
    [ARION_ID]       = 2,
    [TERRASTRA_ID]   = 2,
    [MONSTROSITY_ID] = 3,
};

local SHIELD_AMOUNT = {
    ["10N"] =  500000,
    ["25N"] = 1500000,
    ["10H"] =  700000,
    ["25H"] = 2000000,
};

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "Frozen",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 82772,
                 symbol = 1,
                 multiple = true, },
    },
    {
        id = "LightningRod",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 83099,
                 symbol = 1,
                 multiple = true, },
    },
    {
        id = "GravityCrush",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 84948,
                 symbol = 1,
                 multiple = true, },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Title"] = "Ascendant Council",
        ["Shield"] = "Break the shield!",
        ["Transition"] = "Transition",
        ["QuakeTrigger"] = "The ground beneath you",
        ["ThundershockTrigger"] = "The surrounding air",
    },
    ["frFR"] = {
        ["Title"] = "Conseil des Ascendants",
        ["Shield"] = "Percez le bouclier !",
        ["Transition"] = "Transition",
        ["QuakeTrigger"] = "Le sol sous vos pieds",
        ["ThundershockTrigger"] = "L'air qui vous entoure",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("AscendantCouncil", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    aegisGUID = "",
    aegisAmount = 0,
    aegisAmountMax = 0,

    frozen = { },
    lightningRod = { },
    gravityCrush = { },

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(65);
        self:SetScoreBenchmarks(270, 420);

        self:RegisterAnyYell(self:Localize("QuakeTrigger"), "QuakeSoon");
        self:RegisterAnyYell(self:Localize("ThundershockTrigger"), "ThundershockSoon");

        self:SetHealthThresholdDelay(0.15); -- Warn 15% ahead of adds' health instead of default 10%.

        self:DisableStandardClear();
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    OnAddFound = function(self, guid, id, name)
        if id ~= MONSTROSITY_ID then
            local EnemyList = self:GetWidget("EnemyList");
            if EnemyList then
                EnemyList:AddThreshold(guid, self:Localize("Transition"), 0.25);
            end
        end
    end,

    OnAddDeath = function(self, guid, id, name)
        if id == MONSTROSITY_ID then
            self:OnCleared();
        end
    end,

    OnDamage = function(self, actorGUID, actorName, targetGUID, targetName, spellId, spellName, isPeriodic, amount, overkill, school, absorbed)
        local id = self:GetMobID(actorGUID) or 0;
        local phase = PHASE_MOB[id];
        if (not spellId) and phase then
            if phase == 2 then
                 self:OnPhaseTwo();

        elseif phase == 3 then
                 self:OnPhaseThree();
            end
        end
        if targetGUID == self.aegisGUID and absorbed then
            self.aegisAmount = max(0, self.aegisAmount - absorbed);
        end
    end,

    -- *******
    -- Methods
    -- *******

    GetAegisInfo = function(self)
        return self.aegisAmount, self.aegisAmountMax;
    end,

    RemoveTransitionThreshold = function(self, id)
        local guid = self:GetAddGUID(id);
        if guid then
            local EnemyList = self:GetWidget("EnemyList");
            if EnemyList then
                EnemyList:RemoveThreshold(guid, self:Localize("Transition"));
            end
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:InterruptTimer("RisingFlames");
            self:InterruptTimer("HeartIce");
            self:InterruptTimer("Glaciate");
            self:InterruptTimer("WaterBomb");

            self:StartTimer("Quake", 28);
            self:StartTimer("Thundershock", 65);

            self:RemoveTransitionThreshold(FELUDIUS_ID);
            self:RemoveTransitionThreshold(IGNACIOUS_ID);
        end
    end,

    OnPhaseThree = function(self)
        if self:SetIncreasingPhase(3) then
            self:InterruptTimer("HardenSkin");
            self:InterruptTimer("Eruption");
            self:InterruptTimer("Quake");
            self:InterruptTimer("Disperse");
            self:InterruptTimer("Thundershock");

            self:StartTimer("GravityCrush", 10);

            self:RemoveTransitionThreshold(ARION_ID);
            self:RemoveTransitionThreshold(TERRASTRA_ID);

            self:SetMainUnit(MONSTROSITY_ID);
        end
    end,

    OnAegisGain = function(self, guid)
        self.aegisAmount = SHIELD_AMOUNT[ self:GetCode() ] or 1;
        self.aegisAmountMax = self.aegisAmount;
        self.aegisGUID = guid;

        self:StartMirrorValue(self:Localize("Shield"), self.GetAegisInfo);
    end,

    OnAegisFade = function(self, guid)
        self.aegisAmount = 0;
        self.aegisGUID = "";

        self:StopMirrorValue();
    end,

    OnHydrolance = function(self)
        self:InterruptMob(FELUDIUS_ID, "high");
    end,

    OnGlaciate = function(self)
        self:AlertIncomingSpell("melee/high", 82746);
    end,

    OnPowerUp = function(self, guid, spellId)
        if self:IsMe(guid) then
            self:AlertSpell("all/normal", spellId, guid);
        end
    end,

    OnWaterlogged = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/normal", 82762, guid);
        end
    end,

    OnFrozenTarget = function(self, guid)
        tinsert(self.frozen, guid);
    end,

    OnFrozen = function(self)
        if self:GetSetting("Frozen") then
            self:AnnounceUnitListSpell(self.frozen, 82772, true, 10);
        end
        wipe(self.frozen);
    end,

    OnQuakeSoon = function(self)
        self:AlertIncomingSpell("all/high", 83565);
    end,

    OnThundershockSoon = function(self)
        self:AlertIncomingSpell("all/high", 83067);
    end,

    OnLightningRodTarget = function(self, guid)
        tinsert(self.lightningRod, guid);
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 83099, guid);
        end
    end,

    OnLightningRod = function(self)
        if self:GetSetting("LightningRod") then
            self:AnnounceUnitListSpell(self.lightningRod, 83099, true, 10);
        end
        wipe(self.lightningRod);
    end,

    OnGravityCrushTarget = function(self, guid)
        tinsert(self.gravityCrush, guid);
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 84948, guid);
        end
    end,

    OnGravityCrush = function(self)
        if self:GetSetting("GravityCrush") then
            self:AnnounceUnitListSpell(self.gravityCrush, 84948, true, 10);
        end
        wipe(self.gravityCrush);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss({FELUDIUS_ID, IGNACIOUS_ID, ARION_ID, TERRASTRA_ID, MONSTROSITY_ID}, false, false); -- ID, volatile, bypass

-- Events

M:RegisterEffectGainEvent(82762, "Waterlogged");
M:RegisterEffectGainEvent(83099, "LightningRodTarget");
M:RegisterEffectGainEvent(82631, "AegisGain");
M:RegisterEffectGainEvent(82772, "FrozenTarget");
M:RegisterEffectGainEvent(84948, "GravityCrushTarget");
M:RegisterEffectGainEvent({82665, 82660}, "PowerUpGain"); -- Both Heart of Ice and Burning Blood.
M:RegisterEffectFadeEvent({82665, 82660}, "PowerUpFade");
M:RegisterEffectFadeEvent(82631, "AegisFade");
M:RegisterCastEvent(82636, "RisingFlames");
M:RegisterCastEvent(82665, "HeartIce");
M:RegisterCastEvent(82699, "WaterBomb");
M:RegisterCastEvent(83087, "Disperse");
M:RegisterCastEvent(84913, "LavaSeed");
M:RegisterCastEvent(83675, "Eruption");
M:RegisterCastEvent(82752, "Hydrolance");
M:RegisterCastEvent(82746, "Glaciate");
M:RegisterCastEvent(83718, "HardenSkin");
M:RegisterCastEvent(83565, "Quake");
M:RegisterCastEvent(83067, "Thundershock");

-- TODO: frost beacon
-- TODO: static overload
-- TODO: gravity core

-- Timers

M:DefineInitialRepeatingVoidTimer("RisingFlames/82636", "all/high", 60, 31);
M:DefineInitialRepeatingVoidTimer("HeartIce/82665", "all/normal", 23, 18);
M:DefineInitialRepeatingVoidTimer("Glaciate/82746", "melee/high", 34, 30);
M:DefineInitialRepeatingVoidTimer("WaterBomb/82699", "melee/high", 34, 15);

M:DefineRepeatingVoidTimer("HardenSkin/83718", "offensive/high", 42);
M:DefineRepeatingVoidTimer("Eruption/83675", "melee/high", 15);
M:DefineRepeatingVoidTimer("Quake/83565", "all/high", 70);
M:DefineRepeatingVoidTimer("Disperse/83087", "all/normal", 30);
M:DefineRepeatingVoidTimer("Thundershock/83067", "all/high", 70);

M:DefineRepeatingVoidTimer("LavaSeed/84913", "all/high", 23);
M:DefineRepeatingVoidTimer("GravityCrush/84948", "all/high", 26);

-- Zones

M:DefineAnchoredZone("PowerUp", "all/normal", "yellow", 10, 60, true);
M:DefineAnchoredZone("LightningRod", "all/high", "red", 12, 15, true);

-- Actions

-- Phase 1

M:DeclareAction("AegisGain");
M:AddCallbackToAction("AegisGain", "OnAegisGain", "targetguid");

M:DeclareAction("AegisFade");
M:AddCallbackToAction("AegisFade", "OnAegisFade", "targetguid");

M:DeclareAction("RisingFlames");
M:AddTriggerTimerToAction("RisingFlames", "RisingFlames");

M:DeclareAction("HeartIce");
M:AddTriggerTimerToAction("HeartIce", "HeartIce");

M:DeclareAction("WaterBomb");
M:AddTriggerTimerToAction("WaterBomb", "WaterBomb");

M:DeclareAction("Hydrolance");
M:AddCallbackToAction("Hydrolance", "OnHydrolance");

M:DeclareAction("Glaciate");
M:AddTriggerTimerToAction("Glaciate", "Glaciate");
M:AddCallbackToAction("Glaciate", "OnGlaciate");

M:DeclareAction("Waterlogged");
M:AddCallbackToAction("Waterlogged", "OnWaterlogged", "targetguid");

M:DeclareAction("FrozenTarget");
M:AddCallbackToAction("FrozenTarget", "OnFrozenTarget", "targetguid");
M:AddExternalActionToAction("FrozenTarget", "Frozen");

M:DeclareAction("Frozen", 5, 0.5);
M:AddCallbackToAction("Frozen", "OnFrozen");

M:DeclareAction("PowerUpGain");
M:AddShowZoneToAction("PowerUpGain", "PowerUp");
M:AddCallbackToAction("PowerUpGain", "OnPowerUp", "targetguid", "spellid");

M:DeclareAction("PowerUpFade");
M:AddHideZoneToAction("PowerUpFade", "PowerUp");

-- Phase 2

M:DeclareAction("Disperse");
M:AddTriggerTimerToAction("Disperse", "Disperse");

M:DeclareAction("Eruption");
M:AddTriggerTimerToAction("Eruption", "Eruption");

M:DeclareAction("HardenSkin");
M:AddTriggerTimerToAction("HardenSkin", "HardenSkin");

M:DeclareAction("Quake");
M:AddTriggerTimerToAction("Quake", "Quake");

M:DeclareAction("Thundershock");
M:AddTriggerTimerToAction("Thundershock", "Thundershock");

M:DeclareAction("QuakeSoon");
M:AddCallbackToAction("QuakeSoon", "OnQuakeSoon");

M:DeclareAction("ThundershockSoon");
M:AddCallbackToAction("ThundershockSoon", "OnThundershockSoon");

M:DeclareAction("LightningRodTarget");
M:AddShowZoneToAction("LightningRodTarget", "LightningRod");
M:AddCallbackToAction("LightningRodTarget", "OnLightningRodTarget", "targetguid");
M:AddExternalActionToAction("LightningRodTarget", "LightningRod");

M:DeclareAction("LightningRod", 5, 0.5);
M:AddCallbackToAction("LightningRod", "OnLightningRod");

-- Phase 3

M:DeclareAction("LavaSeed");
M:AddTriggerTimerToAction("LavaSeed", "LavaSeed");

M:DeclareAction("GravityCrushTarget");
M:AddCallbackToAction("GravityCrushTarget", "OnGravityCrushTarget", "targetguid");
M:AddExternalActionToAction("GravityCrushTarget", "GravityCrush");

M:DeclareAction("GravityCrush", 5, 0.5);
M:AddTriggerTimerToAction("GravityCrush", "GravityCrush");
M:AddCallbackToAction("GravityCrush", "OnGravityCrush");

-- Errors

M:DefineAvoidDamageError(82752); -- Hydrolance
M:DefineAvoidDamageError(83565); -- Quake
M:DefineAvoidDamageError(83067); -- Thundershock