﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local gongs = {
    [41445] = true,
    [42956] = true,
};

local gongsAlgorithm = {
    label = {
        ["enUS"] = "Gongs",
        ["frFR"] = "Gongs",
    },
    color = {0, 1, 1},
    allowDead = true,
    rate = 0.10,
    digits = 2,
    useMultiples = false,

    GetValue = function(self, uid)
        return self.module:GetRemainingGongs();
    end,

    GetValueMasked = function(self, guid)
        return self.module:GetRemainingGongs();
    end,
};

local function soundFunc(self, uid)
    local sound, soundMax = UnitPower(uid, ALTERNATE_POWER_INDEX), UnitPowerMax(uid, ALTERNATE_POWER_INDEX);
    local ratio = sound / max(1, soundMax);
    if sound >= 80 then
        self:SetBarColor(1, 0, 0);
elseif sound >= 50 then
        self:SetBarColor(1, 1, 0);
  else
        self:SetBarColor(0, 1, 0);
    end
    if sound == 100 then
        self:SetTextColor(1, 0, 0);
  else
        self:SetTextColor(1, 1, 1);
    end
    self:SetBarFill(ratio);
    self:SetText(sound);
end

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "ShowSound",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Show sound",
            ["frFR"] = "Montrer l'état du son",
        },
        explain = {
            ["enUS"] = "Display everyone's sound in a list.\n\nIf this option is disabled, BE3 will display instead only the highest sound player (recommanded setting).",
            ["frFR"] = "Affiche le son de tout le monde dans une liste.\n\nSi cette option est désactivée, BE3 affichera alors uniquement le joueur avec le son le plus élevé (configuration recommandée).",
        },
    },
    {
        id = "Track",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 78092,
                 symbol = 8 },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["AirTrigger"] = "Yes, run",
        ["GroundPhase"] = "Ground Phase",
        ["AirPhase"] = "Air Phase",
        ["HighestSound"] = "High sound: %s",
        ["FullSound"] = "has reached 100 sound.",
        ["SoundUI"] = "Sound",
    },
    ["frFR"] = {
        ["AirTrigger"] = "Oui, fuyez",
        ["GroundPhase"] = "Phase au sol",
        ["AirPhase"] = "Phase en vol",
        ["HighestSound"] = "Son élevé : %s",
        ["FullSound"] = "a atteint 100 au son.",
        ["SoundUI"] = "Etat : son",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Atramedes", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    gongs = 10,
    highestSoundGUID = false,
    soundUI = false,
    fullSound = { },

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(65);
        self:SetScoreBenchmarks(400, 600);

        self:RegisterAnyYell(self:Localize("AirTrigger"), "AirPhase");

        self:CheckSoundDisplay();

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:SetExtraAlgorithm(gongsAlgorithm);
        end
    end,

    OnEngaged = function(self)
        self:OnPhaseOne(true);
        self:TriggerTimer("ScanSound");
    end,

    OnSettingUpdate = function(self, id, value)
        if id == "ShowSound" then
            self:CheckSoundDisplay();
        end
    end,

    OnDeath = function(self, guid, name)
        local mobID = self:GetMobID(guid);
        if mobID and gongs[mobID] then
            self.gongs = max(0, self.gongs - 1);
        end
    end,

    -- *******
    -- Methods
    -- *******

    GetRemainingGongs = function(self)
        return self.gongs, 10;
    end,

    GetSoundInfo = function(self)
        local uid = Root.Unit:GetUID(self.highestSoundGUID);
        if uid then
            return UnitPower(uid, ALTERNATE_POWER_INDEX), UnitPowerMax(uid, ALTERNATE_POWER_INDEX);
      else
            return 0, 1;
        end
    end,

    ScanSound = function(self)
        self:EnumerateRaid();

        local highestGUID = nil;
        local highestName = nil;
        local sound = 0;
        local i, uid, pow;
        for i=1, self:GetEnumerationSize() do
            uid = self:GetEnumerationUID(i);
            pow = UnitPower(uid, ALTERNATE_POWER_INDEX);
            if pow > sound then
                highestGUID = UnitGUID(uid);
                highestName = UnitName(uid);
                sound = pow;
            end
        end

        if sound >= 50 and highestGUID and highestName and not self.soundUI then
            if highestGUID ~= self.highestSoundGUID then
                self.highestSoundGUID = highestGUID;
                self:StartMirrorValue(self:FormatLoc("HighestSound", highestName), self.GetSoundInfo);
            end

    elseif self.highestSoundGUID then
            self.highestSoundGUID = false;
            self:StopMirrorValue();
        end
    end,

    CheckSoundDisplay = function(self)
        self.soundUI = self:GetSetting("ShowSound");

        if self.soundUI then
            self:StartUnitMonitor(self:Localize("SoundUI"), soundFunc);
      else
            self:StopUnitMonitor();
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self, first)
        self:SetPhase(1);

        if not first then
            self:StartTimer("AirPhase");
            self:StartTimer("SearingFlame");
            self:StartTimer("SonicBreath", 25);
        end
    end,

    OnPhaseTwo = function(self)
        self:SetPhase(2);

        self:InterruptTimer("SearingFlame");
        self:InterruptTimer("SonicBreath");
        self:StartTimer("GroundPhase");
    end,

    OnTrack = function(self, guid)
        if self:GetSetting("Track") then
            self:AnnounceUnitSpell(78092, guid, 8);
        end
        if self:IsMe(guid) then
            self:AlertSpell("all/critical", 78092, guid);
        end
    end,

    OnDevastation = function(self, name)
        if not self.fullSound[name] then
            self.fullSound[name] = true;
            self:TriggerError("FullSound", name);
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(41442, false, false); -- ID, volatile, bypass
M:DefineBerserk(nil, 600);

-- Events

M:RegisterCastEvent(78075, "SonicBreath");
M:RegisterEffectGainEvent(77840, "SearingFlame");
M:RegisterEffectGainEvent(78092, "TrackStart");
M:RegisterEffectFadeEvent(78092, "TrackStop");
M:RegisterDamageEvent(78868, "Devastation");

-- Timers

M:DefineTimer("GroundPhase", "all/high", 32.5, nil, "GroundPhase", true);
M:DefineRepeatingTimer("ScanSound", "invisible", 1.5, nil, "ScanSound", true);
M:DefineInitialVoidTimer("AirPhase", "all/high", 90);
M:DefineInitialVoidTimer("SearingFlame/77840", "all/critical", 45, 45);
M:DefineInitialVoidTimer("SonicBreath/78075", "all/critical", 40, 25);

-- Zones

M:DefineAnchoredZone("RoaringFlame", "all/high", "red", 10, 20, true);

-- Actions

M:DeclareAction("ScanSound");
M:AddCallbackToAction("ScanSound", "ScanSound");

M:DeclareAction("SonicBreath");
M:AddTriggerTimerToAction("SonicBreath", "SonicBreath");

M:DeclareAction("SearingFlame", 8);
M:AddTriggerTimerToAction("SearingFlame", "SearingFlame");

M:DeclareAction("TrackStart");
M:AddShowZoneToAction("TrackStart", "RoaringFlame");
M:AddCallbackToAction("TrackStart", "OnTrack", "targetguid");

M:DeclareAction("TrackStop");
M:AddHideZoneToAction("TrackStop", "RoaringFlame");

M:DeclareAction("AirPhase");
M:AddTriggerTimerToAction("AirPhase", "AirPhase");
M:AddCallbackToAction("AirPhase", "OnPhaseTwo");

M:DeclareAction("GroundPhase");
M:AddCallbackToAction("GroundPhase", "OnPhaseOne");

M:DeclareAction("Devastation");
M:AddCallbackToAction("Devastation", "OnDevastation", "targetname");

-- Errors

M:DefineBossAggroError();
M:DefineCustomError("FullSound");
M:DefineAvoidDamageError(78100); -- Sonic Breath