﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local ANSHAL_ID = 45870;
local ROHASH_ID = 45872;
local NEZIR_ID  = 45871;

local AREA_LIST = {
    radius = 0.05,

    [ANSHAL_ID] = {0.342, 0.493},
    [ROHASH_ID] = {0.605, 0.499},
     [NEZIR_ID] = {0.476, 0.301},
};

local regenAlgorithm = {
    label = {
        ["enUS"] = "Regen",
        ["frFR"] = "Régen",
    },
    color = {0, 1, 0},
    allowDead = false,
    digits = 2,
    useMultiples = false,

    GetValue = function(self, uid)
        local timeLeft, castTime = self.module:GetCastingInfo(86307, uid);
        if timeLeft and castTime then
            return castTime - timeLeft, castTime;
      else
            return 0, 1;
        end
    end,
};

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "EnergyThreshold",
        type = "NUMBER",
        min = 40,
        max = 91,
        step = 1,
        default = 91,
        label = {
            ["enUS"] = "Energy threshold",
            ["frFR"] = "Seuil d'énergie",
        },
        explain = {
            ["enUS"] = "When the boss reaches this energy amount, an alert will be displayed. Use 91 to disable.",
            ["frFR"] = "Quand le boss atteint ce montant d'énergie, une alerte sera affichée. Mettez 91 pour désactiver.",
        },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Title"] = "Conclave of Wind",
        ["EnergyFull"] = "Full energy",
        ["EnergyAlert"] = "%d energy reached!",
        ["BossRegen"] = "has recovered his health because the other bosses weren't killed on time.",
    },
    ["frFR"] = {
        ["Title"] = "Conclave du vent",
        ["EnergyFull"] = "Energie max",
        ["EnergyAlert"] = "%d énergie atteinte !",
        ["BossRegen"] = "a récupéré sa vie car les autres boss n'ont pas été tués à temps.",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("DjinnCouncil", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    energyAlert = false,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(55);
        self:SetScoreBenchmarks(330, 480);
        self:DisableDamageMeter(); -- because players will be out-of-range.

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:SetExtraAlgorithm(regenAlgorithm);
        end
    end,

    OnEngaged = function(self)
        self:StartTimer("EnergyCheck");
    end,

    -- *******
    -- Methods
    -- *******

    CheckEnergy = function(self)
        local power = UnitPower("boss1") or 0;
        if power >= self:GetSetting("EnergyThreshold") then
            if not self.energyAlert then
                self.energyAlert = true;
                self:Alert(self:FormatLoc("EnergyAlert", power));
            end
      else
            self.energyAlert = false;
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnWindBlast = function(self)
        if self:GetArea(AREA_LIST) == ROHASH_ID then
            self:AlertIncomingSpell("all/high", 86193);
        end
    end,

    OnBossRegen = function(self, name)
        self:TriggerError("BossRegen", name);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss({ANSHAL_ID, ROHASH_ID, NEZIR_ID}, false, false); -- ID, volatile, bypass
M:DefineBerserk(480);

-- Events

M:RegisterCastEvent({86193, 85480, 85483}, "WindBlast");
M:RegisterCastEvent({84643, 84644, 84638}, "EnergyFull");
M:RegisterHealEvent(86307, "BossRegen");

-- Timers

M:DefineRepeatingTimer("EnergyCheck", "invisible", 1, nil, "EnergyCheck", true);
M:DefineInitialRepeatingVoidTimer("EnergyFull", "all/critical", 115, 90);
M:DefineInitialRepeatingVoidTimer("WindBlast/86193", "all/normal", 62, 32);

-- Actions

M:DeclareAction("WindBlast");
M:AddTriggerTimerToAction("WindBlast", "WindBlast");
M:AddCallbackToAction("WindBlast", "OnWindBlast");

M:DeclareAction("EnergyCheck");
M:AddCallbackToAction("EnergyCheck", "CheckEnergy");

M:DeclareAction("EnergyFull", 20);
M:AddTriggerTimerToAction("EnergyFull", "EnergyFull");

M:DeclareAction("BossRegen");
M:AddCallbackToAction("BossRegen", "OnBossRegen", "targetname");

-- Errors

M:DefineAggroError(ANSHAL_ID, 15);
M:DefineAggroError(NEZIR_ID, 15);
M:DefineCustomError("BossRegen");
M:DefineAvoidDamageError({86193, 85480, 85483}); -- Wind Blast
M:DefineAvoidDamageError(86111, 3); -- Ice Patch