﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "MalevolentStrikesThreshold",
        type = "NUMBER",
        min = 1,
        max = 20,
        step = 1,
        default = 10,
        meta = { spell = 39171,
                 threshold = true },
    },
    {
        id = "InterruptChain",
        type = "STRING",
        max = 100,
        default = "Player1 ; Player2",
        exclusive = true,
        action = "RAID_WARNING",
        label = {
            ["enUS"] = "Players assigned to interrupt",
            ["frFR"] = "Joueurs assignés à l'interruption",
        },
        explain = {
            ["enUS"] = "Enter here the names of players who are supposed to interrupt the Nova, separated by a semicolon.\n\nIgnore this option if the Storm Rider is wounded.",
            ["frFR"] = "Entrez ici les noms des joueurs assignés à l'interruption de la nova, séparés par un point-virgule.\n\nIgnorez cette option si le chevaucheur des tempêtes est blessé.",
        },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["MalevolentStrikes"] = "You have %d Malevolent Strikes!",
        ["FirstRoar"] = "First Furious Roar",
    },
    ["frFR"] = {
        ["MalevolentStrikes"] = "Vous avez %d frappes malveillantes !",
        ["FirstRoar"] = "Premier rugissement",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Halfus", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    novaInterruptable = false,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(65);
        self:SetScoreBenchmarks(240, 360);
    end,

    OnEngaged = function(self)
        self:DisableError("Barrage");
    end,

    OnBossFound = function(self, uid)
        self:RegisterHealthThreshold(0.50, self:Localize("FirstRoar"));
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnNovaInterruptable = function(self)
        if not self.novaInterruptable then
            self.novaInterruptable = true;
            self:StartInterruptChain("InterruptChain");
        end
    end,

    OnMalevolentStrikes = function(self, guid, stack)
        if self:IsMe(guid) then
            self:AlertThreshold("MalevolentStrikesThreshold", stack, self:Localize("MalevolentStrikes"));
        end
    end,

    OnBarrage = function(self)
        self:AlertIncomingSpell("all/high", 83706);
    end,

    OnBarrageSlowed = function(self)
        self:EnableError("Barrage");
    end,

    OnParalysis = function(self, guid)
        if guid == self:GetBossGUID() then -- Just to be sure.
            self:StartTimer("Paralysis");
            self:AlertSpell("all/normal", 84030, guid);
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(44600, false, false); -- ID, volatile, bypass
M:DefineBerserk(360);
M:DefineMinRange(4);

-- Events

M:RegisterCastEvent(83710, "Roar");
M:RegisterCastEvent(83707, "Breath");
M:RegisterCastEvent(83612, "NovaInterruptable");
M:RegisterCastEvent(83601, "BarrageSlowed");
M:RegisterEffectGainEvent({83706, 83721}, "Barrage");
M:RegisterEffectGainEvent(39171, "MalevolentStrikes");
M:RegisterEffectGainEvent(84030, "Paralysis");

-- Timers

M:DefineVoidTimer("Paralysis/84030", "all/high", 12);
M:DefineRepeatingVoidTimer("Roar/83710", "all/critical", 30);
M:DefineRepeatingVoidTimer("Barrage/83706", "shield/high", 30);
M:DefineInitialRepeatingVoidTimer("Breath/83707", "shield/high", 22.5, 30);

-- Actions

M:DeclareAction("Roar", 10);
M:AddTriggerTimerToAction("Roar", "Roar");

M:DeclareAction("Breath");
M:AddTriggerTimerToAction("Breath", "Breath");

M:DeclareAction("Barrage");
M:AddTriggerTimerToAction("Barrage", "Barrage");
M:AddCallbackToAction("Barrage", "OnBarrage");

M:DeclareAction("NovaInterruptable");
M:AddCallbackToAction("NovaInterruptable", "OnNovaInterruptable");

M:DeclareAction("BarrageSlowed");
M:AddCallbackToAction("BarrageSlowed", "OnBarrageSlowed");

M:DeclareAction("MalevolentStrikes");
M:AddCallbackToAction("MalevolentStrikes", "OnMalevolentStrikes", "targetguid", "stack");

M:DeclareAction("Paralysis");
M:AddCallbackToAction("Paralysis", "OnParalysis", "targetguid");

-- Errors

M:DefineBossAggroError();
M:DefineError("Barrage", "avoid-damage", {83721, 83734}, 1);

-- Special

M:DefineInterruptChain(83703, 0, 15); -- Shadow Nova interruptions