﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local MAIN_ID = 41570;
local HEAD_ID = 42347;

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "Transition",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Announce transition",
            ["frFR"] = "Annoncer la transition",
        },
        explain = {
            ["enUS"] = "Display a raid warning when the boss is about to break free of the spike.",
            ["frFR"] = "Affiche un avertissement de raid quand le boss est sur le point de se libérer de la pointe.",
        },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["ShadowBreath"] = "Shadow Breath",
        ["ImpaleEnd"] = "End of Impale",
        ["Transition"] = "Boss back soon!",
        ["ImpaleFailed"] = "has been instakilled because the boss wasn't exposed in time.",
        ["MoltenTentrum"] = "had no target in melee range.",
    },
    ["frFR"] = {
        ["ShadowBreath"] = "Souffle d'ombre",
        ["ImpaleEnd"] = "Fin de l'empaler",
        ["Transition"] = "Boss de retour bientôt !",
        ["ImpaleFailed"] = "a été tué instantanément car le boss n'a pas été exposé à temps.",
        ["MoltenTentrum"] = "n'avait pas de cible à portée de corps à corps.",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Magmaw", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(55);
        self:SetScoreBenchmarks(400, 600);

        self:SetRelevantDamagePending(false);
        self:AddRelevantTargets(HEAD_ID);
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();

        if self:IsHeroic() then
            self:RegisterHealthThreshold(0.305, self:Localize("ShadowBreath"));

            self:StartTimer("BlazingInferno", 30);
        end
    end,

    OnInstakill = function(self, actorGUID, actorName, targetGUID, targetName, spellId, spellName)
        if spellId == 78362 then
            -- No longer apply to normal mode.
            self:TriggerError("ImpaleFailed", targetName);
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);

        self:InterruptTimer("ImpaleEnd");

        self:StartTimer("CheckHead");
    end,

    OnPhaseTwo = function(self)
        self:SetPhase(2);

        self:InterruptTimer("Spew");
        self:InterruptTimer("Pillar");

        self:StartTimer("ImpaleEnd");
        self:StartTimer("ImpaleEndSoon");
    end,

    OnPhaseTwoSoon = function(self)
        self:SearchUnit(MAIN_ID, self.OnPhaseOne);

        if self:GetSetting("Transition") then
            self:Announce(self:Localize("Transition"));
        end
    end,

    OnPillar = function(self)
        self:AlertIncomingSpell("caster/high", 78006);
    end,

    OnMangle = function(self, guid)
        self:AlertSpell("shield/critical", 78412, guid);
    end,

    OnCheckHead = function(self)
        self:SearchUnit(HEAD_ID, self.OnPhaseTwo);
    end,

    OnMoltenTentrum = function(self, name)
        self:TriggerError("MoltenTentrum", name);
    end,

    OnArmageddon = function(self)
        self:AlertIncomingSpell("caster/critical", 92177);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(MAIN_ID, false, false); -- ID, volatile, bypass
M:DefineBerserk(600);

-- Events

M:RegisterCastEvent({77689, 77690, 77839}, "Spew");
M:RegisterCastEvent(92154, "BlazingInferno");
M:RegisterCastEvent(92177, "Armageddon");
M:RegisterEffectGainEvent({78412, 89773}, "Mangle");
M:RegisterEffectGainEvent(78006, "Pillar");
M:RegisterEffectGainEvent(78403, "MoltenTentrum");

-- Timers

M:DefineTimer("CheckHead", "invisible", 10, nil, "CheckHead", true);
M:DefineTimer("ImpaleEndSoon", "invisible", 25, nil, "ImpaleEndSoon", true);
M:DefineVoidTimer("ImpaleEnd", "all/high", 30);
M:DefineRepeatingVoidTimer("Spew/77689", "healer/high", 30);
M:DefineRepeatingVoidTimer("BlazingInferno/92154", "offensive/high", 35);
M:DefineInitialRepeatingVoidTimer("Pillar/78006", "caster/high", 32.5);
M:DefineInitialRepeatingVoidTimer("Mangle/78412", "shield/critical", 95);

-- Actions

M:DeclareAction("Pillar");
M:AddTriggerTimerToAction("Pillar", "Pillar");
M:AddCallbackToAction("Pillar", "OnPillar");

M:DeclareAction("Spew", 8);
M:AddTriggerTimerToAction("Spew", "Spew");

M:DeclareAction("Mangle");
M:AddTriggerTimerToAction("Mangle", "Mangle");
M:AddCallbackToAction("Mangle", "OnMangle", "targetguid");

M:DeclareAction("ImpaleEndSoon");
M:AddCallbackToAction("ImpaleEndSoon", "OnPhaseTwoSoon");

M:DeclareAction("CheckHead");
M:AddCallbackToAction("CheckHead", "OnCheckHead");

M:DeclareAction("MoltenTentrum");
M:AddCallbackToAction("MoltenTentrum", "OnMoltenTentrum", "targetname");

M:DeclareAction("BlazingInferno");
M:AddTriggerTimerToAction("BlazingInferno", "BlazingInferno");

M:DeclareAction("Armageddon", 5);
M:AddCallbackToAction("Armageddon", "OnArmageddon");

-- Errors

M:DefineBossAggroError();
M:DefineCustomError("ImpaleFailed");
M:DefineCustomError("MoltenTentrum");
M:DefineAvoidDamageError(88287); -- Massive Crash
M:DefineAvoidDamageError(77971); -- Pillar
M:DefineAvoidDamageError(92182); -- Armageddon