﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local addsAlgorithm = {
    label = {
        ["enUS"] = "Add",
        ["frFR"] = "Add",
    },
    color = {0, 1, 1},
    allowDead = true,
    rate = 0.10,
    digits = 2,
    useMultiples = false,

    GetValue = function(self, uid)
        return self.module:GetUnreleasedAdds();
    end,

    GetValueMasked = function(self, guid)
        return self.module:GetUnreleasedAdds();
    end,
};

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "FlashFreeze",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 77699,
                 symbol = 1,
                 multiple = true },
    },
    {
        id = "Remedy",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Announce Remedy",
            ["frFR"] = "Annoncer le remède",
        },
        explain = {
            ["enUS"] = "Announce when the boss gets the Remedy.",
            ["frFR"] = "Annonce quand le boss obtient le remède.",
        },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["RedTrigger"] = "red",
        ["BlueTrigger"] = "blue",
        ["GreenTrigger"] = "green",
        ["DarkTrigger"] = "dark",
        ["VialSwitch"] = "Vial switch",
        ["DebilitatingSlimeEnd"] = "Debilitating Slime: end",
        ["BossHeal"] = "The boss is healing!",
        ["NoHealDispel"] = "'s heal wasn't dispelled quick enough.",
    },
    ["frFR"] = {
        ["RedTrigger"] = "rouge",
        ["BlueTrigger"] = "bleu",
        ["GreenTrigger"] = "vert",
        ["DarkTrigger"] = "sombre",
        ["VialSwitch"] = "Autre fiole",
        ["DebilitatingSlimeEnd"] = "Fin de gelée débilitante",
        ["BossHeal"] = "Le boss se soigne !",
        ["NoHealDispel"] = "a pu bénéficier de son soin dissipable.",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Maloriak", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    flashFreeze = { },
    abominations = 18,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(60);
        self:SetScoreBenchmarks(300, 420);
        self:SetHeroicScoreBenchmarks(420, 720);

        self:RegisterAnyYell(self:Localize("RedTrigger"),   self.OnVialChange, self, "red");
        self:RegisterAnyYell(self:Localize("BlueTrigger"),  self.OnVialChange, self, "blue");
        self:RegisterAnyYell(self:Localize("GreenTrigger"), self.OnVialChange, self, "green");
        self:RegisterAnyYell(self:Localize("DarkTrigger"),  self.OnVialChange, self, "dark");

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:SetExtraAlgorithm(addsAlgorithm);
        end
    end,

    OnBossFound = function(self, uid)
        self:RegisterHealthThreshold(0.25, self:Localize("PhaseTwo"));
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    -- *******
    -- Methods
    -- *******

    GetUnreleasedAdds = function(self)
        return self.abominations, 18;
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:InterruptTimer("VialSwitch");
            self:InterruptTimer("ScorchingBlast");
            self:InterruptTimer("FlashFreeze");

            self:ChangeMinRange(6);
        end
    end,

    OnReleaseStart = function(self)
        self:StartTimer("ReleaseSuccess");
    end,

    OnReleaseInterrupt = function(self)
        self:InterruptTimer("ReleaseSuccess");
    end,

    OnReleaseSuccess = function(self)
        self.abominations = max(0, self.abominations - 3);
    end,

    OnVialChange = function(self, color)
        self:TriggerTimer("VialSwitch");
        self:InterruptTimer("ScorchingBlast");
        self:InterruptTimer("FlashFreeze");

        if color == "dark" then
            self:StartTimer("EngulfingDarkness", 16.5);
            self:ChangeMinRange(0);

            self:StartTimer("VialSwitch", 100);
      else
            if color == "red" then
                self:StartTimer("ScorchingBlast");
                self:ChangeMinRange(0);

        elseif color == "blue" then
                self:StartTimer("FlashFreeze");
                self:ChangeMinRange(10);

        elseif color == "green" then
                self:ChangeMinRange(0);
            end

            self:StartTimer("VialSwitch");
        end
    end,

    OnMagmaJets = function(self)
        self:AlertIncomingSpell("tank/high", 78194);
    end,

    OnArcaneStorm = function(self)
        self:InterruptMob();
    end,

    OnRemedyTick = function(self, name)
        if name == self:GetBossName() then
            self:TriggerError("NoHealDispel", name);
        end
    end,

    OnRemedyStart = function(self)
        if self:GetSetting("Remedy") then
            self:Announce(self:Localize("BossHeal"));
        end
    end,

    OnConsumingFlames = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 77786, guid);
        end
    end,

    OnBitingChill = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/normal", 77760, guid);
        end
    end,

    OnSludge = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 92930, guid);
        end
    end,

    OnFlashFreezeTarget = function(self, guid)
        tinsert(self.flashFreeze, guid);
    end,

    OnFlashFreeze = function(self)
        if self:GetSetting("FlashFreeze") then
            self:AnnounceUnitListSpell(self.flashFreeze, 77699, true, 10);
        end
        wipe(self.flashFreeze);
    end,

    OnEngulfingDarkness = function(self, guid)
        self:AlertSpell("shield/high", 92754, guid);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(41378, false, false); -- ID, volatile, bypass
M:DefineBerserk(420, 720);

-- Events

M:RegisterCastEvent(77991, "PhaseTwo");
M:RegisterCastEvent(77569, "ReleaseStart");
M:RegisterCastEvent(78194, "MagmaJets");
M:RegisterCastEvent(77908, "ArcaneStorm");
M:RegisterCastEvent(77679, "ScorchingBlast");
M:RegisterInterruptEvent(77569, "ReleaseInterrupt");
M:RegisterHealEvent(77912, "RemedyTick", "PERIODIC");
M:RegisterEffectGainEvent(77912, "RemedyStart");
M:RegisterEffectGainEvent(77786, "ConsumingFlames");
M:RegisterEffectGainEvent(77760, "BitingChill");
M:RegisterEffectGainEvent(92930, "Sludge");
M:RegisterEffectGainEvent(77699, "FlashFreezeTarget");
M:RegisterEffectGainEvent(77615, "DebilitatingSlime");
M:RegisterEffectGainEvent(92754, "EngulfingDarkness");
M:RegisterEffectFadeEvent(77699, "FlashFreezeFade");

-- Timers

M:DefineTimer("ReleaseSuccess", "invisible", 2, nil, "ReleaseSuccess", true);
M:DefineVoidTimer("DebilitatingSlimeEnd", "all/high", 15);
M:DefineInitialVoidTimer("VialSwitch", "all/high", 50, 15);
M:DefineRepeatingVoidTimer("ScorchingBlast/77679", "all/high", 17.5);
M:DefineRepeatingVoidTimer("FlashFreeze/77699", "caster/high", 25);
M:DefineRepeatingVoidTimer("EngulfingDarkness/92754", "shield/high", 12);

-- Zones

M:DefineAnchoredZone("BitingChill", "all/high", "red", 8, 10, true);
M:DefineAnchoredZone("FlashFreeze", "all/high", "red", 10, 10, true);
M:DefineAimedZone("Sludge", "all/high", "red", 6, 12, true);

-- Actions

M:DeclareAction("PhaseTwo");
M:AddCallbackToAction("PhaseTwo", "OnPhaseTwo");

M:DeclareAction("ReleaseStart");
M:AddCallbackToAction("ReleaseStart", "OnReleaseStart");

M:DeclareAction("ReleaseInterrupt");
M:AddCallbackToAction("ReleaseInterrupt", "OnReleaseInterrupt");

M:DeclareAction("ReleaseSuccess");
M:AddCallbackToAction("ReleaseSuccess", "OnReleaseSuccess");

M:DeclareAction("MagmaJets");
M:AddCallbackToAction("MagmaJets", "OnMagmaJets");

M:DeclareAction("ArcaneStorm");
M:AddCallbackToAction("ArcaneStorm", "OnArcaneStorm");

M:DeclareAction("ScorchingBlast");
M:AddTriggerTimerToAction("ScorchingBlast", "ScorchingBlast");

M:DeclareAction("RemedyTick");
M:AddCallbackToAction("RemedyTick", "OnRemedyTick", "targetname");

M:DeclareAction("RemedyStart", 2.5);
M:AddCallbackToAction("RemedyStart", "OnRemedyStart");

M:DeclareAction("ConsumingFlames");
M:AddCallbackToAction("ConsumingFlames", "OnConsumingFlames", "targetguid");

M:DeclareAction("BitingChill");
M:AddShowZoneToAction("BitingChill", "BitingChill");
M:AddCallbackToAction("BitingChill", "OnBitingChill", "targetguid");

M:DeclareAction("Sludge");
M:AddShowZoneToAction("Sludge", "Sludge");

M:DeclareAction("FlashFreezeTarget");
M:AddShowZoneToAction("FlashFreezeTarget", "FlashFreeze");
M:AddCallbackToAction("FlashFreezeTarget", "OnFlashFreezeTarget", "targetguid");
M:AddExternalActionToAction("FlashFreezeTarget", "FlashFreeze");

M:DeclareAction("FlashFreeze", 5, 0.5);
M:AddTriggerTimerToAction("FlashFreeze", "FlashFreeze");
M:AddCallbackToAction("FlashFreeze", "OnFlashFreeze");

M:DeclareAction("FlashFreezeFade");
M:AddHideZoneToAction("FlashFreezeFade", "FlashFreeze");

M:DeclareAction("DebilitatingSlime", 5);
M:AddStartTimerToAction("DebilitatingSlime", "DebilitatingSlimeEnd");

M:DeclareAction("EngulfingDarkness", 5);
M:AddTriggerTimerToAction("EngulfingDarkness", "EngulfingDarkness");
M:AddCallbackToAction("EngulfingDarkness", "OnEngulfingDarkness", "targetguid");

-- Errors

M:DefineBossAggroError();
M:DefineCustomError("NoHealDispel", 3);
M:DefineAvoidDamageError(77908, 1); -- Arcane Storm
M:DefineAvoidDamageError(78208, 0); -- Absolute Zero
M:DefineAvoidDamageError({78095, 78124, 78194, 93018, 93022}, 2); -- Magma Jets
M:DefineAvoidBombError(77760, 3); -- Biting Chill