﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local NEFARIAN_ID = 41376;
local ONYXIA_ID   = 41270;

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "DisableWaterCollision",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Disable water collision",
            ["frFR"] = "Désactiver la collision avec l'eau",
        },
        explain = {
            ["enUS"] = "Enabling this option can make your life easier when swimming in the lava.",
            ["frFR"] = "Activer cette option peut vous faciliter la vie quand vous nagez dans la lave.",
        },
    },
    {
        id = "Dominion",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 79318,
                 symbol = 1,
                 multiple = true,
                 heroic = true },
    },
    {
        id = "Cinder",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 79339,
                 symbol = 1,
                 multiple = true,
                 heroic = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Electrocute"] = "Electrocute %d",
        ["ElectrocuteTrigger"] = "air crackles with electricity",
        ["CinderAlertTitle"] = "You're a bomb!",
        ["CinderAlertExplain"] = "Get out!",
        ["Shadowblaze"] = "You're burning!",
        ["PhaseThreeTrigger"] = "I have tried to be an accommodating host",
    },
    ["frFR"] = {
        ["Electrocute"] = "Electrocution %d",
        ["ElectrocuteTrigger"] = "crépite dans l'air",
        ["CinderAlertTitle"] = "Vous êtes une bombe !",
        ["CinderAlertExplain"] = "Eloignez-vous !",
        ["Shadowblaze"] = "Vous brûlez !",
        ["PhaseThreeTrigger"] = "J'ai tout fait pour être un hôte accomodant",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Nefarian", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    addsLeft = 3,
    cinder = { },
    dominion = { },
    landingSoon = false,
    blazeFrequency = 30,
    blazeMinFrequency = 15,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(80);
        self:SetScoreBenchmarks(400, 600);
        self:DefineNearVictory(0.300, 0.75, 90);

        self:RegisterAdd(ONYXIA_ID);
        self:SetMainUnit(ONYXIA_ID);

        self:RegisterAnyYell(self:Localize("ElectrocuteTrigger"), "Electrocute");
        self:RegisterAnyYell(self:Localize("PhaseThreeTrigger"), self.OnPhaseThreeExact, self);
    end,

    OnBossFound = function(self, uid)
        local i;
        for i=1, 9 do
            self:RegisterHealthThreshold(0.10 * i + 0.005, self:FormatLoc("Electrocute", 10-i));
        end
        self:SetHealthThresholdDelay(0.05); -- Electrocutes are warned 5% ahead and not the default 10%.
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();

        if self:IsHeroic() then
            self.blazeMinFrequency = 5;

            if not self:IsIncomplete() then
                self:StartTimer("Dominion", 50);
            end
        end

        if self:GetSetting("DisableWaterCollision") then
            self:OverrideCVar("cameraWaterCollision", 0);
        end
    end,

    OnDeath = function(self, guid, name)
        local mobID = self:GetMobID(guid);
        if mobID == 41948 then
            self.addsLeft = max(0, self.addsLeft - 1);
            if not self.landingSoon and (self:IsHeroic() or self.addsLeft == 0) then
                self:InterruptTimer("PhaseThree");
                self.landingSoon = true;
            end
        end
    end,

    OnDamage = function(self, actorGUID, actorName, targetGUID, targetName, spellId, spellName, isPeriodic, amount, overkill, school, absorbed)
        if (not spellId) and actorGUID == self:GetBossGUID() then
            if self:GetPhase() <= 2 and self.landingSoon then
                self:OnPhaseThree();
            end
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:InterruptTimer("Dominion");

            self:StartTimer("PhaseThree");

            self:SetMainUnit(nil);
        end
    end,

    OnPhaseThreeExact = function(self)
        self:StartTimer("Shadowblaze", 12);

        self:OnPhaseThree();
    end,

    OnPhaseThree = function(self)
        if self:SetIncreasingPhase(3) then
            self:TriggerTimer("PhaseThree");

            self:SetMainUnit(NEFARIAN_ID);
        end
    end,

    OnBlastNova = function(self, guid)
        self:InterruptMob(guid);
    end,

    OnSpark = function(self)
        self:StartTimer("Shadowblaze", self.blazeFrequency);
        self.blazeFrequency = max(self.blazeMinFrequency, self.blazeFrequency - 5);
    end,

    OnDominionTarget = function(self, guid)
        tinsert(self.dominion, guid);
    end,

    OnDominion = function(self)
        if self:GetSetting("Dominion") then
            self:AnnounceUnitListSpell(self.dominion, 79318, true, 15);
        end
        wipe(self.dominion);
    end,

    OnCinderTarget = function(self, guid)
        if self:IsMe(guid) then
            self:SuperAlert(self:Localize("CinderAlertTitle"), self:Localize("CinderAlertExplain"), 8, true);
        end
        tinsert(self.cinder, guid);
    end,

    OnCinder = function(self)
        if self:GetSetting("Cinder") then
            self:AnnounceUnitListSpell(self.cinder, 79339, true, 8);
        end
        wipe(self.cinder);
    end,

    OnShadowblaze = function(self, guid)
        if self:IsMe(guid) and self:CheckCooldown("Shadowblaze", 5) then
            self:AlertCategorized("all/normal", self:Localize("Shadowblaze"));
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(NEFARIAN_ID, false, false); -- ID, volatile, bypass
M:DeclareNPC(ONYXIA_ID, false, false); -- Onyxia is affiliated with the encounter
M:DefineBerserk(nil, 630);

-- Events

M:RegisterCastEvent(80734, "BlastNova");
M:RegisterEffectGainEvent(79318, "DominionTarget");
M:RegisterEffectGainEvent(79339, "CinderTarget");
M:RegisterDamageEvent(78621, "ShadowflameBarrage");
M:RegisterDamageEvent(81007, "Shadowblaze");

-- Timers

M:DefineTimer("Shadowblaze/81007", "tank/high", 30, nil, "Spark", true);
M:DefineVoidTimer("PhaseThree", "all/high", 180);
M:DefineVoidTimer("Electrocute/81272", "shield/critical", 6);
M:DefineRepeatingVoidTimer("Dominion/79318", "all/high", 15);

-- Zones

M:DefineAnchoredZone("Cinder", "all/critical", "red", 10, 8, true);

-- Actions

M:DeclareAction("BlastNova");
M:AddCallbackToAction("BlastNova", "OnBlastNova", "actorguid");

M:DeclareAction("Spark");
M:AddCallbackToAction("Spark", "OnSpark");

M:DeclareAction("ShadowflameBarrage");
M:AddCallbackToAction("ShadowflameBarrage", "OnPhaseTwo");

M:DeclareAction("DominionTarget");
M:AddCallbackToAction("DominionTarget", "OnDominionTarget", "targetguid");
M:AddExternalActionToAction("DominionTarget", "Dominion");

M:DeclareAction("Dominion", 10, 0.5);
M:AddTriggerTimerToAction("Dominion", "Dominion");
M:AddCallbackToAction("Dominion", "OnDominion");

M:DeclareAction("CinderTarget");
M:AddShowZoneToAction("CinderTarget", "Cinder");
M:AddCallbackToAction("CinderTarget", "OnCinderTarget", "targetguid");
M:AddExternalActionToAction("CinderTarget", "Cinder");

M:DeclareAction("Cinder", 10, 0.5);
M:AddCallbackToAction("Cinder", "OnCinder");

M:DeclareAction("Shadowblaze");
M:AddCallbackToAction("Shadowblaze", "OnShadowblaze", "targetguid");

M:DeclareAction("Electrocute");
M:AddStartTimerToAction("Electrocute", "Electrocute");

-- Errors

M:DefineBossAggroError(60);
M:DefineAvoidDamageError(78999); -- Electrical Overload
M:DefineAvoidDamageError(79347); -- Explosive Cinders
M:DefineAvoidDamageError({77939, 77942, 77943, 77944}, 1); -- Electric Discharge (WTF?)
M:DefineAvoidDamageError(80735, 1); -- Blast Nova
M:DefineAvoidDamageError(81007, 1); -- Shadowblaze