﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local Utility = Root:GetOrNewModule("Utility");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local function vitalSparkFunc(self, uid)
    local guid = UnitGUID(uid);
    local cache = self.module.sparkCount;
    local pending, fill = 0, 0;
    local timer = "";
    local _, pendingCount = self.module:GetEffectInfo(uid, 99262);
    local timeLeft = self.module:GetEffectInfo(uid, 99263);
    if pendingCount then
        pending = pendingCount;
        cache[guid] = pending;
    end
    if timeLeft then
        pending = cache[guid] or pending;
        fill = min(1, timeLeft / 15);
        timer = Utility:FormatCountdown("%S'%C ", timeLeft, true, true, true);
    end
    self:SetBarFill(fill);
    self:SetBarColor(0, 1, 0);
    self:SetText(timer..pending);
    if fill > 0 then
        local a = 0.5 * (1 + cos(GetTime() * 180));
        if timeLeft <= 3 then
            self:SetTextColor(1, a, a);
      else
            self:SetTextColor(a, 0.5, 1-a);
        end
  else
        self:SetTextColor(1, 1, 1);
    end
end

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Relay"] = "Your turn",
        ["RelaySoon"] = "You relay soon!",
        ["NextBlade"] = "Next Blade",
        ["ActiveBlade"] = "Blade: end",
        ["TormentSay"] = "> %d stacks! <",
        ["VitalFlame"] = "Vital Flame: end",
        ["VitalSparkUI"] = "Sparks",
    },
    ["frFR"] = {
        ["Relay"] = "Votre tour",
        ["RelaySoon"] = "Vous relayez bientôt !",
        ["NextBlade"] = "Lame suivante",
        ["ActiveBlade"] = "Lame : fin",
        ["TormentSay"] = "> %d empilements ! <",
        ["VitalFlame"] = "Flamme vitale : fin",
        ["VitalSparkUI"] = "Etincelles",
    },
};

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "ShardRelay",
        type = "STRING",
        max = 25,
        default = "Player",
        label = {
            ["enUS"] = "Shards: I am after...",
            ["frFR"] = "Eclats : je suis après...",
        },
        explain = {
            ["enUS"] = "If you are assigned to shards, enter here the name of the player preceding you.",
            ["frFR"] = "Si vous êtes assigné sur les éclats, entrez ici le nom du joueur qui vous précède.",
        },
    },
    {
        id = "ShardCount",
        type = "NUMBER",
        min = 5,
        max = 20,
        step = 1,
        default = 12,
        label = {
            ["enUS"] = "Shards: warning threshold",
            ["frFR"] = "Eclats : seuil d'avertissement",
        },
        explain = {
            ["enUS"] = "If you are assigned to shards, specify here the number of Torment stacks the player preceding you should accumulate before you relay him/her.",
            ["frFR"] = "Si vous êtes assigné sur les éclats, spécifiez ici le nombre d'empilements de Tourment que le joueur qui vous précède doit accumuler avant que vous ne le relayiez.",
        },
    },
    {
        id = "ShardSay",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Shards: announce stacks",
            ["frFR"] = "Eclats : annoncer les empilements",
        },
        explain = {
            ["enUS"] = "Enable this option to /say the number of Torment stacks on your character when it's close to the warning threshold.",
            ["frFR"] = "Activer cette option pour /dire le nombre d'empilements de Tourment sur votre personnage quand il est proche du seuil d'avertissement.",
        },
    },
    {
        id = "Countdown",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 99516,
                 symbol = 1,
                 multiple = true,
                 heroic = true },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Baleroc", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    shardRelay = "",
    shardState = 0,
    countdownTargets = { },
    sparkCount = { }, -- Used to cache the number of spark stacks while vital flame is active.

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(70);
        self:SetScoreBenchmarks(210, 360);

        self:EnumerateRaidByRole("HEALER");
        local healerList = { };
        local i, name;
        for i=1, self:GetEnumerationSize() do
            name = UnitName(self:GetEnumerationUID(i));
            tinsert(healerList, name);
        end
        self:StartUnitMonitor(self:Localize("VitalSparkUI"), vitalSparkFunc, healerList);
    end,

    OnEngaged = function(self)
        local list = { };
        self:FillPlayerList(list, self:GetSetting("ShardRelay"));
        if #list > 0 then
            self.shardRelay = list[1];
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnShards = function(self)
        self:AlertIncomingSpell("offensive/high", 99259);
    end,

    OnInfernoBlade = function(self)
        self:AlertIncomingSpell("shield/high", 99350);
    end,

    OnDecimationBlade = function(self)
        self:AlertIncomingSpell("shield/high", 99352);
    end,

    OnCountdownTarget = function(self, guid)
        tinsert(self.countdownTargets, guid);
    end,

    OnCountdown = function(self)
        if self:GetSetting("Countdown") then
            self:AnnounceUnitListSpell(self.countdownTargets, 99516, true, 8);
        end
        wipe(self.countdownTargets);
    end,

    OnVitalFlame = function(self, guid)
        if self:IsMe(guid) then
            self:StartTimer("VitalFlame");
        end
    end,

    OnTormentGain = function(self, guid, stackAmount)
        local threshold = self:GetSetting("ShardCount");
        if guid == self.shardRelay then
            if stackAmount >= threshold and self.shardState == 1 then
                self.shardState = 2;
                self:TriggerTimer("Relay");

        elseif self.shardState == 0 then
                self.shardState = 1;
                self:StartTimer("Relay", threshold - stackAmount);
                self:Alert(self:Localize("RelaySoon"));
            end
        end
        if self:IsMe(guid) and self:GetSetting("ShardSay") then
            if stackAmount >= (self:GetSetting("ShardCount") - 3) then
                self:Say(self:FormatLoc("TormentSay", stackAmount));
            end
        end
    end,

    OnTormentFade = function(self, guid)
        if guid == self.shardRelay then
            self.shardState = 0;
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(53494, false, false); -- ID, volatile, bypass
M:DefineBerserk(360);

-- Events

M:RegisterCastEvent(99259, "Shards");
M:RegisterCastEvent(99350, "InfernoBlade");
M:RegisterCastEvent(99352, "DecimationBlade");
M:RegisterEffectGainEvent(99516, "CountdownTarget");
M:RegisterEffectGainEvent(99263, "VitalFlame");
M:RegisterEffectGainEvent(99256, "TormentGain");
M:RegisterEffectGainEvent({99257, 99489}, "TormentedGain");
M:RegisterEffectFadeEvent({99350, 99352}, "BladeFade");
M:RegisterEffectFadeEvent(99256, "TormentFade");
M:RegisterEffectFadeEvent({99257, 99489}, "TormentedFade", "LAST");

-- Timers

M:DefineVoidTimer("Relay", "all/critical", 10);
M:DefineVoidTimer("Countdown/99516", "all/critical", 8);
M:DefineVoidTimer("VitalFlame", "healer/critical", 15);
M:DefineVoidTimer("ActiveBlade", "shield/high", 15);
M:DefineInitialVoidTimer("NextBlade", "shield/high", 30);
M:DefineRepeatingVoidTimer("Shards/99259", "dps/high", 35);

-- Zones

M:DefineAnchoredZone("Tormented", "all/high", "red", 8, 40, true);

-- Actions

M:DeclareAction("Shards");
M:AddTriggerTimerToAction("Shards", "Shards");
M:AddCallbackToAction("Shards", "OnShards");

M:DeclareAction("InfernoBlade");
M:AddCallbackToAction("InfernoBlade", "OnInfernoBlade");
M:AddTriggerTimerToAction("InfernoBlade", "NextBlade");
M:AddStartTimerToAction("InfernoBlade", "ActiveBlade");

M:DeclareAction("DecimationBlade");
M:AddCallbackToAction("DecimationBlade", "OnDecimationBlade");
M:AddTriggerTimerToAction("DecimationBlade", "NextBlade");
M:AddStartTimerToAction("DecimationBlade", "ActiveBlade");

M:DeclareAction("CountdownTarget");
M:AddCallbackToAction("CountdownTarget", "OnCountdownTarget", "targetguid");
M:AddExternalActionToAction("CountdownTarget", "Countdown");

M:DeclareAction("Countdown", 5, 0.5);
M:AddCallbackToAction("Countdown", "OnCountdown");
M:AddStartTimerToAction("Countdown", "Countdown");

M:DeclareAction("VitalFlame");
M:AddCallbackToAction("VitalFlame", "OnVitalFlame", "targetguid");

M:DeclareAction("TormentedGain");
M:AddShowZoneToAction("TormentedGain", "Tormented");

M:DeclareAction("TormentedFade");
M:AddShowZoneToAction("TormentedFade", "Tormented");

M:DeclareAction("TormentGain");
M:AddCallbackToAction("TormentGain", "OnTormentGain", "targetguid", "stack");

M:DeclareAction("TormentFade");
M:AddCallbackToAction("TormentFade", "OnTormentFade", "targetguid");

M:DeclareAction("BladeFade");
M:AddTriggerTimerToAction("BladeFade", "ActiveBlade");
M:AddStartTimerToAction("BladeFade", "NextBlade");

-- Errors

M:DefineBossAggroError();
M:DefineAvoidDamageError(99516);