﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Spinners"] = "Spinners",
        ["Drone"] = "Drone",
        ["DevastationTitle"] = "Devastation",
        ["DevastationExplain"] = "Go down.",
    },
    ["frFR"] = {
        ["Spinners"] = "Tisseuses",
        ["Drone"] = "Ouvrière",
        ["DevastationTitle"] = "Dévastation",
        ["DevastationExplain"] = "Allez en bas.",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("BethTilac", nil, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    boomCount = 0,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(60);
        self:SetScoreBenchmarks(340, 430);

        self:RegisterRespawnableAdd(53635);
        self:IgnoreAdds();

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:SetExtraAlgorithm("power");
        end
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:InterruptTimer("PhaseTwo");
        end
    end,

    OnSmolderingDevastation = function(self)
        if self:GetEffectInfo("player", 100048) then
            self:SuperAlert(self:Localize("DevastationTitle"), self:Localize("DevastationExplain"), 8, true);
      else
            self:AlertIncomingSpell("all/critical", 99052);
        end

        self.boomCount = self.boomCount + 1;
        if self.boomCount == 3 then
            self:InterruptTimer("Spinners");
            self:InterruptTimer("Drone");

            self:ScheduleAction("PhaseTwo");
        end
    end,

    OnFixate = function(self, guid) -- Heroic.
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 99559, guid);
        end
    end,

    OnKiss = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 99476, guid);
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(52498, false, false); -- ID, volatile, bypass

-- Events

M:RegisterCastEvent(99052, "SmolderingDevastation");
M:RegisterEffectGainEvent({99526, 99559}, "Fixate");
M:RegisterEffectGainEvent({99476, 99506}, "KissGain", "FIRST");
M:RegisterEffectFadeEvent({99476, 99506}, "KissFade");

-- Timers

M:DefineInitialRepeatingTimer("Drone", "offensive/high", 60, 45, "Drone", true);
M:DefineInitialVoidTimer("Spinners", "offensive/high", 15, 12.5);
M:DefineInitialVoidTimer("PhaseTwo", "all/high", 260);

-- Zones

M:DefineAnchoredZone("Kiss", "all/high", "red", 10, 20, true);

-- Actions

M:DeclareAction("PhaseTwo", nil, 10);
M:AddCallbackToAction("PhaseTwo", "OnPhaseTwo");

M:DeclareAction("SmolderingDevastation");
M:AddStartTimerToAction("SmolderingDevastation", "Spinners");
M:AddCallbackToAction("SmolderingDevastation", "OnSmolderingDevastation");

M:DeclareAction("Fixate");
M:AddCallbackToAction("Fixate", "OnFixate", "targetguid");

M:DeclareAction("KissGain");
M:AddShowZoneToAction("KissGain", "Kiss");
M:AddCallbackToAction("KissGain", "OnKiss", "targetguid");

M:DeclareAction("KissFade");
M:AddHideZoneToAction("KissFade", "Kiss");

M:DeclareAction("Drone");
-- No action

-- Errors

M:DefineAvoidDamageError(99052); -- Smoldering Devastation
M:DefineAvoidDamageError(99076); -- Meteor Burn
M:DefineAvoidDamageError(97079); -- Seeping Venom