﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Fire"] = "You're burning!",
    },
    ["frFR"] = {
        ["Fire"] = "Vous brûlez !",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("OccuThar", nil, myLocalization);

Module:InstallExtra(M, {
    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(55);
        self:SetScoreBenchmarks(150, 300);
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnFire = function(self, guid)
        if self:IsMe(guid) and self:CheckCooldown("Fire", 5) then
            self:AlertCategorized("all/normal", self:Localize("Fire"));
        end
    end,

    OnEyes = function(self)
        self:AlertIncomingSpell("all/high", 96920);
        self:StartTimer("Boom");
    end,

    OnSearingShadows = function(self, guid)
        self:AlertSpell("all/high", 96913, guid);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(52363, false, false); -- ID, volatile, bypass
M:DefineBerserk(300);

-- Events

M:RegisterDamageEvent({96883, 97212}, "Fire");
M:RegisterCastEvent(96884, "FireCast");
M:RegisterCastEvent(96920, "Eyes");
M:RegisterEffectGainEvent(96913, "SearingShadows");

-- Timers

M:DefineVoidTimer("Boom/96968", "all/high", 14, nil);
M:DefineInitialRepeatingVoidTimer("Eyes/96920", "dps/high", 60, 30);
M:DefineInitialRepeatingVoidTimer("SearingShadows/96913", "shield/high", 22, 11);
M:DefineInitialRepeatingVoidTimer("Fire/96883", "all/high", 30);

-- Actions

M:DeclareAction("Fire");
M:AddCallbackToAction("Fire", "OnFire", "targetguid");

M:DeclareAction("FireCast", 5);
M:AddTriggerTimerToAction("FireCast", "Fire");

M:DeclareAction("Eyes");
M:AddCallbackToAction("Eyes", "OnEyes");
M:AddTriggerTimerToAction("Eyes", "Eyes");

M:DeclareAction("SearingShadows");
M:AddCallbackToAction("SearingShadows", "OnSearingShadows", "targetguid");
M:AddTriggerTimerToAction("SearingShadow", "SearingShadow");

-- Errors

M:DefineBossAggroError();
M:DefineAvoidDamageError({96883, 97212}, 2);