﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "Leap",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 98476,
                 symbol = 8,
                 silent = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Staghelm", mySettings, nil);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    leapAttempts = 0,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(70);
        self:SetScoreBenchmarks(360, 600);

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:SetExtraAlgorithm("power");
        end
    end,

    -- *******
    -- Methods
    -- *******

    CheckLeap = function(self)
        local tGUID = self:GetTargetInfo();
        local aGUID = self:GetTankUnit();
        if tGUID and tGUID ~= aGUID then
            if self:IsMe(tGUID) then
                self:AlertSpell("all/high", 98476, tGUID);
            end
            if self:GetSetting("Leap") then
                self:PlaceSymbol(tGUID, 8, 4);
            end
            self:ShowZone("Leap", tGUID);
      else
            self.leapAttempts = self.leapAttempts - 1;
            if self.leapAttempts > 0 then
                self:StartTimer("CheckLeap");
            end
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnOrbs = function(self)
        self:AlertIncomingSpell("tank/high", 98451);
    end,

    OnLeap = function(self)
        self.leapAttempts = 10;
        self:InterruptTimer("CheckLeap");
        self:CheckLeap();
    end,

    OnBombApplied = function(self, guid)
        local timeLeft = self:GetEffectInfo(guid, 98450);
        if timeLeft then
            local EventList = self:GetWidget("EventList");
            if EventList then
                EventList:StartEvent("BombHack:"..guid, nil, timeLeft - 7, 0, self.OnBomb, self, guid);
            end
        end
    end,

    OnBomb = function(self, guid)
        if self:GetEffectInfo(guid, 98450) then
            self:ShowZone("Bomb", guid);
            if self:IsMe(guid) then
                self:Bomb(guid);
            end
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(52571, false, false); -- ID, volatile, bypass
M:DefineBerserk(600);

-- Events

M:RegisterCastEvent(98451, "Orbs");
M:RegisterCastEvent({98535, 98476}, "Leap");
M:RegisterEffectGainEvent(98450, "Bomb");

-- Timers

M:DefineTimer("CheckLeap", "invisible", 0.05, nil, "CheckLeap", true);

-- Zones

M:DefineAimedZone("Leap", "all/high", "red", 9, 60, true);
M:DefineAnchoredZone("Bomb", "all/high", "red", 12, 10, true);

-- Actions

M:DeclareAction("Orbs");
M:AddCallbackToAction("Orbs", "OnOrbs");

M:DeclareAction("Leap", 3);
M:AddCallbackToAction("Leap", "OnLeap");

M:DeclareAction("CheckLeap");
M:AddCallbackToAction("CheckLeap", "CheckLeap");

M:DeclareAction("Bomb");
M:AddCallbackToAction("Bomb", "OnBombApplied", "targetguid");

-- Errors

M:DefineBossAggroError();
M:DefineAvoidBombError(98450); -- Seeds