﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local VOLCANUS_ID  = 53833;
local PROTECTOR_ID = 53825;

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Transition"] = "Transition",
        ["WaitTitle"] = "Please wait...",
        ["WaitExplain"] = "Don't dispel yet!",
    },
    ["frFR"] = {
        ["Transition"] = "Transition",
        ["WaitTitle"] = "Patientez...",
        ["WaitExplain"] = "Attendez avant de dissiper!",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Volcanus", nil, myLocalization);

Module:InstallExtra(M, {
    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(30);
        self:DisableScore();

        self:RegisterAdd(PROTECTOR_ID);
        self:IgnoreAdds();

        self:DisableCombatCheck();
        self:DisableWipeCheck();
    end,

    OnBossFound = function(self, uid)
        self:DefineNearVictory(0.400, 0.80);
        self:RegisterAnyYell(self:Localize("PhaseTwoTrigger"), self.OnPhaseTwo, self);
    end,

    OnAddFound = function(self, guid, id, name)
        if id == PROTECTOR_ID then
            local EnemyList = self:GetWidget("EnemyList");
            if EnemyList then
                EnemyList:AddThreshold(guid, self:Localize("Transition"), 0.25);
            end
        end
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);

        self:SetMainUnit(PROTECTOR_ID);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:StartTimer("Transition");

            self:InterruptTimer("Roots");

            self:SetMainUnit(VOLCANUS_ID);
        end
    end,

    OnRoots = function(self)
        self:SuperAlert(self:Localize("WaitTitle"), self:Localize("WaitExplain"), 6, true);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(VOLCANUS_ID, true, false); -- ID, volatile, bypass
M:DeclareNPC(PROTECTOR_ID, false, false);

-- Events

M:RegisterCastEvent(100274, "Transition");
M:RegisterCastEvent(100146, "Roots");

-- Timers

M:DefineVoidTimer("Transition", "all/high", 35);
M:DefineInitialVoidTimer("Roots/100146", "all/high", 17);

-- Actions

M:DeclareAction("Transition");
M:AddCallbackToAction("Transition", "OnPhaseTwo");

M:DeclareAction("Roots");
M:AddTriggerTimerToAction("Roots", "Roots");
M:AddCallbackToAction("Roots", "OnRoots");