﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Special"] = "Special",
    },
    ["frFR"] = {
        ["Special"] = "Spécial",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Alizabal", nil, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    skewer = false,
    hate = false,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(60);
        self:SetScoreBenchmarks(150, 300);
    end,

    -- *******
    -- Methods
    -- *******

    ScheduleSpecial = function(self, id)
        if not self.skewer and not self.hate then
            self:StartTimer("Special");

    elseif not self.skewer and self.hate then
            self:StartTimer("Skewer", 8);

    elseif self.skewer and not self.hate then
            self:StartTimer("Hate", 8);

    elseif id == "skewer" then
            self:StartTimer("Skewer");

    elseif id == "hate" then
            self:StartTimer("Hate");
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnBladeDance = function(self)
        self:AlertIncomingSpell("all/high", 105738);
        self:InterruptTimer("Skewer");
        self:InterruptTimer("Hate");
    end,

    OnBladeDanceEnd = function(self)
        self.skewer = false;
        self.hate = false;
        self:ScheduleSpecial();
    end,

    OnSkewer = function(self)
        self.skewer = true;
        self:ScheduleSpecial("skewer");
    end,

    OnHate = function(self)
        self.hate = true;
        self:ScheduleSpecial("hate");
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(55869, false, false); -- ID, volatile, bypass
M:DefineBerserk(300);

-- Events

M:RegisterEffectGainEvent(105738, "BladeDance");
M:RegisterEffectGainEvent(104936, "Skewer");
M:RegisterEffectGainEvent(105067, "Hate");
M:RegisterEffectFadeEvent(105738, "BladeDanceEnd");

-- Timers

M:DefineInitialRepeatingVoidTimer("BladeDance/104995", "all/high", 60, 35);
M:DefineInitialVoidTimer("Special", "shield/high", 8, 6);
M:DefineVoidTimer("Skewer/104936", "shield/high", 20);
M:DefineVoidTimer("Hate/105067", "shield/high", 20);

-- Actions

M:DeclareAction("BladeDance");
M:AddTriggerTimerToAction("BladeDance", "BladeDance");
M:AddCallbackToAction("BladeDance", "OnBladeDance");

M:DeclareAction("Skewer");
M:AddTriggerTimerToAction("Skewer", "Special");
M:AddTriggerTimerToAction("Skewer", "Skewer");
M:AddCallbackToAction("Skewer", "OnSkewer");

M:DeclareAction("Hate");
M:AddTriggerTimerToAction("Hate", "Special");
M:AddTriggerTimerToAction("Hate", "Hate");
M:AddCallbackToAction("Hate", "OnHate");

M:DeclareAction("BladeDanceEnd");
M:AddCallbackToAction("BladeDanceEnd", "OnBladeDanceEnd");

-- Errors

M:DefineBossAggroError();