﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local PRIMARY_ID   = 56173; -- Cannot be damaged.
local SECONDARY_ID = 57962; -- You hit this version of Deathwing in phase 2.
local ARM1_ID      = 56167;
local ARM2_ID      = 56846;
local WING_ID      = 56168;
local TENTACLE_ID  = 56471;

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "ImpaleThreshold",
        type = "NUMBER",
        min = 1,
        max = 4,
        step = 1,
        default = 1,
        meta = { spell = 106444,
                 threshold = true },
    },
    {
        id = "Shrapnel",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 106794,
                 symbol = 1,
                 multiple = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Title"]           = "Deathwing II",
        ["DragonSoul"]      = "Dragon Soul",
        ["Maelstrom"]       = "The Maelstrom",
        ["Impale"]          = "You have %d Impale stacks!",
        ["ClearTrigger"]    = "We are one step closer|The fire|It is time|all my arcanic power",
        ["ShrapnelTitle"]   = "Shrapnel",
        ["ShrapnelExplain"] = "Use the dream!",
    },
    ["frFR"] = {
        ["Title"]           = "Aile de mort II",
        ["DragonSoul"]      = "Âme des dragons",
        ["Maelstrom"]       = "Le Maelström",
        ["Impale"]          = "Vous avez %d fois Empaler !",
        ["ClearTrigger"]    = "Nous avons fait un pas en avant|Le feu de mon|Il est temps|toute ma puissance arcanique",
        ["ShrapnelTitle"]   = "Shrapnel",
        ["ShrapnelExplain"] = "Utilisez le rêve !",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("BlackwingMadness", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    assaultCount = 0,
    shrapnel = { },

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(90);
        self:SetScoreBenchmarks(660, 900);

        self:RegisterAdd(ARM1_ID);
        self:RegisterAdd(ARM2_ID);
        self:RegisterAdd(WING_ID);
        self:RegisterRespawnableAdd(TENTACLE_ID);
        self:IgnoreAdds();

        self:RegisterAnyYell(self:Localize("ClearTrigger"), self.OnCleared, self);
    end,

    OnBossFound = function(self)
        if not self:IsLFR() then
            self:DefineNearVictory(0.100, 0.90, 120);
        end
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    OnDeath = function(self, guid, name)
        local id = self:GetMobID(guid);
        if id == ARM1_ID or id == ARM2_ID or id == WING_ID then
            self:OnSectionCompleted();

    elseif id == TENTACLE_ID then
            self:InterruptTimer("Impale");

    elseif id == PRIMARY_ID then
            self:OnCleared();
        end
    end,

    OnBossCast = function(self, unit, spellId, spellName)
        if spellName == GetSpellInfo(106708) then
            self:OnPhaseTwo();

    elseif spellName == GetSpellInfo(105853) then
            self:TriggerTimer("Hemorrhage");

    elseif spellId == 106765 then
            self:TriggerTimer("Terror");
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnPhaseOne = function(self)
        self:SetPhase(1);
    end,

    OnPhaseTwo = function(self)
        if self:SetIncreasingPhase(2) then
            self:StartTimer("Terror", 36);
        end
    end,

    OnSectionCompleted = function(self)
        self:SendViewEvent("OnFloatingTimerStop", 1.0);
        self:InterruptTimer("Cataclysm");
        self:InterruptTimer("Hemorrhage");
        self:InterruptTimer("ElementiumBolt");
    end,

    OnAssault = function(self)
        self.assaultCount = self.assaultCount + 1;
        if self.assaultCount == 1 then
            -- Apparently the first assault has lower timers.
            self:StartTimer("Impale", 22.5);
            self:StartTimer("ElementiumBolt", 40);
            self:StartTimer("Hemorrhage", 85);
            self:StartTimer("Cataclysm", 115);
      else
            -- Regular timers
            self:StartTimer("Impale", 37.5);
            self:StartTimer("ElementiumBolt");
            self:StartTimer("Hemorrhage");
            self:StartTimer("Cataclysm");
        end
    end,

    OnElementiumBolt = function(self)
        self:AlertIncomingSpell("all/high", 105651);
        if not self:GetEffectInfo("player", 105823) then
            self:StartTimer("ElementiumBlast");
      else
            self:StartTimer("ElementiumBlast", 17); -- More time if Nozdormu is here.
        end
    end,

    OnCataclysm = function(self)
        self:SendViewEvent("OnFloatingTimerStart", 59.99);
    end,

    OnImpale = function(self, guid, stack)
        if self:IsMe(guid) then
            self:AlertThreshold("ImpaleThreshold", stack, self:Localize("Impale"));
        end
    end,

    OnShrapnelTarget = function(self, guid)
        if self:IsMe(guid) then
            local difficulty = self:GetDifficulty();
            if difficulty == 4 or not self:CheckCooldown("ShrapnelProtection", 25) then
                self:AlertSpell("all/critical", 106794, "player");

        elseif difficulty < 4 then
                self:SuperAlert(self:Localize("ShrapnelTitle"), self:Localize("ShrapnelExplain"), 5, false); -- The debuff actually lasts 7 seconds.
            end
        end
        tinsert(self.shrapnel, guid);
    end,

    OnShrapnel = function(self)
        if self:GetSetting("Shrapnel") then
            self:AnnounceUnitListSpell(self.shrapnel, 106794, true, 6);
        end
        wipe(self.shrapnel);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(SECONDARY_ID, true, true); -- ID, volatile, bypass
M:DeclareNPC(PRIMARY_ID, true, true);
M:DeclareNPC(ARM1_ID, true, true);
M:DeclareNPC(ARM2_ID, true, true);
M:DeclareNPC(WING_ID, true, true);
M:DeclareNPC(TENTACLE_ID, true, true);
M:DefineBerserk(900);
M:DeclareZone("Maelstrom", true);
M:DefineZoneBossFilter("DragonSoul", 1); -- 1 boss killed out of 8 (for convenience)
M:DefineFailCooldown(30);
M:DefineClearCooldown(600);

-- Events

M:RegisterCastEvent(107018, "Assault");
M:RegisterCastEvent(105651, "ElementiumBolt");
M:RegisterCastEvent(106523, "Cataclysm");
M:RegisterEffectGainEvent(106794, "ShrapnelTarget");
M:RegisterEffectGainEvent(106400, "Impale");

-- Timers

M:DefineVoidTimer("Cataclysm/106523", "all/normal", 130);
M:DefineVoidTimer("ElementiumBolt/105651", "all/high", 55);
M:DefineVoidTimer("ElementiumBlast/105723", "all/critical", 9);
M:DefineVoidTimer("Hemorrhage/105863", "dps/high", 100);
M:DefineRepeatingVoidTimer("Impale/106400", "shield/high", 35);
M:DefineRepeatingVoidTimer("Terror/106765", "tank/critical", 90);

-- Actions

M:DeclareAction("Assault");
M:AddCallbackToAction("Assault", "OnAssault");

M:DeclareAction("ElementiumBolt");
M:AddTriggerTimerToAction("ElementiumBolt", "ElementiumBolt");
M:AddCallbackToAction("ElementiumBolt", "OnElementiumBolt");

M:DeclareAction("Cataclysm");
M:AddTriggerTimerToAction("Cataclysm", "Cataclysm");
M:AddCallbackToAction("Cataclysm", "OnCataclysm");

M:DeclareAction("Impale");
M:AddTriggerTimerToAction("Impale", "Impale");
M:AddCallbackToAction("Impale", "OnImpale", "targetguid", "stack");

M:DeclareAction("ShrapnelTarget");
M:AddCallbackToAction("ShrapnelTarget", "OnShrapnelTarget", "targetguid");
M:AddExternalActionToAction("ShrapnelTarget", "Shrapnel");

M:DeclareAction("Shrapnel", 8, 0.5);
M:AddCallbackToAction("Shrapnel", "OnShrapnel");