﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local HAGARA_ID = 55689;

local TOMB_ID = 55695;

local FROSTFLAKE_ID = 109325;
local WATERY_ENTRENCHMENT_ID = 110317;

local function frostflakeFunc(self, uid)
    local _, stackCount = self.module:GetEffectInfo(uid, FROSTFLAKE_ID);
    if stackCount then
        self:SetText("x"..stackCount);
        self:SetBarFill(1);
        local inBubble = self.module:GetEffectInfo(uid, WATERY_ENTRENCHMENT_ID);
        if inBubble then
            local a = 0.5 * (1 + cos((GetTime() / 0.1) * 180));
            self:SetBarColor(0.5, a, 1-a);
      else
            self:SetBarColor(1, 0, 0);
        end
  else
        self:SetText("");
        self:SetBarFill(0);
    end
end

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "ShowFrostflake",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Show Frostflake overview",
            ["frFR"] = "Montrer la vue d'ensemble de flocon de givre",
        },
        meta = { heroic = true },
    },
    {
        id = "FrostflakeAlert",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Super alert: Frostflake",
            ["frFR"] = "Super alerte : Flocon de givre",
        },
        explain = {
            ["enUS"] = "Enable this if you need an aggressive alert when Frostflake is cast on you.",
            ["frFR"] = "Cette option permet d'afficher une alerte agressive quand vous subissez Flocon de givre.",
        },
        meta = { heroic = true },
    },
    {
        id = "Frostflake",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = FROSTFLAKE_ID,
                 symbol = 1,
                 heroic = true },
    },
    {
        id = "AssaultAlert",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Super alert: Focused Assault",
            ["frFR"] = "Super alerte : Assaut focalisé",
        },
        explain = {
            ["enUS"] = "Enable this if you need an aggressive alert when Focused Assault starts.",
            ["frFR"] = "Cette option permet d'afficher une alerte agressive quand Assaut focalisé commence.",
        },
    },
    {
        id = "Tomb",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 104451,
                 symbol = 1,
                 multiple = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Special"] = "Special",
        ["FocusedAssault"] = "Focused Assault",
        ["FocusedAssaultTitle"] = "Assault",
        ["FocusedAssaultSubTitle"] = "Taunt / Cooldown",
        ["FrostflakeUI"] = "Frostflake",
        ["FrostflakeTitle"] = "Frostflake",
        ["FrostflakeSubTitle"] = "Go in the bubble!",
    },
    ["frFR"] = {
        ["Special"] = "Spécial",
        ["FocusedAssault"] = "Assaut focalisé",
        ["FocusedAssaultTitle"] = "Assaut",
        ["FocusedAssaultSubTitle"] = "Taunt / Cooldown",
        ["FrostflakeUI"] = "Flocon",
        ["FrostflakeTitle"] = "Flocon",
        ["FrostflakeSubTitle"] = "Allez dans la bulle !",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Hagara", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    tomb = {},

    frostflake = {},

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(70);
        self:SetScoreBenchmarks(400, 600);

        self:RegisterRespawnableAdd(TOMB_ID);
        self:IgnoreAdds();

        self:CheckFrostflakeDisplay();
    end,

    OnBossFound = function(self)
        if self:IsHeroic() then
            self:DefineNearVictory(0.150, 0.90, 120);
        end
    end,

    OnEngaged = function(self)
        self:SetPhase(1);
    end,

    OnSettingUpdate = function(self, id, value)
        if id == "ShowFrostflake" then
            self:CheckFrostflakeDisplay();
        end
    end,

    -- *******
    -- Methods
    -- *******

    CheckFrostflakeDisplay = function(self)
        if self:GetSetting("ShowFrostflake") and (self:IsHeroic() or true) then
            self:StartUnitMonitor(self:Localize("FrostflakeUI"), frostflakeFunc);
      else
            self:StopUnitMonitor();
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnTempest = function(self)
        self:AlertIncomingSpell("all/high", 105256);
        self:OnSpecial();
    end,

    OnTempestFade = function(self)
        self:StartTimer("LightningStorm");
        self:OnSpecialFade();
    end,

    OnWaterShield = function(self)
        self:AlertIncomingSpell("all/high", 105465);
        self:OnSpecial();
    end,

    OnWaterShieldFade = function(self)
        self:StartTimer("Tempest");
        self:OnSpecialFade();
    end,

    OnSpecial = function(self)
        self:SetPhase(2);
        self:InterruptTimer("Assault");
    end,

    OnSpecialFade = function(self)
        self:SetPhase(1);
        self:StartTimer("Assault", 20);
        if not self:IsLFR() then
            self:StartTimer("Tomb");
        end
    end,

    OnTombTarget = function(self, guid)
        tinsert(self.tomb, guid);
    end,

    OnTomb = function(self)
        if self:GetSetting("Tomb") then
            self:AnnounceUnitListSpell(self.tomb, 104451, true, 15);
        end
        wipe(self.tomb);
    end,

    OnFrostflake = function(self, guid)
        local elapsed = GetTime() - (self.frostflake[guid] or 0);
        if elapsed > 10 then
            self.frostflake[guid] = GetTime();
            if self:GetSetting("Frostflake") then
                self:AnnounceUnitSpell(FROSTFLAKE_ID, guid, 1, 8);
            end
            if self:IsMe(guid) then
                if self:GetSetting("FrostflakeAlert") then
                    self:SuperAlert(self:Localize("FrostflakeTitle"), self:Localize("FrostflakeSubTitle"), 5, true);
              else
                    self:AlertSpell("all/high", FROSTFLAKE_ID, guid);
                end
                self:ShowZone("FrostflakeSelf", guid);
          else
                self:ShowZone("FrostflakeOther", guid);
            end
        end
    end,

    OnFrostflakeFade = function(self, guid)
        self:InterruptZone("FrostflakeSelf", guid);
        self:InterruptZone("FrostflakeOther", guid);
    end,

    OnFocusedAssault = function(self)
        if self:GetSetting("AssaultAlert") then
            self:SuperAlert(self:Localize("FocusedAssaultTitle"), self:Localize("FocusedAssaultSubTitle"), 5, false);
      else
            self:AlertCategorized("shield/high", self:Localize("FocusedAssault"));
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(HAGARA_ID, true, false); -- ID, volatile, bypass
M:DefineBerserk(480, 480, 600);

-- Events

M:RegisterEffectGainEvent(104451, "TombTarget");
M:RegisterEffectGainEvent(FROSTFLAKE_ID, "Frostflake");
M:RegisterEffectGainEvent(107851, "Assault");
M:RegisterEffectGainEvent(105256, "Tempest");
M:RegisterEffectGainEvent(105409, "WaterShield");
M:RegisterEffectFadeEvent(FROSTFLAKE_ID, "FrostflakeFade");
M:RegisterEffectFadeEvent(105256, "TempestFade");
M:RegisterEffectFadeEvent(105409, "WaterShieldFade");

-- Timers

M:DefineVoidTimer("Tomb/104451", "all/critical", 28);
M:DefineVoidTimer("Tempest/105256", "all/critical", 62.5);
M:DefineVoidTimer("LightningStorm/105465", "all/critical", 62.5);
M:DefineInitialVoidTimer("Special", "all/critical", 30);
M:DefineInitialRepeatingVoidTimer("Assault/107851", "shield/high", 15.5, 4);

-- Zones

M:DefineAnchoredZone("FrostflakeSelf", "all/high", "red", 6, 8, true);
M:DefineAnchoredZone("FrostflakeOther", "all/high", "yellow", 6, 8, true);

-- Actions

M:DeclareAction("TombTarget");
M:AddCallbackToAction("TombTarget", "OnTombTarget", "targetguid");
M:AddExternalActionToAction("TombTarget", "Tomb");

M:DeclareAction("Tomb", 5, 0.5);
M:AddTriggerTimerToAction("Tomb", "Tomb");
M:AddCallbackToAction("Tomb", "OnTomb");

M:DeclareAction("Assault");
M:AddTriggerTimerToAction("Assault", "Assault");
M:AddCallbackToAction("Assault", "OnFocusedAssault");

M:DeclareAction("Frostflake");
M:AddCallbackToAction("Frostflake", "OnFrostflake", "targetguid");

M:DeclareAction("FrostflakeFade");
M:AddCallbackToAction("FrostflakeFade", "OnFrostflakeFade", "targetguid");

M:DeclareAction("Tempest");
M:AddCallbackToAction("Tempest", "OnTempest");

M:DeclareAction("TempestFade");
M:AddCallbackToAction("TempestFade", "OnTempestFade");

M:DeclareAction("WaterShield");
M:AddCallbackToAction("WaterShield", "OnWaterShield");

M:DeclareAction("WaterShieldFade");
M:AddCallbackToAction("WaterShieldFade", "OnWaterShieldFade");

-- Errors

M:DefineBossAggroError();