﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local MORCHOK_ID = 55265;
local KOHCROM_ID = 57773;

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "CrushArmorThreshold",
        type = "NUMBER",
        min = 1,
        max = 10,
        step = 1,
        default = 3,
        meta = { spell = 103687,
                 threshold = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["Enrage"] = "Enrage",
        ["CrushArmor"] = "You have %d Crush Armor stacks!",
        ["TwinAbility"] = "Twin's ability",
    },
    ["frFR"] = {
        ["Enrage"] = "Enrager",
        ["CrushArmor"] = "Vous avez %d fois Ecraser armure !",
        ["TwinAbility"] = "Sort du jumeau",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Morchok", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    crystal = 1,
    twin = false,

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(65);
        self:SetScoreBenchmarks(270, 420);

        if self:IsHeroic() then
            self:RegisterAdd(KOHCROM_ID);
            self:IgnoreAdds();
        end
    end,

    OnBossFound = function(self, uid)
        self:RegisterHealthThreshold(0.205, self:Localize("Enrage"));
        if self:IsHeroic() then
            self:DefineNearVictory(0.205, 0.90, 120);
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnTwin = function(self)
        self:AlertIncomingSpell("all/high", 109017);

        self.twin = true;

        self:InterruptTimer("Stomp");
        self:InterruptTimer("Crystal");
        self:StartTimer("Stomp", 12);
        self:StartTimer("Crystal", 6);
    end,

    OnCrystal = function(self, guid)
        if self:GetMobID(guid) == MORCHOK_ID then
            self:AlertIncomingSpell("caster/high", 103639);

            self.crystal = self.crystal + 1;
            if self.crystal < 3 then
                self:StartTimer("Crystal");
            end

            if self.twin then
                self:StartTimer("TwinAbility");
            end
        end
    end,

    OnVortex = function(self)
        self:AlertIncomingSpell("all/high", 103821);

        self.crystal = 0;

        self:InterruptTimer("Stomp");
        self:InterruptTimer("TwinAbility");
    end,

    OnVortexEnd = function(self)
        self:StartTimer("Stomp", 19);
        self:StartTimer("Crystal", 26);
        self:StartTimer("Vortex");
    end,

    OnStomp = function(self, guid)
        if self:GetMobID(guid) == MORCHOK_ID then
            self:AlertIncomingSpell("all/high", 103414);

            if self.crystal < 3 then
                self:StartTimer("Stomp");
            end

            if self.twin then
                self:StartTimer("TwinAbility");
            end
        end
    end,

    OnCrushArmor = function(self, guid, stack)
        if self:IsMe(guid) then
            self:AlertThreshold("CrushArmorThreshold", stack, self:Localize("CrushArmor"));
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareNPC(KOHCROM_ID, true, false);
M:DeclareBoss(MORCHOK_ID, false, false); -- ID, volatile, bypass
M:DefineBerserk(nil, 420);
M:DefineMinRange(0);

-- Events

M:RegisterCastEvent(109017, "Twin");
M:RegisterCastEvent(103639, "Crystal");
M:RegisterCastEvent(103821, "Vortex");
M:RegisterCastEvent(103414, "Stomp");
M:RegisterEffectGainEvent(103687, "CrushArmor");
M:RegisterEffectFadeEvent(103851, "VortexEnd");

-- Timers

M:DefineInitialVoidTimer("Vortex/103821", "all/critical", 71, 54);
M:DefineInitialVoidTimer("Crystal/103639", "caster/high", 12.5, 19);
M:DefineInitialVoidTimer("Stomp/103414", "shield/high", 12.5);
M:DefineVoidTimer("TwinAbility", "all/high", 5);

-- Actions

M:DeclareAction("Twin");
M:AddCallbackToAction("Twin", "OnTwin");

M:DeclareAction("Crystal");
M:AddTriggerTimerToAction("Crystal", "Crystal");
M:AddCallbackToAction("Crystal", "OnCrystal", "actorguid");

M:DeclareAction("Vortex", 10);
M:AddTriggerTimerToAction("Vortex", "Vortex");
M:AddCallbackToAction("Vortex", "OnVortex");

M:DeclareAction("Stomp");
M:AddCallbackToAction("Stomp", "OnStomp", "actorguid");

M:DeclareAction("CrushArmor");
M:AddCallbackToAction("CrushArmor", "OnCrushArmor", "targetguid", "stack");

M:DeclareAction("VortexEnd", 10);
M:AddCallbackToAction("VortexEnd", "OnVortexEnd");

-- Errors

M:DefineBossAggroError();
M:DefineAvoidDamageError(103785); -- Evil black ink