﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local Utility = Root:GetOrNewModule("Utility");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local function darknessFunc(self, uid)
    local timeLeft = self.module:GetEffectInfo(uid, 106498) or 0;
    local fill = max(0, 1 - timeLeft / 120);

    self:SetBarColor(1 - fill, fill, 0);
    self:SetBarFill(fill);

    if timeLeft > 600 then
        self:SetText("??'??");
  else
        self:SetText(Utility:FormatCountdown("%M:%S", timeLeft));
    end
    if timeLeft > 0 then
        self:SetTextColor(1, 0.3, 0.3);
  else
        self:SetTextColor(1, 1, 1);
    end
end

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "RepositionButton",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Reposition action button",
            ["frFR"] = "Repositionner le bouton d'action",
        },
        explain = {
            ["enUS"] = [[Force the action button to be displayed and placed on the middle of the screen.

Enable this option if you use an AddOn which interferes with Blizzard interface.

If you use this option, you might have to reload your interface (/reload) to make the button disappear.]],
            ["frFR"] = [[Force le bouton d'action à être affiché et placé au milieu de l'écran.

Activez cette option si vous utilisez un AddOn qui interfère avec l'interface de Blizzard.

En utilisant cette option, vous devrez peut être faire un rechargement d'interface (/reload) pour faire disparaître le bouton.]],
        },
    },
    {
        id = "ShowLoomingDarkness",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Show Looming Darkness overview",
            ["frFR"] = "Montrer la vue d'ensemble de ténèbres menaçantes",
        },
        meta = { heroic = true },
    },
    {
        id = "FadingLight",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 109075,
                 symbol = 1,
                 multiple = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["ButtonTitle"] = "Get ready!",
        ["ButtonSubTitle"] = "Push on the button!",
        ["DarknessUI"] = "Darkness",
    },
    ["frFR"] = {
        ["ButtonTitle"] = "Soyez prêt !",
        ["ButtonSubTitle"] = "Appuyez sur le bouton !",
        ["DarknessUI"] = "Ténèbres",
    },
};

local FADING_LIGHT = GetSpellInfo(109075);

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Ultraxion", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    fadingLight = {},
    lastFadingLight = {},

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(80);
        self:SetScoreBenchmarks(240, 360);

        self:OnSettingUpdate("RepositionButton", self:GetSetting("RepositionButton"));
        self:CheckLoomingDarknessDisplay();
    end,

    OnBossFound = function(self, uid)
        self:SetMainUnit(nil); -- We don't want the boss health bar to be shown; it is over the button in default layout.
        if self:IsHeroic() then
            self:DefineNearVictory(0.150, 0.90, 60);
        end
    end,

    OnEngaged = function(self)
        self:StartTimer("ScanFadingLight");
        self:InterruptTimer("FixButton");
    end,

    OnSettingUpdate = function(self, id, value)
        if id == "RepositionButton" then
            if value then
                self:StartTimer("FixButton");
          else
                self:InterruptTimer("FixButton");
            end
    elseif id == "ShowLoomingDarkness" then
            self:CheckLoomingDarknessDisplay();
        end
    end,

    -- *******
    -- Methods
    -- *******

    PushButtonAlert = function(self, timer, spellId)
        local difficulty = self:GetDifficulty();
        if difficulty == 4 then
            self:AlertSpell("all/critical", spellId, "player");

    elseif difficulty < 4 then
            self:SuperAlert(self:Localize("ButtonTitle"), self:Localize("ButtonSubTitle"), timer, false);
        end
    end,

    AlertMyFadingLight = function(self)
        timeLeft = self:GetEffectInfo("player", FADING_LIGHT);
        if timeLeft then
            self:PushButtonAlert(timeLeft, 109075);
        end
    end,

    ScanFadingLight = function(self)
        self:EnumerateRaid();
        local i, uid, guid, timeLeft;
        for i=1, self:GetEnumerationSize() do
            uid = self:GetEnumerationUID(i);
            guid = UnitGUID(uid);
            timeLeft = self:GetEffectInfo(uid, FADING_LIGHT);
            if timeLeft then
                local elapsed = GetTime() - (self.lastFadingLight[guid] or 0);
                if elapsed > 10 and timeLeft > 1 and timeLeft < 15 then
                    self:OnFadingLightTarget(guid, timeLeft);
                end
            end
        end
        self:StartTimer("ScanFadingLight");
    end,

    FixButton = function(self)
        if not InCombatLockdown() then
            ExtraActionButton1:SetParent(UIParent);
            ExtraActionButton1:SetPoint("CENTER", 0, -150);
            ExtraActionButton1:Show();
        end
    end,

    CheckLoomingDarknessDisplay = function(self)
        if self:GetSetting("ShowLoomingDarkness") and self:IsHeroic() then
            self:StartUnitMonitor(self:Localize("DarknessUI"), darknessFunc);
      else
            self:StopUnitMonitor();
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnHourOfTwilight = function(self)
        local timer = 5;
        if self:IsHeroic() then timer = 3; end
        self:PushButtonAlert(timer, 106371);
    end,

    OnFadingLightTarget = function(self, guid, timeLeft)
        if self:IsMe(guid) then
            local EventList = self:GetWidget("EventList");
            if EventList then
                EventList:StartEvent("FL:"..guid, nil, max(0.2, timeLeft - 4.8), 0, self.AlertMyFadingLight, self);
            end
        end
        tinsert(self.fadingLight, guid);
        self.lastFadingLight[guid] = GetTime();
        self:ScheduleAction("FadingLight");
    end,

    OnFadingLight = function(self)
        if self:GetSetting("FadingLight") then
            self:AnnounceUnitListSpell(self.fadingLight, 109075, true, 10);
        end
        wipe(self.fadingLight);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(55294, true, false); -- ID, volatile, bypass
M:DefineBerserk(360);

-- Events

M:RegisterCastEvent(106371, "HourOfTwilight");

-- Timers

M:DefineTimer("FixButton", "invisible", 0.5, nil, "FixButton", true);
M:DefineTimer("ScanFadingLight", "invisible", 0.5, nil, "ScanFadingLight", true);
M:DefineInitialVoidTimer("GiftOfLife/105896",      "shield/high",  80);
M:DefineInitialVoidTimer("EssenceOfDreams/105900", "shield/high", 155);
M:DefineInitialVoidTimer("SourceOfMagic/105903",   "shield/high", 215);
M:DefineInitialRepeatingVoidTimer("HourOfTwilight/106371", "all/critical", 45);

-- Actions

M:DeclareAction("ScanFadingLight");
M:AddCallbackToAction("ScanFadingLight", "ScanFadingLight");

M:DeclareAction("FixButton");
M:AddCallbackToAction("FixButton", "FixButton");

M:DeclareAction("HourOfTwilight");
M:AddTriggerTimerToAction("HourOfTwilight", "HourOfTwilight");
M:AddCallbackToAction("HourOfTwilight", "OnHourOfTwilight");

M:DeclareAction("FadingLight", 1.5, 0.5);
M:AddCallbackToAction("FadingLight", "OnFadingLight");

-- Errors

M:DefineBossAggroError();