﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local Utility = Root:GetOrNewModule("Utility");
local Sort = Root:GetOrNewModule("Sort");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

local YORSAHJ_ID = 55312;
local BUBBLE_ID = 56231;

local ooze = {
    -- This is for heroic mode.
    -- SpellIDs are the same for normal though.
    [105420] = "purple/green/black/blue",
    [105435] = "green/red/blue/black",
    [105436] = "green/yellow/black/red",
    [105437] = "blue/purple/green/yellow",
    [105439] = "blue/black/purple/yellow",
    [105440] = "purple/red/yellow/black",
};

local focus = {
    -- This is the focus order that works fine for us.
    ["purple/green/black/blue"]  = "black", -- Twin.
    ["green/red/blue/black"]     = "green",
    ["green/yellow/black/red"]   = "green", -- EXPERIMENTAL: yellow
    ["blue/purple/green/yellow"] = "yellow", -- Twin.
    ["blue/black/purple/yellow"] = "yellow",
    ["purple/red/yellow/black"]  = "yellow",
};

local corruptionColor = {
[0]={0,   0, 0},
    {0,   1, 0},
    {1,   1, 0},
    {1, 0.5, 0},
    {1,   0, 0},
    {1,   1, 1},
};

local function corruptionFunc(self, uid)
    local _, count = self.module:GetEffectInfo(uid, 103628);
    timeLeft = self.module:GetEffectInfo(uid, 105171) or 0;
    count = count or 0;

    self:SetBarColor(1, 0, 1);
    self:SetBarFill(min(1, count / (#corruptionColor - 1)));
    if timeLeft > 99 then
        str = ":?? ";
  else
        str = Utility:FormatCountdown(":%S ", timeLeft);
    end
    self:SetText(str.." x"..count);
    self:SetTextColor(unpack(corruptionColor[count]));
end

local POSITION_NEUTRAL    = 0;
local POSITION_MELEE      = 1;
local POSITION_MELEE_BACK = 2;
local POSITION_HEALER     = 3;
local POSITION_TANK       = 4;
local POSITION_USED       = 99;

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "ShowDeepCorruption",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Show Deep Corruption counter",
            ["frFR"] = "Montrer le compteur de corruption profonde",
        },
        explain = {
            ["enUS"] = "Display the stacks of Deep Corruption on players.\n\nThis option is ignored in LFR mode.",
            ["frFR"] = "Affiche les empilements de corruption profonde sur les joueurs.\n\nCette option est ignorée en mode Outil Raids.",
        },
    },
    {
        id = "VoidBoltThreshold",
        type = "NUMBER",
        min = 1,
        max = 10,
        step = 1,
        default = 3,
        meta = { spell = 104849,
                 threshold = true },
    },
    {
        id = "UsePositionTool",
        type = "BOOLEAN",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Use radar assistance during Acid Blood",
            ["frFR"] = "Utiliser l'aide du radar pendant sang acide",
        },
        explain = {
            ["enUS"] = [[This option will assign circular positions to players when Acid Blood is enabled.

This function is sophisticated enough to spread healers and place as much as possible melee players behind the boss. The tank will always be placed south.

In 25 players mode, this feature will create 2 circles, the smallest one allowing up to 6 players in melee range.

This feature is only effective when the whole raid has installed BE3, as it requires network communication.]],
            ["frFR"] = [[Cette option permet d'assigner des placements circulaires aux joueurs quand sang acide est activé.

Cette fonction est suffisamment sophistiquée pour répartir les soigneurs et placer autant que possible les joueurs corps à corps derrière le boss. Le tank sera toujours placé au sud.

En mode 25 joueurs, cette fonction crée 2 cercles, le plus petit permettant jusqu'à 6 joueurs d'être au corps à corps.

Cette fonction n'est efficace que si l'ensemble du raid a installé BE3 car elle nécessite une communication réseau.]],
        },
        meta = { heroic = true },
    },
    {
        id = "UseDefaultFocus",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Use built-in ooze focus order",
            ["frFR"] = "Utiliser l'ordre par défaut des gelées",
        },
        explain = {
            ["enUS"] = "Show tips according to the built-in ooze focus order.\n\nThe built-in focus order that has been chosen in this version is taken from Icy Veins strategy.",
            ["frFR"] = "Affiche des indications selon l'ordre par défaut des gelées.\n\nL'ordre par défaut choisi dans cette version est tiré du guide stratégique de Icy Veins.",
        },
        meta = { heroic = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        ["VoidBolt"] = "You have %d Void Bolt stacks!",
        ["CorruptionUI"] = "Corruption",

        ["Focus"] = "***** FOCUS: %s *****",
        ["purple"] = "Purple",
        ["black"]  = "Black",
        ["blue"]   = "Blue",
        ["red"]    = "Red",
        ["green"]  = "Green",
        ["yellow"] = "Yellow",

        ["purple/green/black/blue"]  = "→ Mana bubble\n→ Limited healing\n→ Regular raid damage\n→ Space the raid",
        ["green/red/blue/black"]     = "→ Mana bubble\n→ Adds spawn once\n→ Moderate raid damage\n→ Stack the raid in the boss",
        ["green/yellow/black/red"]   = "→ Adds spawn twice\n→ BIG raid damage\n→ Stack the raid in the boss",
        ["blue/purple/green/yellow"] = "→ Mana bubble\n→ Limited healing\n→ Regular raid damage\n→ Space the raid",
        ["blue/black/purple/yellow"] = "→ Mana bubble\n→ Adds spawn once\n→ Limited healing\n→ Stack the raid in the boss",
        ["purple/red/yellow/black"]  = "→ Adds spawn once\n→ Limited healing\n→ Moderate raid damage\n→ Stack the raid in the boss",
    },
    ["frFR"] = {
        ["VoidBolt"] = "Vous avez %d fois Eclair de Vide !",
        ["CorruptionUI"] = "Corruption",

        ["Focus"] = "***** FOCUS: %s *****",
        ["purple"] = "Violet",
        ["black"]  = "Noir",
        ["blue"]   = "Bleu",
        ["red"]    = "Rouge",
        ["green"]  = "Vert",
        ["yellow"] = "Jaune",

        ["purple/green/black/blue"]  = "→ Bulle de mana\n→ Limitation des soins\n→ Dégâts de raid réguliers\n→ S'espacer des autres",
        ["green/red/blue/black"]     = "→ Bulle de mana\n→ 1 pop d'adds\n→ Dégâts de raid modérés\n→ Placement dans le boss",
        ["green/yellow/black/red"]   = "→ 2 pops d'adds\n→ GROS dégâts de raid\n→ Placement dans le boss",
        --["green/yellow/black/red"]   = "→ 1 pop d'adds\n→ GROS dégâts de raid\n→ S'espacer des autres [EXPERIMENTAL]", -- EXPERIMENTAL
        ["blue/purple/green/yellow"] = "→ Bulle de mana\n→ Limitation des soins\n→ Dégâts de raid réguliers\n→ S'espacer des autres",
        ["blue/black/purple/yellow"] = "→ Bulle de mana\n→ 1 pop d'adds\n→ Limitation des soins\n→ Placement dans le boss",
        ["purple/red/yellow/black"]  = "→ 1 pop d'adds\n→ Limitation des soins\n→ Dégâts de raid modérés\n→ Placement dans le boss",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Yorsahj", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    _classification = { },

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(70);
        self:SetScoreBenchmarks(270, 420);

        self:RegisterRespawnableAdd(BUBBLE_ID);
        self:IgnoreAdds();

        self:CheckDeepCorruptionDisplay();
        self:StartTimer("FixCartography", 3);
    end,

    OnEngaged = function(self)
        self:StartTimer("SendClassification", 12.5);
    end,

    OnBossFound = function(self)
        if self:IsHeroic() then
            self:DefineNearVictory(0.150, 0.90, 120);
        end
    end,

    OnBossCast = function(self, unit, spellId, spellName)
        if ooze[spellId] then
            self:OnOoze(ooze[spellId]);
        end
    end,

    OnSettingUpdate = function(self, id, value)
        if id == "ShowDeepCorruption" then
            self:CheckDeepCorruptionDisplay();
        end
    end,

    OnNetCommand = function(self, command, sender, ...)
        if command == "CUSTOM" then
            local category = ...
            if category == "yc" then -- "yorsahj circle"
                local _, points, index, x, y = ...
                points = tonumber(points);
                index = tonumber(index);
                x = tonumber(x);
                y = tonumber(y);
                if points and index and x and y then
                    local enabled, decider = self:GetSetting("UsePositionTool");
                    if enabled and decider == sender then
                        x = x / 1000;
                        y = y / 1000;
                        self:Debug("Received circle from "..sender);

                        local c1, c2 = self:GetCircleSizes(points);
                        self:MakeCircle(c1, 1, index, x, y);
                        self:MakeCircle(c2, c1 + 1, index, x, y);
                    end
                end

        elseif category == "yp" then -- "yorsahj profile"
                local _, classification = ...
                classification = tonumber(classification);
                if classification then
                    self._classification[sender] = classification;
                    self:Debug(sender.." classification: "..classification);
                end
            end
        end
    end,

    -- *******
    -- Methods
    -- *******

    FixCartography = function(self)
        SetMapToCurrentZone();
    end,

    GetClassification = function(self)
        local range = self:GetRange();
        if range == "HEALER" then
            return POSITION_HEALER;

    elseif range == "MELEE" then
            if self:HasBossAggro("player") then
                return POSITION_TANK;
          else
                local _, class = UnitClass("player");
                if class == "DRUID" or class == "ROGUE" then
                    return POSITION_MELEE_BACK;
              else
                    return POSITION_MELEE;
                end
            end
      else
            return POSITION_NEUTRAL;
        end
    end,

    CheckDeepCorruptionDisplay = function(self)
        if self:GetSetting("ShowDeepCorruption") and not self:IsLFR() then
            self:StartUnitMonitor(self:Localize("CorruptionUI"), corruptionFunc);
      else
            self:StopUnitMonitor();
        end
    end,

    SendCircle = function(self, x, y)
        if (not x) or (not y) then
            x, y = GetPlayerMapPosition("player");
        end
        x = floor(x * 1000 + 0.5);
        y = floor(y * 1000 + 0.5);
        self:EnumerateRaid();

        local count = self:GetEnumerationSize();
        local healers = 0;
        local p = { };
        local i, uid, name, classification, position;

        for i=1, count do
            uid = self:GetEnumerationUID(i);
            name = UnitName(uid);
            classification = self._classification[name] or POSITION_NEUTRAL;
            tinsert(p, {name = name, v = classification});
            if classification == POSITION_HEALER then
                healers = healers + 1;
            end
        end

        local t = self:FillCirclePositions(count, healers);
        Sort:NumericField(t, "v", true);
        Sort:NumericField(p, "v", true);
        for i=1, count do
            name = p[i].name;
            position = t[i].i;
            self:SendPacketToPlayer(name, "yc", count, position, x, y);
            self:Debug("Assigning "..name.." to position "..position);
        end
    end,

    SendClassification = function(self)
        self:SendPacket("yp", self:GetClassification());
    end,

    GetCircleSizes = function(self, points)
        if points >= 13 then
            local melee = min(10, floor((points - 8) / 3 + 0.5));
            return melee, points - melee;
      else
            return points, 0;
        end
    end,

    MakeCircle = function(self, points, startIndex, index, mx, my)
        local Radar = self:GetWidget("Radar");
        if Radar and (points or 0) >= 2 then
            if (not mx) or (not my) then
                mx, my = GetPlayerMapPosition("player");
            end
            -- Must have 4 yds between players, so we show 4.5 on the radar
            -- and use 5.5 when computing the radius of the smallest circle.
            local circle_radius = 5.5 / (2 - 2 * cos(360 / points)) ^ 0.5;
            local i, a, x, y, radius, color;
            for i=1, points do
                a = -90 - 360 * (i-1) / points;
                x, y = Radar:GetCoordinates(mx, my);
                if x and y then
                    radius, color = 4.5, "red";
                    if index == startIndex + i - 1 then radius, color = 1.0, "green"; end
                    Radar:AddDirectZone(tostring(i), color, radius, x + cos(a) * circle_radius, y + sin(a) * circle_radius, 80);
                end
            end
            Radar:SetVisibility(30);
            self:Debug("Radius: "..circle_radius);
        end
    end,

    FillCirclePositions = function(self, count, healers)
        local t = { };
        if count > 0 then
            local c1, c2 = self:GetCircleSizes(count);
            local c = c1 + c2;
            local i, a, r;
            for i=1, c do
                tinsert(t, {i = i, v = POSITION_NEUTRAL});
            end
            t[1].v = POSITION_TANK;
            for i=2, c1 do
                a = -90 - 360 * (i-1) / c1;
                if a > -355 and a < -185 then
                    t[i].v = POSITION_MELEE_BACK;
              else
                    t[i].v = POSITION_MELEE;
                end
            end
            if healers > 0 then
                if c2 > 0 then
                    r = max(1, floor(c2 / healers + 0.5));
                    for i=c1+1, c do
                        if math.fmod(i-2, r) == 0 then
                            t[i].v = POSITION_HEALER;
                        end
                    end
              else
                    r = max(1, floor(c1 / healers + 0.5));
                    for i=2, c1 do
                        if math.fmod(i-2, r) == 0 then
                            t[i].v = POSITION_HEALER;
                        end
                    end
                end
            end
        end
        return t;
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnOoze = function(self, combination)
        self:AlertIncomingSpell("all/high", 105420);
        self:TriggerTimer("Ooze");
        if self:IsHeroic() then
            local kill = focus[combination];
            if self:GetSetting("UseDefaultFocus") then
                local summary = self:Localize(combination);
                local color = self:FormatLoc("Focus", self:Localize(kill));
                if color and summary then
                    local lines = {color, strsplit("\n", summary)};
                    local i;
                    for i=1, #lines do
                        self:RaidMessage(lines[i]);
                    end
                    self:Announce(color, true);
                end
            end
            if combination:find("green") and kill ~= "green" then
                self:OnAcidBloodSoon();
            end
            self:StartTimer("Ooze", 75);
            self:StartTimer("SendClassification", 65);
      else
            self:StartTimer("Ooze");
            self:StartTimer("SendClassification");
        end
    end,

    OnVoidBolt = function(self, guid, stack)
        if self:IsMe(guid) then
            self:AlertThreshold("VoidBoltThreshold", stack, self:Localize("VoidBolt"));
        end
    end,

    OnAcidBloodSoon = function(self)
        local enabled, decider = self:GetSetting("UsePositionTool");
        if enabled then
            if self:IsMe(decider) then
                self:Debug("Sending circle...");
                self:SendCircle(0.473, 0.489);
            end
        end
    end,

    OnAcidBlood = function(self)
        if not self:IsLFR() then
            self:ChangeMinRange(4);
        end
    end,

    OnAcidBloodEnd = function(self)
        if not self:IsLFR() then
            self:ChangeMinRange(0);
        end
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(YORSAHJ_ID, false, false); -- ID, volatile, bypass
M:DefineBerserk(600);
M:DefineMinRange(0);

-- Events

M:RegisterEffectGainEvent(104849, "VoidBolt");
M:RegisterEffectGainEvent(104898, "AcidBlood");
M:RegisterEffectFadeEvent(104898, "AcidBloodEnd");

-- Timers

M:DefineTimer("SendClassification", "invisible", 80, nil, "SendClassification", true);
M:DefineRepeatingTimer("FixCartography", "invisible", 3, nil, "FixCartography", true);
M:DefineInitialVoidTimer("Ooze/105420", "all/critical", 90, 22.5);

-- Actions

M:DeclareAction("VoidBolt");
M:AddCallbackToAction("VoidBolt", "OnVoidBolt", "targetguid", "stack");

M:DeclareAction("AcidBlood");
M:AddCallbackToAction("AcidBlood", "OnAcidBlood");

M:DeclareAction("AcidBloodEnd");
M:AddCallbackToAction("AcidBloodEnd", "OnAcidBloodEnd");

M:DeclareAction("SendClassification");
M:AddCallbackToAction("SendClassification", "SendClassification");

M:DeclareAction("FixCartography");
M:AddCallbackToAction("FixCartography", "FixCartography");

-- Errors

M:DefineBossAggroError();