﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "Shadow",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 103434,
                 symbol = 1,
                 multiple = true },
    },
    version = 1,
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:NewCataclysmRaid("Zonozz", mySettings);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    shadow = {},

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:SetImportance(70);
        self:SetScoreBenchmarks(240, 360);
    end,

    OnBossFound = function(self)
        if self:IsHeroic() then
            self:DefineNearVictory(0.150, 0.90, 120);
        end
    end,

    OnEngaged = function(self)
        if self:IsHeroic() then
            self:ChangeMinRange(10);
        end
    end,

    OnBossCast = function(self, unit, spellId, spellName)
        if spellId == 103571 then
            self:OnVoid();
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    OnVoid = function(self)
        self:AlertIncomingSpell("all/high", 103571);
        self:TriggerTimer("Void");
    end,

    OnShadowTarget = function(self, guid)
        if self:IsMe(guid) then
            self:AlertSpell("all/high", 103434, guid);
        end
        tinsert(self.shadow, guid);
    end,

    OnShadow = function(self)
        if self:GetSetting("Shadow") then
            self:AnnounceUnitListSpell(self.shadow, 103434, true, 8);
        end
        wipe(self.shadow);
    end,

    OnBlackBlood = function(self)
        self:AlertIncomingSpell("all/high", 104378);
        self:InterruptTimer("PsychicDrain");
    end,

    OnBlackBloodEnd = function(self)
        self:StartTimer("PsychicDrain", 6);
        self:StartTimer("Shadow", 6);
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(55308, false, false); -- ID, volatile, bypass
M:DefineBerserk(360, nil, 600);

-- Events

M:RegisterCastEvent(104322, "PsychicDrain");
M:RegisterEffectGainEvent(103434, "ShadowTarget");
M:RegisterEffectGainEvent(104378, "BlackBlood");
M:RegisterEffectFadeEvent(104378, "BlackBloodEnd");

-- Timers

M:DefineVoidTimer("BlackBlood/104378", "all/high", 30);
M:DefineInitialRepeatingVoidTimer("Void/104002", "all/high", 90, 6);
M:DefineInitialRepeatingVoidTimer("PsychicDrain/104322", "tank/high", 20);
M:DefineInitialRepeatingVoidTimer("Shadow/103434", "all/high", 25);

-- Actions

M:DeclareAction("PsychicDrain");
M:AddTriggerTimerToAction("PsychicDrain", "PsychicDrain");

M:DeclareAction("ShadowTarget");
M:AddCallbackToAction("ShadowTarget", "OnShadowTarget", "targetguid");
M:AddExternalActionToAction("ShadowTarget", "Shadow");

M:DeclareAction("Shadow", 5, 0.5);
M:AddTriggerTimerToAction("Shadow", "Shadow");
M:AddCallbackToAction("Shadow", "OnShadow");

M:DeclareAction("BlackBlood", 5);
M:AddStartTimerToAction("BlackBlood", "BlackBlood");
M:AddCallbackToAction("BlackBlood", "OnBlackBlood");

M:DeclareAction("BlackBloodEnd", 5);
M:AddTriggerTimerToAction("BlackBloodEnd", "BlackBlood");
M:AddCallbackToAction("BlackBloodEnd", "OnBlackBloodEnd");

-- Errors

M:DefineBossAggroError();