local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    _berserkTimer = 0, -- The defined berserk timer
    _berserkTimerHeroic = false, -- Same for heroic
    _berserkTimerLFR = false, -- Same for LFR
    berserkTimer = 0, -- The current berserk timer
    berserkStart = 0,

    -- Methods
    -- *******

    -- ============================================================
    -- Define the berserk timer of the fight.
    -- An alternative time amount can be specified for heroic mode.
    -- ============================================================
    DefineBerserk = function(self, timer, timerHeroic, timerLFR)
        self._berserkTimer = timer or 0;
        self._berserkTimerHeroic = timerHeroic or false;
        self._berserkTimerLFR = timerLFR or false;
    end,

    -- ===============================================
    -- Check if there is a berserk timer in the fight.
    -- ===============================================
    HasBerserk = function(self)
        return (self.berserkTimer > 0);
    end,

    -- ======================
    -- Set the berserk timer.
    -- ======================
    SetBerserk = function(self, timer)
        self.berserkTimer = timer;
        self.berserkStart = 0;
        self:SendViewEvent("OnBerserkSet", timer);
    end,

    -- ============================================
    -- Check for a berserk timer.
    -- Set it if it exists and hasn't been set yet.
    -- ============================================
    CheckBerserk = function(self)
        local amount = self._berserkTimer;
        if self:IsLFR() then
            amount = self._berserkTimerLFR or amount;
    elseif self:IsHeroic() then
            amount = self._berserkTimerHeroic or amount;
        end
        if amount > 0 then
            self:SetBerserk(amount);
        end
    end,

    -- ============================================
    -- Start the berserk timer once previously set.
    -- ============================================
    BeginBerserk = function(self)
        if self:HasBerserk() then
            self.berserkStart = GetTime();
            self:SendViewEvent("OnTimerStart");
            return true;
        end
        return false;
    end,

    -- ========================================
    -- Get time amount left before the berserk.
    -- ========================================
    GetBerserkTimeLeft = function(self)
        if self:HasBerserk() then
            if self.berserkStart > 0 then
                return max(0, self.berserkTimer + self.berserkStart - GetTime());
          else
                return self.berserkTimer;
            end
      else
            return math.huge;
        end
    end,

    -- ==========================
    -- Suspend the berserk timer.
    -- ==========================
    SuspendBerserk = function(self)
        if self:HasBerserk() then
            self.berserkTimer = self:GetBerserkTimeLeft();
            self.berserkStart = 0;
            self:SendViewEvent("OnTimerSuspend");
        end
    end,

    -- =========================
    -- Resume the berserk timer.
    -- =========================
    ResumeBerserk = function(self)
        if self:HasBerserk() then
            self.berserkStart = GetTime();
            self:SendViewEvent("OnTimerResume", self:GetBerserkTimeLeft());
        end
    end, 
} );