local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

-- Category string is a special string used by several components of the mod to filter stuff according to role and difficulty setting. It is defined as follow:
-- category = "role/importance"
--
-- role is a lowercase string between {dps_melee, dps_ranged, heal, tank, melee, caster, dps, shield, offensive, all}.
-- importance is a lowercase string between {low, normal, high, critical}.
--
-- Category string can also take 2 extra special values (disregarding the previous syntax of "role/importance").
-- * "ooc" which stands for "out of combat" and should be used for elements that are related to out of combat stuff.
-- * "invisible" for elements that shouldn't be visible in the GUI.
--
-- Each importance has a value tied to it:
-- ***************************************
-- Invisible: -oo
-- Low: 0
-- Normal: 1
-- High: 2
-- Critical: 3
-- OutOfCombat: +oo
--
-- If role of the player doesn't match the role specified in the category string, the value will be reduced by 2.

local values = {
   ["invisible"] = -math.huge,
         ["low"] = 0,
      ["normal"] = 1,
        ["high"] = 2,
    ["critical"] = 3,
         ["ooc"] = math.huge,
};

local difficultyValue = {
    [1] =  2, -- Very easy
    [2] =  1, --
    [3] =  0, -- Normal
    [4] = -1, --
    [5] = -2, -- Hard
};

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    -- Methods
    -- *******

    GetImportanceValue = function(self, importance)
        if importance then
            return values[importance] or 0;
      else
            return 0;
        end
    end,

    GetRoleModifier = function(self, role)
        if role then
            return self:IfRole(role:upper(), 0, -2);
      else
            return 0;
        end
    end,

    GetDifficultyValue = function(self)
        local difficulty = self:GetDifficulty();
        return difficultyValue[difficulty] or 0;
    end,

    ParseCategory = function(self, category)
        local role, importance = strsplit("/", category, 2);
        if (not role) or role == "ooc" or role == "invisible" then
            return nil, role;
      else
            return role, importance;
        end
    end,

    GetCategoryValue = function(self, category)
        local role, importance = self:ParseCategory(category);
        local impValue = self:GetImportanceValue(importance);
        local roleMod = self:GetRoleModifier(role);
        local total = impValue + roleMod;

        return total, impValue, roleMod;
    end,

    GetFilterValue = function(self, category)
        local importanceValue = self:GetCategoryValue(category);
        local difficultyValue = self:GetDifficultyValue();
        local filterValue = importanceValue + difficultyValue;
        if category == "ooc" or category == "invisible" then
            importanceValue = 0;
        end
        return filterValue, importanceValue, difficultyValue;
    end,
} );