local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local System = Root:GetOrNewModule("System");
local View = System:GetOrNewModule("View");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    widgets = { },

    -- Methods
    -- *******

    -- ========================================
    -- Get a widget by name for direct control.
    -- ========================================
    GetWidget = function(self, name)
        return self.widgets[name];
    end,

    -- ================================
    -- Add a widget for direct control.
    -- ================================
    AddWidget = function(self, name, widget)
        self.widgets[name] = widget;
    end,

    -- ===============================================================================
    -- Method to call to get an access to the GUI elements once the module is enabled.
    -- ===============================================================================
    ConnectGUI = function(self)
        View:Enable();
        self:AddWidget("EventList", View:GetEventList());
        self:AddWidget("EnemyList", View:GetEnemyList());
        self:AddWidget("UnitList", View:GetUnitList());
        self:AddWidget("BossBar", View:GetBossBar());
        self:AddWidget("Radar", View:GetRadar());
    end,

    -- ==============================================================================
    -- Method to call to suspend elements once module is finished (but not disabled).
    -- ==============================================================================
    FinalizeGUI = function(self)
        self:SendViewEvent("OnTimerSuspend");
        self:StopMirrorValue();

        local EventList = self:GetWidget("EventList");
        if EventList then
            EventList:Clear();
        end

        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList then
            EnemyList:Clear();
        end
    end,

    -- ==================================================================================
    -- Method to call to dispose of elements of the GUI properly once module is disabled.
    -- ==================================================================================
    DisconnectGUI = function(self)
        View:Disable();
    end,

    -- ===========================================================================
    -- Fire an event on the view. Events are used to handle simple GUI operations.
    -- ===========================================================================
    SendViewEvent = function(self, event, ...)
        if View[event] then
            View[event](View, ...);
        end
    end,

    -- ==================================================
    -- Close the settings frame, once combat has started.
    -- ==================================================
    CloseOptions = function(self)
        View:CloseOptions();
    end,

    -- ========================================================
    -- Asks the GUI to show an emphasized bar.
    -- This bar calls the specified method to update the value.
    -- ========================================================
    StartMirrorValue = function(self, label, method, methodAlt)
        self:SendViewEvent("OnMirrorValueStart", label, self, method, methodAlt);
    end,

    -- =======================================
    -- Stop the display of the emphasized bar.
    -- =======================================
    StopMirrorValue = function(self)
        self:SendViewEvent("OnMirrorValueStop");
    end,

    -- ===============================
    -- Monitor units in the unit list.
    -- ===============================
    StartUnitMonitor = function(self, title, updateFunc, playerList)
        local UnitList = self:GetWidget("UnitList");
        if UnitList then
            UnitList:SetTitle(title);
            UnitList:SetPlayers(playerList or "ALL");
            UnitList:SetUpdateFunc(updateFunc);
        end
    end,

    -- ===================
    -- Hide the unit list.
    -- ===================
    StopUnitMonitor = function(self)
        local UnitList = self:GetWidget("UnitList");
        if UnitList then
            UnitList:Clear();
        end
    end,

    -- ===================================
    -- Set the main unit of the encounter.
    -- ===================================
    SetMainUnit = function(self, id)
        local EnemyList = self:GetWidget("EnemyList");
        if EnemyList and self:GetSetting("EnableBossBar") then
            EnemyList:SetMainUnit(id);
        end
    end,
} );