local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");
local L = Root:GetOrNewModule("Localization");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    _localization = { }, -- Will be populated externally.

    -- Methods
    -- *******

    -- =======================================================================================
    -- Localize a string. This method looks into the private localization table of the module.
    -- If the localization is not found, then the global localization table is used, unless
    -- noGlobal option is set, in which case the return will be nil.
    -- =======================================================================================
    Localize = function(self, key, noGlobal)
        local localeTable = self._localization;
        local myLocale = localeTable[L:GetLocale()];

        if myLocale and myLocale[key] then
            return myLocale[key];
      else
            local defaultLocale = localeTable[L:GetDefaultLocale()];
            if defaultLocale and defaultLocale[key] then
                return defaultLocale[key];
          else
                if not noGlobal then
                    return L:Get(key);
              else
                    return nil;
                end
            end
        end
    end,

    -- ===================================
    -- Localize + format at the same time.
    -- ===================================
    FormatLoc = function(self, key, ...)
        return string.format(self:Localize(key), ...);
    end,

    -- ==========================================================================
    -- Check if there is a "title" field in the localization table of the module.
    -- ==========================================================================
    CheckTitle = function(self)
        local title = self:Localize("Title", true);
        if title then
            self:SetTitle(title);
        end
    end,
} );