local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local Unit = Root:GetOrNewModule("Unit");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    searches = { },

    -- Methods
    -- *******

    -- =======================================================================
    -- Search for an unit with the given mobID and call the specified callback
    -- once it is found. The callback must be a module's method.
    -- The UID of the unit when it is found will be passed to the callback.
    -- Multiple searches may be combined.
    -- =======================================================================
    SearchUnit = function(self, mobID, callback)
        if mobID and callback then
            tinsert(self.searches, {
                id = mobID,
                callback = callback,
            } );
        end
    end,

    -- ================================================
    -- The update function for the search unit service.
    -- Should be called in OnUpdate handler.
    -- ================================================
    CheckSearches = function(self)
        local i, search;
        for i=#self.searches, 1, -1 do
            search = self.searches[i];
            local _, uid = Unit:GetGUIDFromMobID(search.id);
            if uid then
                tremove(self.searches, i);
                search.callback(self, uid);
            end
        end
    end,
} );