local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local System = Root:GetOrNewModule("System");
local Settings = System:GetOrNewModule("Settings");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    _settings = { }, -- Will be populated externally.

    -- Methods
    -- *******

    -- =======================================================
    -- Inject the module's settings into the settings manager.
    -- =======================================================
    DeclareMySettings = function(self)
        Settings:AccessModuleSettings(self._settings, self:GetName());
    end,

    -- =======================================================
    -- Remove the module's settings from the settings manager.
    -- =======================================================
    ClearMySettings = function(self)
        Settings:ClearModuleSettings();
    end,

    -- ===============================================
    -- Get the value of one of the settings parameter.
    -- Return nil if the setting doesn't exist.
    -- This can also read global settings.
    -- ===============================================
    GetSetting = function(self, id)
        return Settings:Read(id);
    end,

    -- ==========================================================================
    -- Determinate if the local player has enabled the filter for the given role.
    -- Possible explicit roles: DPS_MELEE, DPS_RANGED, HEAL, TANK.
    -- Possible implicit roles: MELEE, CASTER, DPS, SHIELD, OFFENSIVE.
    -- ==========================================================================
    HasRole = function(self, role)
        return Settings:HasRole(role);
    end,

    -- ===============================================================
    -- Return ifTrue value if the local player has the specified role.
    -- Return ifFalse if not.
    -- See above for the role list.
    -- ===============================================================
    IfRole = function(self, role, ifTrue, ifFalse)
        if self:HasRole(role) then
            return ifTrue;
      else
            return ifFalse;
        end
    end,

    -- ==========================================================
    -- Return MELEE, RANGED or HEALER according to the expected
    -- position of the player (based on his current talent spec).
    -- ==========================================================
    GetRange = function(self)
        return Settings:GetRange();
    end,

    -- ============================================
    -- Get the difficulty index chosen by the user.
    -- ============================================
    GetDifficulty = function(self)
        return Settings:GetDifficulty();
    end,

    -- ================================
    -- Override the difficulty setting.
    -- ================================
    SetDifficulty = function(self, difficulty)
        Settings:SetDifficulty(difficulty);
    end,

    -- ===========================================
    -- Check if the player has a decent interrupt.
    -- ===========================================
    HasInterrupt = function(self)
        return Settings:HasInterrupt();
    end,
} );