local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    timeoutMax = 60,
    timeout = 0,

    -- Methods
    -- *******

    -- ==========================
    -- Zero out the timeout timer
    -- ==========================
    ZeroTimeout = function(self)
        self.timeout = 0;
    end,

    -- =========================
    -- Set a new timeout maximum
    -- =========================
    SetTimeout = function(self, timer)
        self.timeoutMax = timer;
        self.timeout = 0;
    end,

    -- ===================================================
    -- Set a default timeout, according to the environment
    -- ===================================================
    SetDefaultTimeout = function(self)
        if self:GetEnvironment() == "RAID" then
            self.timeoutMax = 60;
      else
            self.timeoutMax = 30;
        end
        self.timeout = 0;
    end,

    -- =========================
    -- Update the timeout timer.
    -- =========================
    CheckTimeout = function(self, elapsed)
        self.timeout = self.timeout + elapsed;
        if self.timeout >= self.timeoutMax and not self:IsTesting() then
            local state = self:GetState();
            if state == "ENGAGED" then
                self:OnFailed(true);

        elseif state == "STANDBY" then
                self:OnFailed(false);
          else
                -- Timeout has no effect in other states.
            end
        end
    end,
} );