local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local Utility = Root:GetOrNewModule("Utility");

local argTable = { };

Module:RegisterLib( {
    -- Properties [those starting with _ won't be reset between 2 sessions]
    -- **********

    yells = { },
    spellYells = { },

    -- Methods
    -- *******

    -- ===================================================
    -- Add a yell from anyone that will trigger a callback
    -- ===================================================
    RegisterAnyYell = function(self, matches, callback, ...)
        self:RegisterYell("ANY", matches, callback, ...);
    end,

    -- =====================================================
    -- Add a yell from the boss that will trigger a callback
    -- =====================================================
    RegisterBossYell = function(self, matches, callback, ...)
        self:RegisterYell(self:GetBossName(), matches, callback, ...);
    end,

    -- =======================================
    -- Add a yell that will trigger a callback
    -- =======================================
    RegisterYell = function(self, npcName, matches, callback, ...)
        tinsert(self.yells, {
            npc = npcName,
            matches = matches,
            callback = callback,
            args = {...},
        } );
    end,

    -- =============================================
    -- Add a spell yell that will trigger a callback
    -- =============================================
    RegisterSpellYell = function(self, spell, callback, ...)
        self.spellYells[spell] = {
            callback = callback,
            args = {...},
        };
    end,

    -- ===========================================================
    -- Parse a yell and fire associated callback if it's matching.
    -- ===========================================================
    CheckYell = function(self, message, source, target)
        local spellId = Utility:ExtractSpellID(message);
        if spellId then
            local info = self.spellYells[spellId];
            if info then
                if type(info.callback) == "function" then
                    info.callback(unpack(info.args));
              else
                    self:ScheduleAction(info.callback);
                end
            end
        end

        local i, j, yell, numMatches, try;
        for i=1, #self.yells do
            yell = self.yells[i];
            if yell.npc == "ANY" or yell.npc == source then
                numMatches = select('#', strsplit("|", yell.matches));
                for j=1, numMatches do
                    try = select(j, strsplit("|", yell.matches));
                    if string.find(message, try) then
                        if type(yell.callback) == "function" then
                            wipe(argTable);
                            for j=1, #yell.args do
                                tinsert(argTable, yell.args[j]);
                            end
                            if target then
                                tinsert(argTable, target);
                            end
                            tinsert(argTable, message);
                            yell.callback(unpack(argTable));
                      else
                            self:ScheduleAction(yell.callback);
                        end
                        return;
                    end
                end
            end
        end
    end,
} );