﻿local Root = BossEncounter3;
local Module = Root:GetOrNewModule("Module");

local System = Root:GetOrNewModule("System");
local Distance = System:GetOrNewModule("Distance");

-- --------------------------------------------------------------------
-- **                              Locals                            **
-- --------------------------------------------------------------------

-- --------------------------------------------------------------------
-- **                             Settings                           **
-- --------------------------------------------------------------------

local mySettings = {
    {
        id = "warnPlague",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        meta = { spell = 70337,
                 symbol = 7 },
    },

    {
        id = "warnValkyr",
        type = "BOOLEAN",
        action = "SYMBOL",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Warn Val'kyr targets",
            ["frFR"] = "Annoncer les cibles des Val'kyrs",
        },
        explain = {
            ["enUS"] = "List all people caught by the Val'kyrs and put a symbol on them.",
            ["frFR"] = "Liste les personnes attrapées par les Val'kyrs et affiche des symboles sur elles.",
        },
    },

    {
        id = "warnHarvestSoul",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        meta = { spell = 68980,
                 symbol = 7 },
    },

    {
        id = "warnEnrage",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Warn Enrage",
            ["frFR"] = "Annoncer l'enrager",
        },
        explain = {
            ["enUS"] = "Display a raid alert whenever a Shambling Horror enrages.",
            ["frFR"] = "Affiche une alerte de raid chaque fois qu'une horreur titubante devient enragée.",
        },
    },

    {
        id = "defileSymbol",
        type = "BOOLEAN",
        action = "RAID_WARNING",
        default = false,
        exclusive = true,
        label = {
            ["enUS"] = "Symbol: Defile",
            ["frFR"] = "Symbole : profanation",
        },
        explain = {
            ["enUS"] = "Put a symbol on the target of Defile.",
            ["frFR"] = "Place un symbole sur la cible de profanation.",
        },
    },

    version = 1,
};

-- --------------------------------------------------------------------
-- **                           Localization                         **
-- --------------------------------------------------------------------

local myLocalization = {
    ["enUS"] = {
        -- 1. Yells
        ["IntroTrigger"] = "vaunted justice has finally arrived",

        -- 2. Events
        ["FinalPhase"] = "Final Phase",
        ["Transition"] = "Transition %d",
        ["Quake"] = "Quake",
        ["RagingSpirit"] = "Raging Spirit",
        ["Plague"] = "Necrotic Plague",
        ["Infest"] = "Infest",
        ["Horror"] = "Horror",
        ["Valkyr"] = "Val'kyr",
        ["VileSpirits"] = "Vile Spirits enabled",
        ["HarvestSoul"] = "Harvest Soul",
        ["SoulReaper"] = "Soul Reaper",
        ["Defile"] = "Defile",

        -- 3. Misc
        ["Start"] = "Start of combat",
        ["RemorselessWinterAlert"] = "Remorseless Winter incoming!",
        ["QuakeAlert"] = "Quake incoming!",
        ["PlagueAlert"] = "Necrotic Plague: %s",
        ["PlagueSelf"] = "You have the Necrotic Plague!",
        ["EnrageAlert"] = "%s is enraged!",
        ["HarvestSoulAlert"] = "Harvest Soul: %s",
        ["SoulReaperWarning"] = "Soul Reaper incoming!",
        ["DefileTickSelf"] = "Get out of the area!",
        ["DefileSelf"] = "Defile on you!",
        ["DefileSay"] = "Defile here!",
        ["DefileNear"] = "Defile nearby!",
        ["ValkyrAlert"] = "Val'kyr targets:",
    },
    ["frFR"] = {
        -- 1. Yells
        ["IntroTrigger"] = "arrive la fameuse",

        -- 2. Events
        ["FinalPhase"] = "Phase finale",
        ["Transition"] = "Transition %d",
        ["Quake"] = "Secousse",
        ["RagingSpirit"] = "Esprit déchaîné",
        ["Plague"] = "Peste nécrotique",
        ["Infest"] = "Infester",
        ["Horror"] = "Horreur",
        ["Valkyr"] = "Val'kyr",
        ["VileSpirits"] = "Esprits vils actifs",
        ["HarvestSoul"] = "Moisson d'âmes",
        ["SoulReaper"] = "Faucheur",
        ["Defile"] = "Profanation",

        -- 3. Misc
        ["Start"] = "Début du combat",
        ["RemorselessWinterAlert"] = "Hiver impitoyable imminent !",
        ["QuakeAlert"] = "Secousse imminente !",
        ["PlagueAlert"] = "Peste nécrotique : %s",
        ["PlagueSelf"] = "Vous avez la peste nécrotique !",
        ["EnrageAlert"] = "%s enrage !",
        ["HarvestSoulAlert"] = "Moisson d'âmes : %s",
        ["SoulReaperWarning"] = "Faucheur imminent !",
        ["DefileTickSelf"] = "Sortez de la zone !",
        ["DefileSelf"] = "Profanation sur vous !",
        ["DefileSay"] = "Profanation ici !",
        ["DefileNear"] = "Profanation à proximité !",
        ["ValkyrAlert"] = "Cibles des Val'kyrs :",
    },
};

-- --------------------------------------------------------------------
-- **                            Definition                          **
-- --------------------------------------------------------------------

local M = Module:New("Arthas", "LK", "RAID", mySettings, myLocalization);

Module:InstallExtra(M, {
    -- **********
    -- Properties
    -- **********

    -- ***************
    -- Custom handlers
    -- ***************

    OnEnable = function(self)
        self:RegisterAnyYell(self:Localize("IntroTrigger"), self.OnIntro, self);
        self:SetImportance(90);
    end,

    OnBossFound = function(self, uid)
        self:SetEndSequence("FINAL");
        self:SetScoreBenchmarks(720, 1200);
        self:DefineNearVictory(0.08, nil, nil); -- Near victory when the boss gets below 8%.

        self:RegisterHealthThreshold(0.70, self:FormatLoc("Transition", 1));
        self:RegisterHealthThreshold(0.40, self:FormatLoc("Transition", 2));
        self:RegisterHealthThreshold(0.10, self:FormatLoc("Transition", 3));
    end,

    OnEngaged = function(self)
        self:OnPhaseOne();
    end,

    -- *******
    -- Methods
    -- *******

    FinishingPhase = function(self)
        self:InterruptTimer("Infest");
        self:InterruptTimer("Reaper");
        self:InterruptTimer("Defile");

        if self:GetPhase() == 1 then
            self:InterruptTimer("Plague");
            self:InterruptTimer("Horror");

    elseif self:GetPhase() == 2 then
            self:InterruptTimer("Valkyr");
        end

        self:StartTimer("Quake");
        self:StartTimer("RagingSpirit");
    end,

    AdvancePhase = function(self)
        self:StartTimer("Defile");

        self:TriggerTimer("Quake");
        self:InterruptTimer("RagingSpirit");

        if self:GetPhase() == 1 then
            self:OnPhaseTwo();

    elseif self:GetPhase() == 2 then
            self:OnPhaseThree();
        end
    end,

    ScanDefile = function(self)
        local guid, name, uid = self:GetTargetInfo();
        if guid and uid then
            if self:GetSetting("defileSymbol") then
                self:PlaceSymbol(guid, 6, 5.00);
            end
            if self:IsMe(guid) then
                self:Alert(self:Localize("DefileSelf"));
                self:Say(self:Localize("DefileSay"));
          else
                if Distance:GetUnitDistance("player", uid) <= 15 then
                    self:Alert(self:Localize("DefileNear"));
                end
            end
            self:ShowZone("Defile", uid);
        end
    end,

    ScanValkyr = function(self)
        local list = { };
        local i, uid;
        for i=1, GetNumRaidMembers() do
            uid = "raid"..i;
            if UnitInVehicle(uid) then
                tinsert(list, UnitGUID(uid));
            end
        end
        if self:GetSetting("warnValkyr") then
            self:AnnounceUnitList(list, self:Localize("ValkyrAlert"), true, 18);
        end
    end,

    -- ****************
    -- Special handlers
    -- ****************

    -- Flow control

    OnIntro = function(self)
        self:PlaySilence();
        self:StartTimer("Start");
    end,

    OnPhaseOne = function(self, alreadyEngaged)
        if self:SetPhase(1) then
            -- Nothing to do.
        end
    end,

    OnPhaseTwo = function(self)
        if self:SetPhase(2) then
            self:ZeroTimeout();
            self:StartTimer("Infest", 13);
        end
    end,

    OnPhaseThree = function(self)
        if self:SetPhase(3) then
            self:ZeroTimeout();
        end
    end,

    OnPhaseFour = function(self)
        if self:SetPhase(4) then
            self:ZeroTimeout();

            -- Disallows interruption of the boss module
            self:DisableWipeCheck();
            self:DisableCombatCheck();

            -- Stop everything
            self:SuspendBerserk();
            self:InterruptAllTimers();
            self:InterruptAllZones();

            self:PlaySilence();
        end
    end,

    -- Handlers

    OnRemorselessWinter = function(self)
        self:FinishingPhase();
        self:Alert(self:Localize("RemorselessWinterAlert"));
    end,

    OnQuake = function(self)
        self:Alert(self:Localize("QuakeAlert"));
        self:AdvancePhase();
    end,

    OnPlagueStart = function(self, guid)
        if self:IsMe(guid) then
            self:Alert(self:Localize("PlagueSelf"));
        end
        if self:IsUnitAffiliated(guid) then
            if self:GetSetting("warnPlague") then
                self:AnnounceUnit(self:Localize("PlagueAlert"), guid, 7, 4.00);
            end
            self:ShowZone("Plague", guid);
        end
    end,

    OnPlagueEnd = function(self, guid)
        self:InterruptZone("Plague", guid);
    end,

    OnEnrage = function(self, name)
        if self:GetSetting("warnEnrage") then
            self:Announce(self:FormatLoc("EnrageAlert", name));
        end
    end,

    OnHarvestSoul = function(self, guid)
        -- self:RegisterHealAssist(guid, 0, 5);
        if self:GetSetting("warnHarvestSoul") then
            self:AnnounceUnit(self:Localize("HarvestSoulAlert"), guid, 7, 5.00);
        end
    end,

    OnSoulReaper = function(self, guid)
        if self:HasRole("SHIELD") then
            self:Alert(self:Localize("SoulReaperWarning"));
        end
    end,

    OnDefileDamage = function(self, guid)
        if self:IsMe(guid) then
            self:Alert(self:Localize("DefileTickSelf"));
        end
        self:ModifyZoneSize("Defile", 2); -- Try to keep track of the exact Defile size.
    end,
} );

-- --------------------------------------------------------------------
-- **                               Setup                            **
-- --------------------------------------------------------------------

M:DeclareBoss(36597, false, true); -- ID, volatile, bypass
M:DefineBerserk(900);

-- Events

M:RegisterCastEvent(72350, "Phase4");
M:RegisterCastEvent(72262, "Quake");
M:RegisterCastEvent(70498, "VileSpirits");
M:RegisterCastEvent(72762, "DefileCast");
M:RegisterCastEvent(69200, "RagingSpirit");
M:RegisterCastEvent(70372, "HorrorSpawn");
M:RegisterCastEvent(69037, "ValkyrSpawn");
M:RegisterCastEvent({68981, 72259, 74270, 74273}, "Winter");
M:RegisterCastEvent({70541, 73779, 73780, 73781}, "Infest");
M:RegisterCastEvent({68980, 74325, 74326, 74327}, "Harvest");
M:RegisterCastEvent({69409, 73797, 73798, 73799}, "SoulReaper");
M:RegisterDamageEvent({72754, 73708, 73709, 73710}, "DefileDamage");
M:RegisterEffectGainEvent({72143, 72146, 72147, 72148}, "Enrage");
M:RegisterCastEvent({70337, 70338, 73785, 73786, 73787, 73912, 73913, 73914}, "PlagueStart", "FIRST");
-- M:RegisterEffectGainEvent({70337, 70338, 73785, 73786, 73787, 73912, 73913, 73914}, "PlagueStart", "FIRST");
M:RegisterEffectFadeEvent({70337, 70338, 73785, 73786, 73787, 73912, 73913, 73914}, "PlagueEnd", "LAST");

-- Timers

M:DefineVoidTimer(      "Start",          "ooc",  53);
M:DefineVoidTimer( "FinalPhase",          "ooc", 160);
M:DefineVoidTimer("VileSpirits",     "all/high",  25);
M:DefineVoidTimer(      "Quake", "all/critical",  60);

M:DefineInitialRepeatingVoidTimer("Plague",    "all/high", 30, 31);
M:DefineInitialRepeatingVoidTimer("Infest", "heal/normal", 22,  8);

M:DefineRepeatingVoidTimer("RagingSpirit",   "all/normal", 23, 15);
M:DefineRepeatingVoidTimer(      "Horror",  "tank/normal", 60);
M:DefineRepeatingVoidTimer(      "Valkyr",     "all/high", 47, 24);
M:DefineRepeatingVoidTimer( "HarvestSoul",     "all/high", 75, 12);
M:DefineRepeatingVoidTimer(  "SoulReaper",  "shield/high", 30);
M:DefineRepeatingVoidTimer(      "Defile",     "all/high", 32, 30);

-- Zones

M:DefineAnchoredZone("Plague",     "all/high", "yellow", 10,   15,  true);
M:DefineAimedZone(   "Defile", "all/critical",    "red", 10, 31.5, false);

-- Actions

M:DeclareAction("Phase4");
M:AddCallbackToAction("Phase4", "OnPhaseFour");

M:DeclareAction("PlagueStart");
M:AddCallbackToAction("PlagueStart", "OnPlagueStart", "targetguid");

M:DeclareAction("PlagueEnd");
M:AddCallbackToAction("PlagueEnd", "OnPlagueEnd", "targetguid");

M:DeclareAction("Winter");
M:AddCallbackToAction("Winter", "OnRemorselessWinter");

M:DeclareAction("Infest");
M:AddTriggerTimerToAction("Infest", "Infest");

M:DeclareAction("Harvest");
M:AddCallbackToAction("Harvest", "OnHarvestSoul", "targetguid");

M:DeclareAction("SoulReaper");
M:AddCallbackToAction("SoulReaper", "OnSoulReaper", "targetguid");

M:DeclareAction("Quake");
M:AddCallbackToAction("Quake", "OnQuake");

M:DeclareAction("VileSpirits");
M:AddStartTimerToAction("VileSpirits", "VileSpirits");

M:DeclareAction("RagingSpirit");
M:AddTriggerTimerToAction("RagingSpirit", "RagingSpirit");

M:DeclareAction("HorrorSpawn");
M:AddTriggerTimerToAction("HorrorSpawn", "Horror");

M:DeclareAction("ValkyrSpawn", 5, 1);
M:AddTriggerTimerToAction("ValkyrSpawn", "Valkyr");
M:AddExternalActionToAction("ValkyrSpawn", "ValkyrDelay");

M:DeclareAction("ValkyrDelay", 0, 5); -- Triggered by ValkyrSpawn action.
M:AddCallbackToAction("ValkyrDelay", "ScanValkyr");

M:DeclareAction("Enrage", 1.5);
M:AddCallbackToAction("Enrage", "OnEnrage", "targetname");

M:DeclareAction("DefileCast", 3, 0.5); -- Slightly delayed to let the targetting take place.
M:AddCallbackToAction("DefileCast", "ScanDefile");

M:DeclareAction("DefileDamage", 2.0);
M:AddCallbackToAction("DefileDamage", "OnDefileDamage", "targetguid");