local Root = BossEncounter3;

local Module = Root:GetOrNewModule("Module");

-- --------------------------------------------------------------------
-- **                              API                               **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Get infos about the currently running boss module.               *
-- * This API is designed to be easily useable by external AddOns.    *
-- * If a boss module is not running:                                 *
-- * Return nil                                                       *
-- * If a boss module is running:                                     *
-- * Return encounterTitle, bossGUID, healthPct,                      *
-- *        encounterTime, aliveMembers, startingMembers.             *
-- * Some returns can be nil, so be careful.                          *
-- ********************************************************************
function BossEncounter3_GetEncounterInfo()
    local M = Module:GetActive();
    if M then
        if not M:IsTesting() then
            local encounterTitle = M:GetTitle();
            local bossGUID = nil;
            local healthPct = nil;
            local encounterTime = M:GetEngageTime();
            local aliveMembers, startingMembers = M:GetRaidAlive();
            if M:HasBoss() then
                bossGUID = M:GetBossGUID();
                _, _, healthPct = M:GetBossHealth();
                healthPct = healthPct * 100;
            end
            return encounterTitle, bossGUID, healthPct, encounterTime, aliveMembers, startingMembers;
        end
    end
    return nil;
end