﻿local Root = BossEncounter3;
local System = Root:GetOrNewModule("System");

local L = Root:GetOrNewModule("Localization");
local Unit = Root:GetOrNewModule("Unit");
local Social = Root:GetOrNewModule("Social");
local Utility = Root:GetOrNewModule("Utility");

local Settings = System:GetOrNewModule("Settings");

local AutoReply = System:GetOrNewModule("AutoReply");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Service automatically answering people during a boss fight.

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

AutoReply.cooldown = { };

local AUTOREPLY_COOLDOWN = 60;

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> name: the name of the guy checked.                            *
-- ********************************************************************
-- * Determinate if a guy can be whispered. Prevent spamming.         *
-- ********************************************************************
function AutoReply:CanWhisper(name)
    if Settings:Read("EnableAutoReply") then
        local cooldown = self.cooldown[name];
        if (not cooldown) or (GetTime() > cooldown) then
            return true;
        end
    end
    return false;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> name: the name of the target.                                 *
-- ********************************************************************
-- * Send an auto-reply explaining the state of the boss fight.       *
-- ********************************************************************
function AutoReply:Submit(name)
    if self:CanWhisper(name) then
        local encounterTitle, bossGUID, bossHpPercent, encounterTime, aliveMembers, startingMembers = BossEncounter3_GetEncounterInfo();

        if encounterTitle and (encounterTime or 0) > 0 then
            local header, body, tail;
            local clock = Utility:FormatCountdown("%M'%S", encounterTime, true, true, false);
            local me = UnitName("player");

            header = L:FormatLoc("AutoReply-Header", me, encounterTitle);
            if bossHpPercent then
                body = L:FormatLoc("AutoReply-Body", floor(bossHpPercent + 0.5).."%", clock);
          else
                body = L:FormatLoc("AutoReply-Body", "??", clock);
            end
            if aliveMembers and startingMembers then
                tail = L:FormatLoc("AutoReply-Tail", aliveMembers.."/"..startingMembers);
        elseif aliveMembers then
                tail = L:FormatLoc("AutoReply-Tail", aliveMembers);
          else
                tail = L:FormatLoc("AutoReply-Tail", "??");
            end

            Social:Whisper(header, name, true);
            Social:Whisper(body, name, false);
            Social:Whisper(tail, name, false);

            -- Put a cooldown to avoid spam.
            self.cooldown[name] = GetTime() + AUTOREPLY_COOLDOWN;
        end
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

AutoReply.OnWhisper = function(self, ...)
    local _, name, _, _, _, flag = ...;
    if flag ~= 'GM' then
        local uid = Unit:GetUID(name);
        if not uid then
            -- Ok, this guy is outside of our environment.
            self:Submit(name);
        end
    end
end

AutoReply.OnBNWhisper = function(self, ...)
    local _, name, _, _, _, _, _, _, _, _, _, _, id = ...;
    self:Submit("BN:"..id);
end
