﻿local Root = BossEncounter3;
local System = Root:GetOrNewModule("System");

local L = Root:GetOrNewModule("Localization");
local Save = Root:GetOrNewModule("Save");
local Utility = Root:GetOrNewModule("Utility");
local Network = Root:GetOrNewModule("Network");

local View = System:GetOrNewModule("View");
local Stats = System:GetOrNewModule("Stats");
local Settings = System:GetOrNewModule("Settings");
local VersionQuery = System:GetOrNewModule("VersionQuery");
local Capture = System:GetOrNewModule("Capture");
local Error = System:GetOrNewModule("Error");
local EquipChecker = System:GetOrNewModule("EquipChecker");

local Console = System:GetOrNewModule("Console");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Setup mod's slash commands.

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

local inputPortions = { };

-- /!\ Do not set "call" fields to directly a function / method. 
-- Sets it to a function or method name instead, as when this code will be executed, functions / methods themselves won't be defined yet.

Console.data = {
    unknownHelp = {
        ["enUS"] = "/BE command\nCommands available: %s",
        ["frFR"] = "/BE commande\nCommandes disponibles : %s",
    },

    {
        object = System,
        call = "StopActiveModule",
        arguments = { },
        input = {
            ["enUS"] = "clear",
            ["frFR"] = "nettoyer",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = Console,
        call = "ShowAuthorInfo",
        arguments = { },
        input = {
            ["enUS"] = "author",
            ["frFR"] = "auteur",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = Console,
        call = "WipeAllData",
        arguments = { },
        input = {
            ["enUS"] = "reset",
            ["frFR"] = "raz",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = View,
        call = "OpenOptions",
        arguments = { },
        input = {
            ["enUS"] = "options",
            ["frFR"] = "options",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = Console,
        call = "QueryVersion",
        arguments = {"string"},
        input = {
            ["enUS"] = "version",
            ["frFR"] = "version",
        },
        invalidHelp = {
            ["enUS"] = "/BE %s guild/group/player name",
            ["frFR"] = "/BE %s guilde/groupe/nom de joueur",
        },
    },
    {
        object = View,
        call = "ReplayLastSequence",
        arguments = { },
        input = {
            ["enUS"] = "replay",
            ["frFR"] = "rejouer",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = Console,
        call = "PrintLastDps",
        arguments = {"string"},
        input = {
            ["enUS"] = "printdps",
            ["frFR"] = "afficherdps",
        },
        invalidHelp = {
            ["enUS"] = "/BE %s mode/player name\nMode specifies where the result is displayed: 1=Self, 2=Group, 3=Guild, 4=Officer.\nYou can also whisper by providing a player name instead of a digit.",
            ["frFR"] = "/BE %s mode/nom de joueur\nMode indique où afficher le résultat: 1=Soi-même, 2=Groupe, 3=Guilde, 4=Officier.\nVous pouvez aussi chuchoter en donnant un nom de joueur au lieu d'un chiffre.",
        },
    },
    {
        object = Console,
        call = "PrintErrors",
        arguments = {"string"},
        input = {
            ["enUS"] = "printerrors",
            ["frFR"] = "affichererreurs",
        },
        invalidHelp = {
            ["enUS"] = "/BE %s mode/player name\nMode specifies where the result is displayed: 1=Self, 2=Group, 3=Guild, 4=Officer.\nYou can also whisper by providing a player name instead of a digit.",
            ["frFR"] = "/BE %s mode/nom de joueur\nMode indique où afficher le résultat: 1=Soi-même, 2=Groupe, 3=Guilde, 4=Officier.\nVous pouvez aussi chuchoter en donnant un nom de joueur au lieu d'un chiffre.",
        },
    },
    {
        object = Console,
        call = "SetDifficulty",
        arguments = {"number"},
        input = {
            ["enUS"] = "difficulty",
            ["frFR"] = "difficulte",
        },
        invalidHelp = {
            ["enUS"] = "/BE %s n, n is a number between 1-5",
            ["frFR"] = "/BE %s n, n est un nombre entre 1-5",
        },
    },
    {
        object = View,
        call = "ToggleRadar",
        arguments = { },
        input = {
            ["enUS"] = "radar",
            ["frFR"] = "radar",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = View,
        call = "CheckHealRange",
        arguments = { },
        input = {
            ["enUS"] = "healrange",
            ["frFR"] = "porteesoins",
        },
        invalidHelp = "/BE %s",
    },
    {
        object = View,
        call = "Test",
        arguments = { },
        input = {
            ["enUS"] = "test",
            ["frFR"] = "test",
        },
        invalidHelp = "/BE %s",
    },
--[[
    {
        object = EquipChecker,
        call = "AuditAll",
        arguments = { },
        input = {
            ["enUS"] = "checkequipment",
            ["frFR"] = "verifierequipement",
        },
        invalidHelp = "/BE %s",
    },
]]
    {
        object = View,
        call = "OpenMusicManager",
        arguments = { },
        input = {
            ["enUS"] = "music",
            ["frFR"] = "musique",
        },
        invalidHelp = "/BE %s",
    },
};

local debugCmd = {
    input = {
        ["enUS"] = "debug",
        ["frFR"] = "debug",
    },

    unknownHelp = {
        ["enUS"] = "/BE debug (see below)\n%s",
        ["frFR"] = "/BE debug (voir plus bas)\n%s",
    },

    {
        object = Console,
        call = "ShowNpcId",
        arguments = { },
        input = "npcid",
        invalidHelp = "/BE debug %s",
    },
    {
        object = Console,
        call = "ShowSpellId",
        arguments = {"string"},
        input = "spellid",
        invalidHelp = "/BE debug %s [spellLink]",
    },
    {
        object = Console,
        call = "ShowCaptureSample",
        arguments = {"string", "number"},
        input = "showcapture",
        invalidHelp = "/BE debug %s encounterID sampleNumber",
    },
    {
        object = Console,
        call = "ClearAllCaptures",
        arguments = { },
        input = "clearallcaptures",
        invalidHelp = "/BE debug %s",
    },
};

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> table: table in which arguments will be pushed.               *
-- * >> string: the full command line.                                *
-- ********************************************************************
-- * Split an argument string into individual parameters.             *
-- * It would be trivial to do if [] could not be used to put spaces  *
-- * in arguments themselves, though space is the separator.          *
-- ********************************************************************
function Console:SplitAndPopulate(table, string)
    string = strtrim(string);

    local len = #string;
    local bracketLevel = 0;
    local p = 0;
    local c = "";

    if len == 0 then return; end
    while p < len do
        p = p + 1;
        c = string:sub(p, p);
        
        if c == "[" then
            bracketLevel = bracketLevel + 1;
        end
        if c == "]" then
            bracketLevel = bracketLevel - 1;
        end
        if c == " " and bracketLevel <= 0 then
            p = p - 1;
            break;
        end
    end
    tinsert(table, string:sub(1, p));
    if (p+2) <= len then
        self:SplitAndPopulate(table, string:sub(p+2, len));
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> node: the node being checked against.                         *
-- * >> string: the command provided.                                 *
-- ********************************************************************
-- * Test if a given string matches a node's input string.            *
-- ********************************************************************
function Console:IsMatching(node, string)
    if type(string) ~= "string" then return false; end
    string = strlower(string);
    
    local input = node.input;
    if type(input) == "table" then
        for k, v in pairs(input) do
            if strlower(v) == string then
                return true;
            end
        end
  elseif type(input) == "string" then
        if strlower(input) == string then
            return true;
        end
    end
    return false;
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> node: the node being finalized.                               *
-- ********************************************************************
-- * Finalize a slash command node before it can be used.             *
-- ********************************************************************
function Console:FinalizeNode(node)
    local locale = L:GetLocale();
    
    if node.call then
        local invalidHelp = L:ReadTable(node.invalidHelp);
        local inputString = L:ReadTable(node.input) or "?";
        if type(node.invalidHelp) == "table" then
            node.invalidHelp[locale] = (invalidHelp):format(inputString);
    elseif type(node.invalidHelp) == "string" then
            node.invalidHelp = (invalidHelp):format(inputString);
        end
        
elseif #node > 0 then
        local commandList = "";
        local i;
        for i=1, #node do
            local inputString = L:ReadTable(node[i].input);
            if inputString then
                if #commandList > 0 then
                    commandList = commandList.." | ";
                end
                commandList = commandList..inputString;
            end
            self:FinalizeNode(node[i]);
        end
        
        local unknownHelp = L:ReadTable(node.unknownHelp);
        if type(node.unknownHelp) == "table" then
            node.unknownHelp[locale] = (unknownHelp):format(commandList);
    elseif type(node.unknownHelp) == "string" then
            node.unknownHelp = (unknownHelp):format(commandList);
        end
    end
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> tree: the command tree being finalized.                       *
-- ********************************************************************
-- * Finalize a slash command tree before it can be used.             *
-- ********************************************************************
function Console:Finalize(tree)
    self:FinalizeNode(tree);
end

-- ********************************************************************
-- * Arguments:                                                       *
-- * >> input: what we asked the console to do.                       *
-- * >> feedback: if set, the console will print feedback             *
-- * in case of unknown commands, bad syntaxes etc..                  *
-- ********************************************************************
-- * Ask the console to perform an action.                            *
-- ********************************************************************
function Console:Execute(input, feedback)
    -- STEP 1 - Decomposes each word of the input. Also take care of item links.

    -- Split the input string.
    wipe(inputPortions);
    self:SplitAndPopulate(inputPortions, input);
    local portions = #inputPortions;

    -- STEP 2 - Grab the appropriate console table.

    local data = self.data;
    if (not data) then
        -- Uh ? No console data at all !
        return;
    end
    
    -- STEP 3 - Enter the loop ! Oh yeah baby enter the loop =))

    local unknownHelp;
    local invalidHelp;
    local level = 0;
    local levelData;
    local k, node;
    local continue = true;
    local isValid; -- true if the command exists. If isValid is false and feedback set, a message saying you entered an unknown command will fire.
    local hasSyntaxError = false; -- true if the command was a real command and the user passed missing/invalid arguments to it.
    local functionMissing = false; -- true if command was provided good arguments, but the function itself to call did not exist !

    while continue do
        unknownHelp = nil;
        invalidHelp = nil;

        if level == 0 then
            -- Root level dude.
            unknownHelp = L:ReadTable(data.unknownHelp);
            levelData = data;
      else
            -- We're somewhere deep in the tree.
            unknownHelp = L:ReadTable(levelData.unknownHelp);
        end

        isValid = false;
        continue = false;

        for k, node in ipairs(levelData) do
            if self:IsMatching(node, inputPortions[level+1]) then
                -- This command matches.
                isValid = true;

                if node.call then
                    local a;
                    local providedArgument = nil;
                    local awaiting = false;

                    continue = false;

                    -- It has initially a good syntax..
                    hasSyntaxError = false;
                    invalidHelp = L:ReadTable(node.invalidHelp);

                    for a=1, #node.arguments do
                        awaiting = node.arguments[a];
                        providedArgument = inputPortions[level+1+a];
                        
                        if awaiting == "number" then
                            providedArgument = tonumber(providedArgument);
                            if providedArgument then
                                inputPortions[level+1+a] = floor(providedArgument);
                          else
                                hasSyntaxError = true;
                            end
                        end
                        if awaiting == "string" or awaiting == "bracketstring" then
                            providedArgument = providedArgument or '';
                            if #providedArgument <= 0 then
                                hasSyntaxError = true;
                            end
                            if awaiting == "bracketstring" then
                                local _, _, content = (providedArgument):find("%[(.+)%]");
                                if content then
                                    inputPortions[level+1+a] = content;
                              else
                                    hasSyntaxError = true;
                                end
                            end
                        end
                    end

                    -- The console system is strict; you can't pass more arguments than the function expects.
                    if (portions-level-1) > #node.arguments then
                        hasSyntaxError = true;
                    end

                    -- Ok, seems like it's safe to pass arguments to the func without making the system crashes :P
                    if not hasSyntaxError then
                        if not node.object then
                            local funcToCall = _G[node.call];
                            if type(funcToCall) == "function" then
                                funcToCall(inputPortions[level+2], inputPortions[level+3],inputPortions[level+4],inputPortions[level+5], inputPortions[level+6]);
                          else
                                functionMissing = true;
                            end
                      else
                            local funcToCall = node.object[node.call];
                            if type(funcToCall) == "function" then
                                funcToCall(node.object, inputPortions[level+2], inputPortions[level+3], inputPortions[level+4], inputPortions[level+5], inputPortions[level+6]);
                          else
                                functionMissing = true;
                            end
                        end
                    end

            elseif #node > 0 then
                    continue = true;
                    levelData = levelData[k];  -- Progress in the tree.
                    level = level + 1;
              else
                    -- Unknown type. Ignore & exit loop.
                    continue = false;
                end

                break;
            end
        end

        if not isValid then
            continue = false;
        end
    end

    -- STEP 4 - If enabled, tells the user anything that could have gone wrong.

    if feedback then
        local errorMessage = nil;
        if (not isValid) and  unknownHelp then
            errorMessage = L:Get("Console-Unknown") .. unknownHelp;

    elseif hasSyntaxError and invalidHelp then
            errorMessage = L:Get("Console-BadSyntax") .. invalidHelp;

    elseif functionMissing then
            errorMessage = L:Get("Console-Broken");
        end
        if errorMessage then
            Root:Print(errorMessage);
        end
    end
end

-- --------------------------------------------------------------------
-- **                          Implementation                        **
-- --------------------------------------------------------------------

local modeTable = {
    ["1"] = "SELF",
    ["2"] = "GROUP",
    ["3"] = "GUILD",
    ["4"] = "OFFICER",
};

function Console:ShowAuthorInfo()
    Root:Print(L:Get("Console-Author"));
end

function Console:ShowNpcId()
    local guid = UnitGUID("target");
    if guid then
        local Unit = Root:GetOrNewModule("Unit");
        id = Unit:GetMobID(guid) or '?';
        Root:Print("NPC-ID: "..id);
    end
end

function Console:ShowSpellId(spellLink)
    local spellId = Utility:ExtractSpellID(spellLink);
    if spellId then
        Root:Print("Spell-ID: "..spellId);
  else
        Root:Print("No spell link found in your string.");
    end
end

function Console:WipeAllData(confirmed)
    if confirmed then
        Save:Clear();
        ReloadUI();
  else
        StaticPopup_Show("BE3_WIPE_ALL_DATA");
    end
end

function Console:QueryVersion(recipient)
    local upper = string.upper(recipient);
    local guild = L:Get("Version-Guild");
    local group = L:Get("Version-Group");

    if upper == guild then
        recipient = "GUILD";
elseif upper == group then
        recipient = "GROUP";
  else
        -- It is probably a player name.
    end

    local okay = VersionQuery:Query(recipient);
    if okay then
        Root:Print(L:Get("Version-QueryStarted"));
  else
        Root:Print(L:Get("Version-QueryForbidden"));
    end
end

function Console:PrintLastDps(mode)
    local name, _, _, _, _, _, _, _, _, _, dps = Stats:GetCurrentInfo();
    if name and dps then
        if modeTable[mode] then
            mode = modeTable[mode];
        end
        Stats:PrintDpsToChat(dps, mode or "SELF");
  else
        Root:Print(L:Get("NoDpsData"));
    end
end

function Console:PrintErrors(mode)
    if modeTable[mode] then
        mode = modeTable[mode];
    end
    Error:PrintToChat(mode);
end

function Console:SetDifficulty(difficulty)
    local name = Settings:GetDifficultyName(difficulty);
    if name then
        Settings:SetDifficulty(difficulty);
        Root:Print(L:FormatLoc("DifficultySet", name, difficulty));
  else
        Root:Print(L:Get("InvalidDifficulty"));
    end
end

function Console:ShowCaptureSample(id, sample)
    local rid = strlower(id);
    rid = rid:gsub("||", "|");
    if not Capture:Show(rid, sample) then
        Root:Print(L:FormatLoc("CaptureSample", Capture:GetNumSamples(rid), id));
    end
end

function Console:ClearAllCaptures()
    Capture:Clear();
    Root:Print("Capture samples have been cleared.");
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function Console:OnBootComplete()
    -- Register debug commands for developpers
    local pName = UnitName("player");
    if Network:IsAdmin(pName) then
        tinsert(self.data, debugCmd);
    end

    -- Finalize the command tree
    self:Finalize(self.data);
    
    -- Register slash commands
    SlashCmdList["BOSSENCOUNTER3"] = function(msg) Console:OnSlashCommand(msg); end;
    SLASH_BOSSENCOUNTER31 = "/be3";
    SLASH_BOSSENCOUNTER32 = "/bossencounter3";
    SLASH_BOSSENCOUNTER33 = "/be"; -- Be careful with BE2 conflict.

    -- Register popups
    StaticPopupDialogs["BE3_WIPE_ALL_DATA"] = {
	text = L:Get("Console-WipeAllData"),
	button1 = YES,
	button2 = NO,
	timeout = 0,
	showAlert = true,
	whileDead = true,
	OnAccept = function(self) Console:WipeAllData(true); end,
    };
end

function Console:OnSlashCommand(msg)
    self:Execute(msg, true);
end
