﻿local Root = BossEncounter3;
local System = Root:GetOrNewModule("System");

local L = Root:GetOrNewModule("Localization");
local Save = Root:GetOrNewModule("Save");
local Unit = Root:GetOrNewModule("Unit");
local Network = Root:GetOrNewModule("Network");
local Authorization = Root:GetOrNewModule("Authorization");

local SettingPriority = System:GetOrNewModule("SettingPriority");

local Settings = System:GetOrNewModule("Settings");

-- --------------------------------------------------------------------
-- **                           Description                          **
-- --------------------------------------------------------------------

-- Provides an interface to modules to manipulate settings.
-- Also provides core, module-independant global settings.

-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

Settings.loaded = false;

local mySettings = {
    {
        id = "EnableErrorTool",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Enable error detection tool",
            ["frFR"] = "Activer l'outil de détection des erreurs",
        },
        explain = {
            ["enUS"] = "This system monitors registers errors done by players during a raid encounter such as staying in fires.\n\nType /BE3 printerrors 1 to check them privately.\n\nThis system is still under development and its performances vary from boss to boss.",
            ["frFR"] = "Ce système relève les erreurs faites par les joueurs durant un boss de raid telles que rester dans le feu.\n\nTapez /BE3 affichererreurs 1 pour les consulter discrètement.\n\nCe système est encore en développement et ses performances varient selon les boss.",
        },
    },

    {
        id = "EnableAutoReply",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Enable auto-reply in combat",
            ["frFR"] = "Activer réponses auto en combat",
        },
        explain = {
            ["enUS"] = "When enabled, this service will automatically answer whispers from people outside from your party. They will be informed of the status of the fight.",
            ["frFR"] = "Quand activé, ce service répondra automatiquement aux chuchotements des personnes extérieures à votre groupe. Elles seront informées de l'état du combat.",
        },
    },

    {
        id = "EnableBossBar",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Enable boss health bar",
            ["frFR"] = "Activer la barre de vie du boss",
        },
        explain = {
            ["enUS"] = "Shows a dedicated health bar for the main boss.",
            ["frFR"] = "Affiche une barre de vie dédiée au boss principal.",
        },
    },

    {
        id = "EnableJingle",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Enable first attempt jingle",
            ["frFR"] = "Activer le jingle du premier essai",
        },
        explain = {
            ["enUS"] = "Allows a short jingle to be played on the first attempt of a boss.",
            ["frFR"] = "Permet de jouer un court jingle sur le premier essai d'un boss.",
        },
    },

    {
        id = "MaskEndSequence",
        type = "BOOLEAN",
        default = true,
        label = {
            ["enUS"] = "Mask end sequence for well known bosses",
            ["frFR"] = "Masquer la séquence de fin pour les boss bien connus",
        },
        explain = {
            ["enUS"] = "Skips the end sequence if the boss was defeated 3 times or more. If you get a new record the sequence will still be shown.",
            ["frFR"] = "Passe la séquence de fin si le boss a été vaincu 3 fois ou plus. Si vous obtenez un nouveau record, la séquence sera quand même affichée.",
        },
    },

    {
        id = "AutomaticRole",
        type = "BOOLEAN",
        perCharacter = true,
        default = true,
        label = {
            ["enUS"] = "Determinate my role automatically",
            ["frFR"] = "Déterminer mon rôle automatiquement",
        },
        explain = {
            ["enUS"] = "When enabled, BossEncounter3 will automatically determinate your role (melee, ranged, healer or tank). This saves you the burden of setting manually Filter yourself (see below).",
            ["frFR"] = "Quand activé, BossEncounter3 déterminera automatiquement votre rôle (mêlée, distance, soigneur ou tank). Cela vous évite de devoir régler manuellement le filtre (voir plus bas).",
        },
    },

    {
        id = "Difficulty",
        type = "DROPDOWN",
        perCharacter = true,

        min = 1,
        max = 5,
        default = 3,
        valueKey = "Difficulty", -- This will be suffixed by ":X" where X is the ID of the difficulty chosen.

        label = {
            ["enUS"] = "Difficulty",
            ["frFR"] = "Difficulté",
        },
        explain = {
            ["enUS"] = [[This dropdown menu allows you to choose the help brought by BossEncounter3.

Use top difficulties to get more indications during combats.

Use bottom difficulties to reduce or hide informations during combats.]],
            ["frFR"] = [[Ce menu déroulant vous permet de choisir l'aide que vous apporte BossEncounter3.

Utilisez les difficultés du haut pour obtenir davantage d'indications pendant les combats.

Utilisez les difficultés du bas pour réduire ou masquer les informations fournies par BE3.]],
        },
    },

    {
        id = "RoleFilter",
        type = "BIT_TABLE",
        perCharacter = true,

        bits = 4,
        default = 0x00,
        valueKey = "Role", -- This will be suffixed by ":X" where X is the ID of the role.

        label = {
            ["enUS"] = "Filter",
            ["frFR"] = "Filtre",
        },
        explain = {
            ["enUS"] = [[The filter allows you to hide or reduce importance of various alerts throughout boss fights according to your current role(s).

To get all alerts, check all boxes.

|cffff8000WARNING - This will have no effect if "Determinate my role automatically" is enabled.|r]],
            ["frFR"] = [[Ce filtre vous permet de cacher ou de réduire l'importance des différentes alertes durant les combats de boss en fonction de votre(vos) rôle(s) actuel(s).

Pour recevoir toutes les alertes, cochez toutes les cases.

|cffff8000ATTENTION - Ceci n'aura aucun effet si l'option "Déterminer mon rôle automatiquement" est cochée.|r]],
        },
    },

    version = 1,
};

local debugSettings = {
    {
        id = "DebugMode",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Debug mode",
            ["frFR"] = "Mode débug",
        },
        explain = {
            ["enUS"] = "Enables a few debug messages.\n\nNormal users should not enable this.",
            ["frFR"] = "Active quelques messages de débug.\n\nLes utilisateurs normaux ne devraient pas activer ceci.",
        },
    },

    {
        id = "EnableCapture",
        type = "BOOLEAN",
        default = false,
        label = {
            ["enUS"] = "Enable capture",
            ["frFR"] = "Activer la capture",
        },
        explain = {
            ["enUS"] = "Enables the capture system.\n\nIt logs abilities casts and yells during boss encounters.\n\nNormal users should not enable this.",
            ["frFR"] = "Active le système de capture.\n\nIl enregistre les capacités et les messages durant les boss.\n\nLes utilisateurs normaux ne devraient pas activer ceci.",
        },
    },
};

local roleIds = {
    -- Roles that are directly asked in the option box.
    ["DPS_MELEE"] = 0x01,
    ["DPS_RANGED"] = 0x02,
    ["HEALER"] = 0x04,
    ["TANK"] = 0x08,

    -- Roles that are deduced from the roles chosen in the option box.
    ["DPS"] = 0x03, -- ( melee dps ) + ( ranged dps )
    ["CASTER"] = 0x06, -- ( ranged dps ) + ( heal )
    ["MELEE"] = 0x09, -- ( melee dps ) + ( tank )
    ["SHIELD"] = 0x0C, -- ( heal ) + ( tank )
    ["OFFENSIVE"] = 0x0B, -- ( dps ) + ( tank )

    -- Special
    ["INTERRUPT"] = 0x80, -- Has a fairly good interrupt.
};

local rolePerClass = {
    -- Pure classes (all damage dealers, no ambiguity)

    ["HUNTER"] = 0x02,
    ["MAGE"] = 0x82,
    ["ROGUE"] = 0x81,
    ["WARLOCK"] = 0x02,

    -- Hybrid classes

    ["DEATHKNIGHT"] = {
        [1] = 0x08, -- BLOD - Tank
        [2] = 0x01, -- FRST - Melee
        [3] = 0x01, -- UNHY - Melee
    },

    ["DRUID"] = {
        [1] = 0x02, -- BLNC - Ranged
        [2] = 0x81, -- FRAL - Melee
		[3] = 0x88, -- GURD - Tank
        [4] = 0x04, -- REST - Healer
    },

    ["PALADIN"] = {
        [1] = 0x04, -- HOLY - Healer
        [2] = 0x88, -- PROT - Tank
        [3] = 0x81, -- RETB - Melee
    },

    ["PRIEST"] = {
        [1] = 0x04, -- DISC - Healer
        [2] = 0x04, -- HOLY - Healer
        [3] = 0x02, -- SHDW - Ranged
    },

    ["SHAMAN"] = {
        [1] = 0x82, -- ELEM - Ranged
        [2] = 0x81, -- ENHC - Melee
        [3] = 0x84, -- REST - Healer
    },

    ["WARRIOR"] = {
        [1] = 0x81, -- ARMS - Melee
        [2] = 0x81, -- FURY - Melee
        [3] = 0x88, -- PROT - Tank
    },

	-- TODO: MONK
};

local moduleSettings = {
    -- Will be filled by modules.
};

local myLookup = { };
local moduleLookup = { };

local myDB = { };
local moduleDB = { };

-- --------------------------------------------------------------------
-- **                             Methods                            **
-- --------------------------------------------------------------------

function Settings:IsReady()
    return self.loaded;
end

function Settings:GetSection(id) -- Return GLOBAL, MODULE or nil.
    if myLookup[id] then
        return "GLOBAL";

elseif moduleLookup[id] then
        return "MODULE";
    end
    return nil;
end

function Settings:GetKey(id)
    if self:IsPerCharacter(id) then
        return id..":"..UnitGUID("player");
  else
        return id;
    end
end

function Settings:GetInfo(id) -- Return type, default, perCharacter, action, exclusive, label, explanation
    local info = nil;
    if myLookup[id] then
        info = myLookup[id];

elseif moduleLookup[id] then
        info = moduleLookup[id];
    end
    if info then
        return info.type, info.default, info.perCharacter, info.action, info.exclusive, L:ReadTable(info.label) or "?", L:ReadTable(info.explain) or "";
  else
        return nil, nil, false, nil, false, nil, nil;
    end
end

function Settings:GetExtraInfo(id) -- Return min, max, step, bits, valueKey. Note: only relevant returns won't be nil.
    local info = nil;
    if myLookup[id] then
        info = myLookup[id];

elseif moduleLookup[id] then
        info = moduleLookup[id];
    end
    if info then
        return info.min, info.max, info.step, info.bits or 0, info.valueKey;
  else
        return nil, nil, nil, 0, nil;
    end
end

function Settings:Exists(id) -- Check if the setting with the specified ID exists.
    return (self:GetSection(id) ~= nil);
end

function Settings:IsExclusive(id) -- Check if the setting is exclusive (an exclusive setting state is determinated by only 1 people in the raid for the whole raid).
    local _, _, _, _, exclusive = self:GetInfo(id);
    return exclusive;
end

function Settings:IsPerCharacter(id) -- Check if the setting is saved per character.
    local _, _, perCharacter = self:GetInfo(id);
    return perCharacter;
end

function Settings:HasPermission(id, unit) -- Check if given unit has the authorizations required to make use of the setting. Unit defaults to "player".
    local _, _, _, action = self:GetInfo(id);
    if action then
        return Authorization:Check(action, unit);
  else
        return true;
    end
end

function Settings:Read(id) -- Read the value of a setting, taking restrictions in account.
    local type, _, _, action, exclusive = self:GetInfo(id);
    if exclusive then
        local decider, value = SettingPriority:Resolve(id);
        if (action == "RAID_WARNING" or action == "SYMBOL") and decider ~= UnitName("player") and type ~= "STRING" then
            return false, decider; -- Not only alert options are exclusive, they're also limited to one person (otherwise there would be flood!).
      else
            return value, decider;
        end
  else
        if not self:HasPermission(id) then
            return false, nil;
      else
            return self:ReadReal(id), nil;
        end
    end
end

function Settings:ReadReal(id) -- Read the value of a setting, ignoring restrictions.
    local section = self:GetSection(id);
    if section then
        local key = self:GetKey(id);
        if section == "GLOBAL" then
            return myDB[key];

    elseif section == "MODULE" then
            return moduleDB[key];
        end
    end
    return nil;
end

function Settings:Write(id, value) -- Write a new value for a setting.
    local section = self:GetSection(id);
    if section then
        local key = self:GetKey(id);
        if section == "GLOBAL" then
            myDB[key] = value;

    elseif section == "MODULE" then
            moduleDB[key] = value;
        end
        if id == "Difficulty" then
            Root:InvokeHandler("OnDifficultyUpdate", value);
      else
            Root:InvokeHandler("OnSettingUpdate", id, value);
        end
    end
end

function Settings:Initialize() -- Initialize settings.
    -- Build lookup
    local i;
    for i=1, #mySettings do
        myLookup[mySettings[i].id] = mySettings[i];
    end

    myDB = Save:Get("system", "settings") or { };
    Save:Set("system", "settings", myDB);

    self:PrepareDB(myDB, mySettings);
end

function Settings:GetNumGlobal() -- Get the number of global settings.
    return #mySettings;
end

function Settings:GetGlobal(index) -- Get the ID of the n-th global setting.
    return mySettings[index].id;
end

function Settings:GetNumSpecific() -- Get the number of specific settings (of the active module).
    return #moduleSettings;
end

function Settings:GetSpecific(index) -- Get the ID of the n-th specific setting.
    return moduleSettings[index].id;
end

function Settings:GetNum() -- Get the number of global and specific settings.
    return self:GetNumGlobal() + self:GetNumSpecific();
end

function Settings:Get(index) -- Get the ID of the n-th setting.
    local numGlobal = self:GetNumGlobal();
    if index > numGlobal then
        return self:GetSpecific(index - numGlobal);
  else
        return self:GetGlobal(index);
    end
end

function Settings:AccessModuleSettings(settings, namespace) -- Add a module's settings to settings. This remains as long as the module is the active module.
    wipe(moduleSettings);
    wipe(moduleLookup);

    -- Build lookup
    local i;
    for i=1, #settings do
        moduleSettings[i] = settings[i];
        moduleLookup[settings[i].id] = settings[i];
    end

    moduleDB = Save:Get("module", namespace) or { };
    Save:Set("module", namespace, moduleDB);

    self:PrepareDB(moduleDB, settings);

    Root:InvokeHandler("OnSettingsRefresh");
end

function Settings:ClearModuleSettings() -- Remove the current module settings. This is called as soon as the module is no longer the active module.
    wipe(moduleSettings);
    wipe(moduleLookup);

    moduleDB = { };

    Root:InvokeHandler("OnSettingsRefresh");
end

function Settings:HasRole(role) -- Check if the player has the specified role.
    local myRoleBitTable = self:Read("RoleFilter") or 0;
    local realRoleBitTable = self:GetRealRole() or 0;

    if role == "INTERRUPT" then
        myRoleBitTable = realRoleBitTable;
  else
        if self:Read("AutomaticRole") then
            myRoleBitTable = bit.band(realRoleBitTable, self:GetContextRoleMask());
        end
    end

    local matchBitTable = roleIds[role];
    if matchBitTable then
        return bit.band(myRoleBitTable, matchBitTable) ~= 0;
  else
        return true;
    end
end

function Settings:GetRealRole() -- Get the real role of the player according to the talent spec and class.
    local _, class = UnitClass("player");
    local info = rolePerClass[class];

    if not info then
        return 0x00;

elseif type(info) == "number" then
        return info; -- No possible ambiguity

elseif type(info) == "table" then
        local activeSpec = GetActiveSpecGroup();
        local primaryTree = GetSpecialization(false, false, activeSpec);
        if primaryTree then
            return info[primaryTree];
      else
            return 0x00;
        end
    end
end

function Settings:GetContextRoleMask() -- This allows to fine-tune the role for specs with ambiguous roles (Feral in particular).
    local role = Unit:GetRole("player");
    local mask = 0xFF;
    if role == "DAMAGER" then
        mask = roleIds["DPS"];

elseif role == "TANK" then
        mask = roleIds["TANK"];

elseif role == "HEALER" then
        mask = roleIds["HEALER"];
    end
    return mask;
end

function Settings:GetRange() -- Return MELEE, RANGED or HEALER according to the talent spec of the player. You'll need extra code to distinguish a damager dealer from a tank.
    local realRoleBitTable = self:GetRealRole() or 0;
    if bit.band(realRoleBitTable, roleIds["MELEE"]) > 0 then
        return "MELEE";

elseif bit.band(realRoleBitTable, roleIds["HEALER"]) > 0 then
        return "HEALER";
    end
    return "RANGED";
end

function Settings:HasInterrupt() -- Check if the player has a decent interrupt.
    return self:HasRole("INTERRUPT");
end

function Settings:GetDifficulty() -- Get the difficulty index chosen by the user.
    return max(1, min(self:ReadReal("Difficulty"), 5));
end

function Settings:GetDifficultyName(difficulty) -- Get the localized name of a difficulty level.
    difficulty = difficulty or self:GetDifficulty();
    if difficulty >= 1 and difficulty <= 5 then
        return L:Get("Difficulty:"..difficulty);
  else
        return nil;
    end
end

function Settings:SetDifficulty(difficulty)
    if difficulty >= 1 and difficulty <= 5 then
        Settings:Write("Difficulty", difficulty);
    end
end

function Settings:PrepareDB(db, settings)
    local guid = UnitGUID("player");
    local replace = false;

    if db.version ~= settings.version then
        replace = true;
        wipe(db);
        db.version = settings.version;
    end

    local i, setting, key;
    for i=1, #settings do
        setting = settings[i];
        if setting.perCharacter then
            key = setting.id..":"..guid;
      else
            key = setting.id;
        end
        if replace or db[key] == nil then
            db[key] = setting.default;
        end
    end
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

Settings.OnBootCompleteImportant = function(self)
    if not self:IsReady() then
        -- Register debug settings for developpers
        local pName = UnitName("player");
        if Network:IsAdmin(pName) then
            local i;
            for i=1, #debugSettings do
                tinsert(mySettings, i, debugSettings[i]);
            end
        end

        -- Prepare and load saved settings
        self:Initialize();

        self.loaded = true;
        Root:InvokeHandler("OnSettingsReady");
    end
end

Settings.OnSettingUpdate = function(self, id, value)
    if id == "DebugMode" then
        self:OnDebugUpdate();
    end
end

Settings.OnSettingsReady = function(self)
    self:OnDebugUpdate();
end

Settings.OnDebugUpdate = function(self)
    local state = self:Read("DebugMode");
    if state then
        Root:EnableDebug(true);
  else
        Root:DisableDebug(true);
    end
end
